#!/usr/bin/python

# MailHeaderTagger.py
#       --copyright--                   Copyright 2007 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       August 25, 2005         bar
#       August 27, 2005         bar     changed name
#       August 29, 2005         bar     process Mozilla mail filter data file
#       September 24, 2005      bar     intab some code
#       June 25, 2006           bar     test something
#       June 30, 2006           bar     allow multiline body regx matches (using DOTALL if the regx has a \n in it)
#       July 24, 2006           bar     play with "to or cc" and toORcc and recipient, none of which work !!!!
#       July 31, 2006           bar     DOTALL for body tests
#       August 1, 2006          bar     qpbody
#       September 1, 2006       bar     return tags' array
#                                       handle mailbox file
#                                       option to print tags only
#       September 8, 2006       bar     get toORcc working
#       October 19, 2006        bar     get rid of a couple of unused re's
#       October 28, 2006        bar     remove unused routine
#       December 7, 2006        bar     allow body, qbody and toORcc to be case-insensitive
#       December 21, 2006       bar     take the .tmp file write out
#       November 18, 2007       bar     turn on doxygen
#       November 27, 2007       bar     insert boilerplate copyright
#       December 23, 2007       bar     put in header with filter line number
#       December 24, 2007       bar     update the pickled filters if we are newer than they
#       December 25, 2007       bar     take the danged debug print statement out
#       May 17, 2008            bar     email adr
#       August 29, 2008         bar     basestring instead of StringType because of unicode strings and others
#       August 5, 2010          bar     text_body
#       May 27, 2012            bar     doxygen namespace
#       December 31, 2012       bar     return value when bad is bogus
#       December 2, 2019        bar     toorcc now includes received:
#       May 12, 2021            bar     shbang python2
#       February 23, 2023       bar     get rid of has_keys
#       March 1, 2023           bar     python3
#       March 2, 2023           bar     use tzlib for pickleing
#       March 7, 2023           bar     be a bit more careful "printing" to output
#       --eodstamps--
##      \file
#       \namespace              tzpython.MailHeaderTagger
#
#
#       stdin/stdout email (procmail) filter to add special header to the mail.
#       The header contains the folder to which the mail should be routed.
#
#       Header:
#
#           X-tagged-for-folder:    folder_name
#
#       Config file format:
#
#           header      header_name         default_folder_name_if_no_filter_matches
#
#
#           Any number of:
#
#               or|and      folder_name
#                   include folder_name
#                   true    hdr                 "re_str"
#                   false   hdr                 "re_str"
#
#
#       For example:
#
#
#           ;
#           ;   Define the email header that will be added/replaced.
#           ;   If no filter is true, then the email header will be added/replaced with:
#           ;
#           ;           X-tagged-for-folder:    None
#           ;
#           header      X-tagged-for-folder:    "None"
#
#           ;
#           ;   Alternate definition - eliminate the header if no filter matches.
#           ;
#           header      X-tagged-for-folder:
#
#           ;
#           ;   The colon is not needed if there is no default 'folder' name.
#           ;
#           header      X-tagged-for-folder     ""
#
#
#           ;
#           ;   This expression is true if from: includes junkmail@yahoo.com or the to: adr is not null@example.com
#           ;   If this expression is true:
#           ;       Tag the email with the header, X-tagged-for-folder: really_junk_inc
#           ;       Continue processing the email through subsequent filters.
#           ;
#           and-continue    "really_junk_inc"
#               t       from    "junkmail@yahoo.com"
#               f       to      "^null@example.com$"
#
#           ;
#           ;   This expression is true if either really_junk_inc is true or the to: adr includes junk@tz.com
#           ;   If this expression is true:
#           ;       Tag the email with the header, X-tagged-for-folder: junk_inc
#           ;       Stop processing the email.
#           ;
#           or  "all_junk_inc"
#               t       x-tagged-for-folder "really_junk_inc"
#               true    to                  "junk@tz.com"
#
#
#           ;
#           ;   ops:
#           ;
#           ;   t   == y    == yes  == true
#           ;   f   == n    == no   == false
#           ;   &   == and  == &&
#           ;   |   == or   == ||
#
#           ;
#           ;   op-final            (default)
#           ;   op-continue
#           ;
#
#           ;
#           ;   Not any more:
#           ;       sender      == from
#           ;
#           ;   toORcc      == either TO: or CC:
#           ;
#
#           ;
#           ;   reply-to    == if not exists reply-to : sender
#           ;
#
#

from    __future__  import  print_function

import  sys
import  email
import  os
import  quopri
import  re

if  sys.version_info[0] < 3 :
    import  string
else :
    string  = str

try :
    basestring
except NameError :
    basestring  = str


if  False :
    #
    #   All because Python won't take -i on the command line, and it seems bogus to set the PYTHONPATH in procmail rules (or in cron jobs, for that matter).
    #
    if  sys.platform == 'win32' :
        tzpyp       = os.path.normpath("C:/tzpython")
        if  os.path.isdir(tzpyp) :
            sys.path.append(tzpyp)
        pass
    else :
        tzpyp       = os.path.normpath("~/tzpython")
        if  os.path.isdir(tzpyp) :
            sys.path.append(tzpyp)

        if  ('HOME' in os.environ) :
            tzpyp   = os.path.normpath(os.path.join(os.environ['HOME'], "tzpython"))
            if  os.path.isdir(tzpyp) :
                sys.path.append(tzpyp)
            pass
        pass
    pass


import  replace_file
import  tzemail
import  tzlib




mz_flt_re   =   re.compile(r'name="[^"]*"\r?\nenabled="yes"\r?\ntype="1"\r?\naction="Move to folder"\r?\nactionValue=".*?/([^/"]+)"\r?\ncondition="(.+?\))"', re.DOTALL)
mz_cmp_re   =   re.compile(r'\b(AND|OR)\s+\((\\\".*?[^\\]\\\"|[^,]+),(contains|doesn\'t contain|is|isn\'t|begins with|ends with),(\\\".*?[^\\]\\\"|[^,]+)\)(?:\s+|$)')

mz_dbq_re   =   re.compile(r'\\".+\\"$')
mz_fdq_re   =   re.compile(r'\\\\"')

def _unescape_str(s) :
    ss = str(s).split("%")
    for i in range(1, len(ss)) :
        ss[i] = chr(int(ss[i][0:2], 16)) + ss[i][2:]

    return("".join(ss))

def translate_mozilla_msgFilterRules_dat(fi) :



    def _strip_outside_double_quotes(s) :
        if  mz_dbq_re.match(s) :
            s = s[2:-2]
        return(s)



    fr  = None
    if  not hasattr(fi, 'read') :
        fr  = open(fi, "r")
    ftxt    = fi.read()
    if  fr  :
        fr.close()

    retval  = ""

    g               = mz_flt_re.findall(ftxt)
    for f in g :
        box         = _unescape_str(_strip_outside_double_quotes(f[0]))
        # print("box", box, f[1])
        cg          = mz_cmp_re.findall(f[1])
        op          = cg[0][0]
        retval     += op + " " + box + "\n"
        for c in cg :
            if      c[0] != op :
                print("Unequal ops in Mozilla filter files:", op, c[0])
                return(None)

            c1      = _strip_outside_double_quotes(c[1])
            c3      = _strip_outside_double_quotes(c[3])
            c3      = mz_fdq_re.sub('"', c3)
            # print(" OP:", c[0], "  HDR:", c1, "  HOW:", c[2], "  WHAT:", c3, "]")

            pre     = post = ""
            if      c[2] == "doesn't contain" :
                    tf    = "f"
            elif    c[2] == "contains" :
                    tf    = "t"
            elif    c[2] == "isn't" :
                    tf    = "f"
                    pre   = "^"
                    post  = "$"
            elif    c[2] == "is" :
                    tf    = "t"
                    pre   = "^"
                    post  = "$"
            elif    c[2] == "begins with" :
                    tf    = "t"
                    pre   = "^"
            elif    c[2] == "ends with" :
                    tf    = "t"
                    post  = "$"
            else :
                    print("Unknown how-to-match in filter for", box, ":", c[2])
                    return(None)

            if      (c1.lower() == "to or cc") :
                    c1    = "toORcc"

            retval += " " + tf + " " + c1 + ' "' + pre + re.escape(c3) + post + "\"\n"

        pass
    # print("g len", len(g))

    return(retval)








class MailHeaderTaggerFilterOp :
    def __init__(me, tf, regx, typ) :
        me.tf   = tf;
        me.regx = regx
        me.typ  = typ
    pass


class MailHeaderTaggerFilter :

    AND_OP      = '&'
    OR_OP       = '|'

    def __init__(me, name = None, op = None, stop = False, ln = 0) :

        if  not name : name = "None"
        if  not op   : op   = MailHeaderTaggerFilter.AND_OP

        me.name     = name
        me.op       = op
        me.stop     = stop
        me.ops      = []
        me.have_qp  = False
        me.have_tx  = False
        me.line_num = ln or 0


    def is_empty(me) :
        return(len(me.ops) == 0)


    def append(me, tf, compiled_re_exp__or__re_str_to_search_case_ignored__or__MailHeaderTaggerFilter) :

        regx    = compiled_re_exp__or__re_str_to_search_case_ignored__or__MailHeaderTaggerFilter

        if  tf  :
            tf  = True
        else :
            tf  = False

        typ     = ""

        if  isinstance(regx, basestring) :
            rl  = regx.lower()
            if  rl.startswith(r"^toorcc:\s+") :
                orflt   = MailHeaderTaggerFilter("tmp", MailHeaderTaggerFilter.OR_OP)
                orflt.append(True, r"^to:\s+" + regx[11:])      # note: remember that startswith is not an re thing. 11 characters gets just past the "\s+" in the regx definition string
                orflt.append(True, r"^cc:\s+" + regx[11:])
                orflt.append(True, r"^received:\s+" + regx[11:])
                me.ops.append(MailHeaderTaggerFilterOp(tf, orflt, typ))

                return

            if  rl.startswith(r"^body:\s+.*") :
                typ         = "body"
                regx        = regx[11:]                         # drop the 11 chars of "^body:\s+.*" from the filter regular expression
            elif  rl.startswith(r"^body:\s+") :
                typ         = "body"
                regx        = "^" + regx[9:]                    # drop the 9 chars of "^body:\s+" but put back the ^ so that it will match to the beginnings of text lines in the body

            if  rl.startswith(r"^qpbody:\s+.*") :
                typ         = "qpbody"
                regx        = regx[13:]
                me.have_qp  = True
            elif  rl.startswith(r"^qpbody:\s+") :
                typ         = "qpbody"
                regx        = "^" + regx[11:]
                me.have_qp  = True
            if  rl.startswith(r"^text_body:\s+.*") :
                typ         = "text_body"
                regx        = regx[16:]
                me.have_tx  = True
            elif  rl.startswith(r"^text_body:\s+") :
                typ         = "text_body"
                regx        = "^" + regx[14:]
                me.have_tx  = True

            # print("re [" + regx + "]")
            ml              = re.MULTILINE
            if  (regx.find(r"\n") >= 0) or (typ == "body") or (typ == "qpbody") or (typ == "text_body") :
                ml          = re.DOTALL

            regx            = re.compile(regx, re.IGNORECASE | ml)

        oe                  = MailHeaderTaggerFilterOp(tf, regx, typ)

        me.ops.append(oe)



    def match_email(me, flattened_email_hdrs, body, qpbody, text_body) :
        """
            Does this filter match the given flattened email header string and body string (containing LF delimited lines).
            Return (the_filter's_name, stop_bool) if there is a match.
            Return (None,None) if the filter does not match.
        """

        for exp in me.ops :
            if  isinstance(exp.regx, MailHeaderTaggerFilter) :
                ( fnd, stp )    = exp.regx.match_email(flattened_email_hdrs, body, qpbody, text_body)   # note: 'stop' is ignored in nested filters
                if  fnd == None :
                    fnd =  False
                else :
                    fnd =  True
                pass

            elif exp.typ == "body" :
                fnd = False
                if  exp.regx.search(body) :
                    fnd = True
                pass

            elif exp.typ == "qpbody" :
                fnd = False
                if  exp.regx.search(qpbody) :
                    fnd = True
                pass

            elif exp.typ == "text_body" :
                fnd = False
                if  exp.regx.search(text_body) :
                    fnd = True
                pass

            else :
                fnd = False
                if  exp.regx.search(flattened_email_hdrs) :
                    fnd = True
                pass

            if  not isinstance(exp.regx, MailHeaderTaggerFilter) :
                # print(fnd, exp.tf, exp.regx.pattern)                                          # ............. print out all regx to email comparisons .............
                pass

            if  fnd == exp.tf :
                if  me.op == MailHeaderTaggerFilter.OR_OP :
                    return( ( me.name, me.stop ) )
                pass
            else :
                if  me.op == MailHeaderTaggerFilter.AND_OP :
                    return( ( None, None ) )
                pass
            pass

        if  me.op == MailHeaderTaggerFilter.OR_OP :
            return( ( None, None ) )
        if  me.op == MailHeaderTaggerFilter.AND_OP :
            return( ( me.name, me.stop ) )

        return( ( None, None ) )                                                                # an exception should have been thrown for a bad op


    pass



hdr_re  = re.compile(r"header\s+(\S+)\s*(.*)",                                      re.IGNORECASE)
flt_re  = re.compile(r"(and|\&|\&\&|or|\||\|\|)(-continue\s+|-final\s+|\s)\s*(.*)", re.IGNORECASE)
exp_re  = re.compile(r"(t|true|y|yes|f|false|n|no)\s+(\S+)\s*\"(.*)\"",             re.IGNORECASE)


def parse_filter_def(s, hdr, tag) :
    """
        Given a string containing a filter config file's text,
        return a list containing:
            a header,
            a default tag (i.e. email destination folder name),
            and an array of MailHeaderTaggerFilter values.
    """

    s       = tzlib.multiline_strip(s)
    s       = list(map(string.strip, s.split("\n")))


    have_qp = False
    have_tx = False

    flts    = []


    chk_hdr = True
    hdr     = hdr or ""
    tag     = tag or ""


    #
    #   Get all the filters
    #
    i       = 0
    while i < len(s) :
        li  = s[i]
        i  += 1
        li  = li.strip()
        if  len(li) and (li[0] != ';') :

            if  chk_hdr :
                chk_hdr = False                                             # the "header" line must be the first non-comment in the file
                g   = hdr_re.match(li)
                if  g :
                    hdr = g.group(1)
                    tag = g.group(2)
                    continue
                pass

            g   = flt_re.match(li)                                          # looking for something like "AND box" or "OR mailbox_name"
            if  not g :
                print("No expression spec in input:", li, i)
                return( ( None, None, None, None, None ) )

            fli     = li

            op      = MailHeaderTaggerFilter.OR_OP
            if  re.match(r"[Aa\&]", g.group(1)) :
                op  = MailHeaderTaggerFilter.AND_OP

            # print(  li)

            stp = True
            if  g.group(2) and (len(g.group(2).strip()) > 0):
                stp = False

            name    = g.group(3)

            #
            #       Get all the things that this filter checks for
            #
            flt     = MailHeaderTaggerFilter(name, op, stp, i)

            while i < len(s) :
                li  = s[i]
                i  += 1
                li  = li.strip()
                if  len(li) and (li[0] != ';') :
                    g   = exp_re.match(li)                                      # looking for something like " t subject blah_blah"
                    if  not g :
                        i  -= 1
                        break

                    #
                    #       Assemble the final regx to recognize the particular header (or "^body:\s+\.*"   or "^body:\s+" )
                    #
                    fhdr    = re.sub(r":$", "", g.group(2))                     # get rid of : at the end of a header name
                    grex    = g.group(3)                                        # the filter op's regular expression
                    rex     = "^" + fhdr + ":"                                  # ok, now make sure the header name is looked for at the beginning of the regx, with a colon at the end of the header name
                    if  grex :
                        if  grex.startswith("^") :
                            grex    = grex[1:]
                            rex    += "\s+"                                     # skip past spaces that follow the header name - then require the filter op's stuff
                        else :
                            rex    += "\s+.*"                                   # skip past spaces and any number of characters before requiring the filter op's stuff
                        rex        += grex                                      # put the filter op's stuff at the end

                    # print("rex", rex)

                    flt.append(re.match(r"[tTyY]", g.group(1)), rex)
                pass

            if  flt.is_empty() :
                print("No expression in filter:", fli, i)
                return( ( None, None, None, None, None ) )

            flts.append(flt)
            if  flt.have_qp :
                have_qp = True
            if  flt.have_tx :
                have_tx = True
            pass
        pass

    tag = tag.strip()
    hdr = hdr.strip()

    return( ( hdr, tag, flts, have_qp, have_tx ) )





class MailHeaderTagger :

    HEADER              = "X-tagged-for-folder"


    def __init__(me, filter_def, header = None, dflt_tag = None) :

        me.header       = None
        me.dflt_tag     = None

        if  hasattr(filter_def, 'read') :
            filter_def  =      filter_def.read()
        elif  os.path.isfile(  filter_def) :
            fi          = open(filter_def, "r")
            filter_def  = fi.read()
            fi.close()

        ( hdr, dflt_tag, me.flts, me.have_qp, me.have_tx )  = parse_filter_def(filter_def, header, dflt_tag)

        if  hdr :
            me.header           = hdr
        if  dflt_tag :
            me.dflt_tag         = dflt_tag

        #
        #   Passed parameters override the config file.
        #
        if  header  :
            me.header           = header
        if  dflt_tag :
            me.dflt_tag         = dflt_tag

        if  not me.header :
            me.header           = MailHeaderTagger.HEADER

        if  not me.dflt_tag :
            me.dflt_tag         = "None"

        pass


    def tag_email(me, msg) :
        """
            Tag an email in text-line-array-no-EOL-chars form, if it should be.
            Return an array of the tags given to the email.
        """

        retval  = []

        tzemail.eliminate_email_header(msg, me.header)

        fhdrs   = tzemail.get_flattened_email_headers(msg)
        body    = tzemail.get_email_body_string(msg)

        qpbody          = None
        if  me.have_qp  :                               # is/are we looking at the email's body for this regx?
            # print("@@@@@", type(body), len(body))
            if  sys.version_info[0] < 3 :
                qpbody  = quopri.decodestring(body)
                # print("@@@@@@@", len(body), len(qpbody), type(body), type(qpbody))
            else        :
                qpbody  = tzlib.convert_to_utf8(body)   # python3 quopri takes bytes, so let's utf8 the body
                try     :
                    qpbody  = quopri.decodestring(body)
                    qpbody  = tzlib.convert_to_unicode(qpbody)
                except ValueError :
                    qpbody  = None                      # python3 give up. the string contains non-ascii, which is a pretty good hint it ain't quote-printable
                pass
            pass

        text_body   = None
        if  me.have_tx :

            ####################
            #########
            def _get_body(m, t = 'text/plain') :
                b           = ""
                for p in m.walk() :
                    if  p.get_content_type() == t :
                        pl      = p.get_payload(decode = True)
                        try     :
                            pl  = tzlib.convert_to_unicode(pl)
                            # pl  = pl.decode('utf8')
                        except UnicodeDecodeError :
                            if  False :
                                try :
                                    pl  = p.get_payload(decode = False)
                                except UnicodeDecodeError :

                                    return("")
                            pass
                        b  += pl
                    pass
                return(b)
            #########
            ####################

            m               = email.message_from_string("\r\n".join(msg) + "\r\n")
            text_body       = _get_body(m)
            if  not text_body :
                text_body   = _get_body(m, 'text/html')
            if  not text_body :
                text_body   = qpbody or body
            pass


        fnd     = False
        for f in me.flts :

            ( name, stop ) = f.match_email(fhdrs, body, qpbody or "", text_body)

            if  stop != None :
                fnd   = True

                if  f.line_num :
                    tzemail.eliminate_email_header(msg, me.header + "-MailHeaderTagger-filter-line-number")
                    tzemail.add_email_header(      msg, me.header + "-MailHeaderTagger-filter-line-number" + ": " + str(f.line_num))

                tzemail.eliminate_email_header(msg, me.header)
                tzemail.add_email_header(      msg, me.header + ": " + name)

                # print("added", me.header + ": " + name + " stop=" + str(stop))

                retval.append(name)
                if  stop :  break
                pass
            pass

        if  not fnd :
            tzemail.eliminate_email_header(msg, me.header)
            tzemail.add_email_header(msg, me.header + ": " + me.dflt_tag);
            retval.append(me.dflt_tag)

        return(retval)


def read_cfg_file_thru_cache(cfg_file_name, my_stat) :
    """ Read and return our config file MailHeaderTagger. If possible, read it from a pickled cache. Keep the cache up to date in any case. """
    flt             = None

    epsilon         = 0.001                                 # jeez, python 2 and 3 don't agree very closely on files' mtimes
    flt_file_name   = cfg_file_name + ".flt_pickle"
    latest_t        = max(os.stat(cfg_file_name).st_mtime, my_stat.st_mtime)
    try             :
        if  os.stat(flt_file_name).st_mtime + epsilon >= latest_t :
            flt = tzlib.unpickle_file(flt_file_name)
            if  os.stat(flt_file_name).st_mtime + epsilon < max(os.stat(cfg_file_name).st_mtime, my_stat.st_mtime) :
                flt = None                                  # the cfg or this program was suddenly changed while we read it
            pass
        pass
    except  :
        # tzlib.print_exception()
        flt = None
    pass

    if  flt == None :
        flt = MailHeaderTagger(cfg_file_name)                               # read the .cfg file to write the pickled version
        tzlib.pickle_file(flt_file_name, flt, tzlib.BEST_PICKLE_PROTOCOL)   # write it to the pickle file cache
        os.utime(flt_file_name, ( latest_t, latest_t ) )

    return(flt)


def process_email(ifile, ofile, cfg_file_name, my_stat) :
    """ Read an email from the input text file and put our header in and write it out to the output text file. """
    flt = read_cfg_file_thru_cache(cfg_file_name, my_stat)

    msg = tzemail.read_email_from_file(ifile)

    flt.tag_email(msg)                  # do our thing

    if  False :
        try :
            fo  = open("MailHeaderTagger_email.tmp", "wb")
            for li in msg :
                fo.write("%s\n" % ( li ) )
            fo.close()
        except :
            pass
        pass

    #
    #   Write the email out to stdout
    #
    for li in msg :
        print(li.rstrip(), file = ofile)

    pass


#
#
#
if __name__ == '__main__' :

    import  TZCommandLineAtFile


    my_stat = os.stat(sys.argv[0])      # the time of this script (though I wonder if it's of the .pyc file) - if it's newer than the cache, refresh the cache

    del(sys.argv[0])

    TZCommandLineAtFile.expand_at_sign_command_line_files(sys.argv)

    if  len(sys.argv) < 1 :
        print(  "No filter definition file given!")
        print(  """
Tell me the filter definition file and pipe in a message, which I will print, tagged.

Or tell me an input file name and I will understand these options:

    --output    output_file_name        Don't safely replace the input file.
    --mozilla   moz_filter_file         Translate the given mozilla filter file
    --tags                              Print only the "From - " and tags if no --output file is given.
""")
        sys.exit(101)

    cfg_file_name   = sys.argv.pop(0)

    flt             = None

    if  len(sys.argv) :                         # do we modify msgs in mozilla mboxes? or whatever?
        ofile_name  = None
        ifile_name  = None
        show_tags   = False

        while len(sys.argv) :
            mbox    = sys.argv.pop(0)

            if   (mbox  == "-o") or (mbox == "--output") :
                ofile_name = sys.argv.pop(0)

            elif (mbox  == "-t") or (mbox == "--tags") :
                show_tags   = True

            elif (mbox  == "-m") or (mbox == "--mozilla") :
                ifile_name  = sys.argv.pop(0)
                ofile_name  = cfg_file_name

                flt_txt = translate_mozilla_msgFilterRules_dat(ifile_name)

                if  not flt_txt :
                    print("Cannot translate Mozilla msgFilterRules.dat file:", ifile_name)
                    sys.exit(101)

                tfn             = ofile_name + ".tmp"
                fo              = open(tfn, "w")
                fo.write(flt_txt)
                fo.close()

                replace_file.replace_file(ofile_name, tfn, ofile_name + ".bak")

                ifile_name      = ofile_name = None

            else                :
                if  flt == None :
                    flt  = MailHeaderTagger(cfg_file_name)

                emails          = tzemail.read_mbox_file(mbox)

                unique_tags     = {}
                for msg in emails :
                    tags        = flt.tag_email(msg)
                    if  show_tags :
                        print(msg[0])
                        for t in tags :
                            unique_tags[t]  = True
                            print("    ", t)
                        pass
                    pass
                if  show_tags :
                    print(len(unique_tags), "unique tags")

                if  (ofile_name == None) and not tags:
                    ofile_name  = mbox

                if  ofile_name  :
                    tfn     = ofile_name + ".tmp"
                    fo      = open(tfn, "w")
                    for msg in emails :
                        for li in msg :
                            print(li.rstrip(), file = fo)       # Write the email out to our output file name
                        pass
                    fo.close()

                    replace_file.replace_file(ofile_name, tfn, ofile_name + ".bak")

                    ifile_name      = ofile_name = None

                pass
            pass
        pass

    else :                                                      # this is the normal, procmail, operating mode code. read stdin, apply our configured header tags, print to stdout

        process_email(sys.stdin, sys.stdout, cfg_file_name, my_stat)

    pass


__ALL__ = [
            'MailHeaderTagger',

            'parse_filter_def',

            'translate_mozilla_msgFilterRules_dat',
          ]

#
#
#
# eof
