#!/usr/bin/python

# average_image.py
#       --copyright--                   Copyright 2017 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       August 28, 2017         bar
#       November 7, 2017        bar     maxint->maxsize
#       April 27, 2019          bar     remove new numpy fuss
#       April 1, 2021           bar     set GPS exif in output file if possible
#                                       run ffmpeg on video file if given
#       April 2, 2021           bar     make the program work for a bunch of test videos
#       April 11, 2021          bar     don't make a .bak file when geotagging the output image
#       April 29, 2021          bar     default clipen is -1
#                                       move exiftool_gps() to show_image_where.py
#                                       get the lat/lon from jpg images, too
#       May 1, 2021             bar     move geotagging jpgs to tz_jpg.py
#       June 4, 2021            bar     clean up the warp code a bit
#                                       center the templates on the 1/4 points
#       June 6, 2021            bar     --nw --ne --se --sw
#                                       --prefix_name
#       March 5, 2023           bar     future print
#       --eodstamps--
##      \file
#       \namespace              tzpython.average_image

from    __future__  import  print_function

import  glob
import  os
import  re
import  sys
import  tempfile

import  cv2
import  numpy

import  show_image_where
import  tz_jpg
import  tzlib


def oops(err_msg) :
    """ Bail on this program. """
    sys.stderr.write(err_msg + "\n")
    sys.exit(1)


def ffmpeg_video_to_images(vfn, output_dir_name = None, output_file_name_pattern = None) :
    """
        Split the given video file in to output images named according to the given pattern. Return the directory name joined with the output_file_name_pattern of the output images (default: a_temp_dir/img_*.jpg ).

        The default output pattern is: img_%06d.jpg
        Note: the %06d is what ffmpeg takes, not %u.

    """
    if  not os.path.isfile(vfn) :
        oops('I cannot find "%s"!' % vfn)

    pat     = output_file_name_pattern  or "img_%06d.jpg"
    dn      = output_dir_name           or tempfile.mkdtemp(prefix = "avimg_")

    cmd     = 'ffmpeg -i "%s" -q:v 1 -f image2 "%s"' % ( vfn, os.path.join(dn, pat), )
    r, rs   = tzlib.run_program(cmd)
    if  r   :
        print(rs, file = sys.stderr)
        print('')
        oops('I cannot do "%s"!' % cmd)

    return(os.path.join(dn, '*.jpg'))


def get_warpnt(v, isz, w) :
    isz     = int(isz)
    w2      = int(w) // 2
    if  (type(v) == type(1.0)) and (v <= 1.0) :
        v   = int(isz *     v)
    v       = min(isz - w2, v)
    v       = max(      w2, v)
    return(v - w2)


def warp_average_image(fns, warpnts, verbose = 0) :
    mi      = len(fns) // 2                                 # start from the video's middle image - first going down to earlier images, then going up to later images

    img     = cv2.imread(fns[mi])
    ih, iw  = img.shape[:2]                                 # start the 4 templates as 1/8th of image squares at the four 1/4 and 3/4 positions in the hopes that these positions will be stable between images

    w       = iw // 8
    h       = ih // 8

    twx     = get_warpnt(warpnts[0][0], iw, w)
    tex     = get_warpnt(warpnts[1][0], iw, w)
    bex     = get_warpnt(warpnts[2][0], iw, w)
    bwx     = get_warpnt(warpnts[3][0], iw, w)
    twy     = get_warpnt(warpnts[0][1], ih, h)
    tey     = get_warpnt(warpnts[1][1], ih, h)
    bey     = get_warpnt(warpnts[2][1], ih, h)
    bwy     = get_warpnt(warpnts[3][1], ih, h)

    # print(  '@@@@', twx, twy, tex, tey, bex, bey, bwx, bwy, iw, ih)

    ttl     = img[twy : twy + h, twx : twx + w]
    ttr     = img[tey : tey + h, tex : tex + w]
    tbr     = img[bey : bey + h, bex : bex + w]
    tbl     = img[bwy : bwy + h, bwx : bwx + w]

    oimg    = img.astype('float')
    TSLACK  = 10                                            # how many pixels + or - the template can move per image
    cnt     = 1.0
    for ia in [ range(mi - 1, -1, -1), range(mi + 1, len(fns)), ] :     # first down, then up the images
        cnt    += len(ia)
        tmps    = [ [ twx, twy, ttl, ], [ tex, tey, ttr, ], [ bex, bey, tbr, ], [ bwx, bwy, tbl, ], ]
        # print('')
        # print([ iii[0:2] for iii in tmps ])
        for i  in ia    :
            fn  = fns[i]
            if  verbose :
                print("img", fn)

            img = cv2.imread(fn)

            for xytpi  in tmps :
                x   = xytpi[0]
                y   = xytpi[1]
                x   = max(x, TSLACK)                                                # don't allow the template to wander off-image
                y   = max(y, TSLACK)
                x   = min(x, img.shape[1] - TSLACK - w)
                y   = min(y, img.shape[0] - TSLACK - h)

                tpi = xytpi[2]
                im  = img[y - TSLACK : y + TSLACK + h, x - TSLACK : x + TSLACK + w ]
                # print("  xy", x, y, tpi.shape, img.shape, im.shape)

                try         :
                    r       = cv2.matchTemplate(tpi, im, cv2.TM_CCOEFF_NORMED)

                    yy, xx  = numpy.unravel_index(r.argmax(), r.shape)
                    xx     += (x - TSLACK)
                    yy     += (y - TSLACK)
                    xytpi[0]    = xx                                                # move the search area to the new, best position in the image
                    xytpi[1]    = yy
                except cv2.error :
                    print("Out of bounds", y, x, h, w, img.shape)                   # should not happen
                pass

            #
            #   Find the way the image must be rotated/warped such that the 4 points are brought around to where they where on the initial image.
            #   What can go wrong? Well, if a template is looking at something flowing, then it will flow, too. Typically, off-image.
            #   This logic was written to fix a hand-held video of things that don't move all that much - and with the moving stuff not in the 1/4 - 3/4 spots in the image.
            #
            wmat        = cv2.getPerspectiveTransform(numpy.array([ iii[0:2] for iii in tmps ], dtype = 'float32'), numpy.array([ [ twx, twy ], [ tex, tey ], [ bex, bey ], [ bwx, bwy ] ], dtype = 'float32'))
            wimg        = cv2.warpPerspective(img, wmat, ( img.shape[1], img.shape[0] ))
            oimg       += wimg.astype('float')

        # print([ iii[0:2] for iii in tmps ])
        pass

    oimg   /= cnt       # average the pixels
    img     = oimg.astype("uint8")

    return(img)


def average_image(fns, warpnts, verbose = 0) :
    """ Just average these images. Don't get fancy, the video was shot from a tripod. """
    im      = cv2.imread(fns[0])
    im      = im.astype("float")
    for fn in fns[1:] :
        if  verbose :
            print("img", fn)
        i   = cv2.imread(fn)
        im += i
    im     /= len(fns)
    im      = im.astype("uint8")
    return(im)


def write_image(fn, img) :
    if  fn and (img is not None) :
        if  os.path.exists(fn) :
            bi      = 0
            while True :
                bfn = fn + ".%u.bak" % bi
                if  not os.path.exists(bfn) :
                    break
                bi += 1
            os.rename(fn, bfn)
        cv2.imwrite(fn, img)
    pass


def geotag_jpg(fn, lat = None, lon = None, alt = None) :
    """ Geotag the given JPEG file with the given 1-decimal-number lat/lon and optional altitude in meters. """
    if  (lat is not None) and (lon is not None) :
        try     :
            jpg = tz_jpg.a_jpg(fn)
            jpg.set_geotag(lat, lon, alt)
            if  False :
                jpg.save(fn)                # we've already saved a .bak file, so don't call this
            else    :
                fo  = open(fn, "wb")
                fo.write(jpg.jpeg_data())
                fo.close()

            return(True)

        except ValueError :
            pass
        pass

    return(False)


def set_jpg_create_modify_dates(fn, zt) :
    """ Set the Create and Modify date in the given JPEG file. """
    if  zt is not None :
        try     :
            jpg = tz_jpg.a_jpg(fn)
            jpg.set_time(zt)
            if  False :
                jpg.save(fn)                # we've already saved a .bak file, so don't call this
            else    :
                fo  = open(fn, "wb")
                fo.write(jpg.jpeg_data())
                fo.close()

            return(True)

        except ValueError :
            pass
        pass

    return(False)


def parse_warp_point(argv) :
    def _parse_warp_point(argv) :
        s   = argv.pop(0).strip()
        g   = re.search(r'^(' + tzlib.float_regx_str + r')\s*%$', s)
        if  g :
            return(float(g.group(1)))
        g   = re.search(r'^(' + tzlib.FLOAT_regx_str + r')$', s)
        if  g :
            return(float(g.group(1)))
        g   = re.search(r'^(\d+)$', s)
        if  g :
            return(int(g.group(1)))
        oops('Warp point must be a percentage (0.0 .. 1.0) or an integer, not "%s"' % s)

    return(_parse_warp_point(argv), _parse_warp_point(argv))


def make_pref_name(fn, pref_nm) :
    if  pref_nm and fn :
        fn  = os.path.join(os.path.dirname(fn), (pref_nm or '') + os.path.basename(fn))
    return(fn)


if  __name__ == '__main__' :
    helpstr = """
    %s (options) input_image(s)     output_image
        or
                 input_video_file   output_image

    Take a given bunch of images (presumably taken from a video
    with the camera held very, very steady) and average them
    together.

    Warp the images to try to keep the unmoving things in the
    images stable.

    The goal is to show fluids in time-lapse, flowing form.

    Options:

            --prefix_name   name    Name to pre-pend to the output image
                                    file name.

            --video file_name       Video file to average if
                                    no input file names given.
                                    And/or to get GPS/altitude
                                    info from.
            --clip          N       Clip N images from the start and end.
            --clip_start    N       Clip N images from the start.
            --clip_end      N       Clip N images from the end.
            --at_for    N how_many  Clip N images from the start and
                                    read only how_many images.

            --warp                  Use the original warping logic.
            --average               Average the (tripoded) images. (default)

            --nw    x y             Set X and Y percentage or pixels
                                    from top-left of image for a warp-point.
                                    .       default:  25%% .25
            --ne    x y             .       default:  75%%  25%%
            --se    x y             .       default: .75   75%%
            --sw    x y             .       default: .25  .75
                                    Integers >= 1 are pixels.
                                    Warp-points are 1/8th of image sized.
                                    Setting a warp-point implies --warp.

    """ % os.path.basename(sys.argv[0] or __file__)

    ex      = 0                 # default exit code

    argv    = sys.argv[1:]

    fnames  = []
    vid_fn  = None
    clipst  = 0
    clipen  = -1
    avg_rtn = average_image
    awname  = "Averaging"
    warpnts = [ [ .25, .25 ], [ .75, .25 ], [ .75, .75 ], [ .25, .75 ] ]                # warp points' xy in percentage down from top-left. order: nw, ne, se, sw
    pref_nm = None
    verbose = 0

    if  tzlib.array_find([ a.lower() for a in argv ], [ "--help", "/help", "-h", "-?", "/?", "-?", "/h", ] ) >= 0 :
        print(helpstr)
        sys.exit(254)

    while len(argv) :
        a   = argv.pop(0)
        la  = a.lower()
        if  not a.startswith('-') :
            fnames.append(a)
        elif tzlib.find_argi(la, [ "--verbose", "-v", ]) >= 0 :
            verbose    += 1
        elif tzlib.find_argi(la, [ "--video", "--vid", "--movie", "--mov", ]) >= 0 :
            vid_fn      = argv.pop(0)
        elif tzlib.find_argi(la, [ "--clip", "-c", ]) >= 0 :
            clipst      =  int(argv.pop(0))
            clipen      = -clipst
        elif tzlib.find_argi(la, [ "--clip_start", "--clip_st", "--clip_s", "--cs", ]) >= 0 :
            clipst      =  int(argv.pop(0))
        elif tzlib.find_argi(la, [ "--clip_end",   "--clip_en", "--clip_e", "--ce", ]) >= 0 :
            clipen      = -int(argv.pop(0))
        elif tzlib.find_argi(la, [ "--at_for", "--af", ]) >= 0 :
            clipst      =  int(argv.pop(0))
            clipen      =  clipst + int(argv.pop(0))
        elif tzlib.find_argi(la, [ "--average", "--avg", "--av", "-a", ]) >= 0 :
            avg_rtn     = average_image
            awname      = "Averaging"
        elif tzlib.find_argi(la, [ "--warp", "-w", ]) >= 0 :
            avg_rtn     = warp_average_image
            awname      = "Warping"
        elif        tzlib.find_argi([ "--nw", "--ne", "--se", "--sw", ], la)   >= 0 :
            warpnts[tzlib.find_argi([ "--nw", "--ne", "--se", "--sw", ], la)]   = parse_warp_point(argv)
            avg_rtn     = warp_average_image
            awname      = "Warping"
        elif tzlib.find_argi(la, [ "--pre", "-pre_name", "--pre_nm", "--pref", "-pref_name", "--pref_nm", "--prefix", "-prefix_name", "--prefix_nm", ]) >= 0 :
            pref_nm     = argv.pop(0)
        else            :
            oops("I cannot understand the option %s" % a)
        pass

    if  not len(fnames) :
        print(helpstr)
        sys.exit(254)

    ofn         = None
    if  pref_nm is None :
        ofn     = fnames.pop()
        if  os.path.splitext(ofn)[1].lower() in [ '.mp4', '.mov', '.mp5', '.vid', '.avi', '.vid', '.ogv', ] :               # query some kind of MIME type !!!!
            fnames.append(ofn)
            ofn = None
        pass

    img_fns = None
    lat     = None
    lon     = None
    alt     = None
    when    = None
    if  vid_fn          :
        lat, lon, alt   = show_image_where.image_location(vid_fn)
        if  lat is None :
            print("I cannot find GPS information in", vid_fn, file = sys.stderr)
            ex          = 1
        if  not len(fnames) :
            img_fns     = ffmpeg_video_to_images(vid_fn)
            fnames.append(img_fns)
        when            = show_image_where.image_when(vid_fn)
        if  when is None :
            print("I cannot find CREATE or MODIFY information in", vid_fn, file = sys.stderr)
            ex          = 1
        pass

    fns         = []
    for fn     in fnames :
        fns    += glob.glob(fn)
    fns.sort()

    fnns        = []
    vfls        = []
    for fn     in fns   :
        if  os.path.splitext(fn)[1].lower() in [ '.mp4', '.mov', '.mp5', '.vid', '.avi', '.vid', '.ogv', ] :            # query some kind of MIME type !!!!
            vfls.append(fn)
        else            :
            fnns.append(fn)
        pass
    fns = fnns

    for fn in vfls  :
        la, ln, al  = show_image_where.image_location(fn)
        if  la is None :
            print("I cannot find GPS information in", fn, file = sys.stderr)
            ex      = 1
        whn         = show_image_where.image_when(fn)
        if  whn is None :
            print("I cannot find CREATE or MODIFY information in", fn, file = sys.stderr)
            ex          = 1
        tafn        = ffmpeg_video_to_images(fn)
        ifns        = glob.glob(tafn)
        ifns.sort()
        ifns        = ifns[ clipst : clipen ]
        if  not len(ifns) :
            print("All images clipped from", fn, file = sys.stderr)
            ex      = 1
        else :
            if  (len(vfls) == 1) and (not len(fns)) :
                vofn        = ofn or os.path.splitext(fn)[0] + '.jpg'   # simple cmd line with video file and output file name
            else            :
                vofn        =        os.path.splitext(fn)[0] + '.jpg'   # he's doing more than one video or doing a video and images and another vidoe (weird) so the output file name
            if  verbose     :
                print("%s images from %s" % ( awname, fn, ))
            img             = avg_rtn(ifns, warpnts, verbose = verbose)
            if  img is not None :
                vofn        = make_pref_name(vofn, pref_nm)
                write_image(vofn, img)
                geotag_jpg( vofn, lat = la, lon = ln, alt = al)
                set_jpg_create_modify_dates(vofn, whn)
            pass
        tzlib.whack_full_dir(os.path.dirname(tafn))

    if  not len(fns)    :
        if  not len(vfls) :
            oops("I could not find any input files for[ %s ]" % " ".join(fnames))
        if  vid_fn      :
            print("I found a video in the input files and the given --video one!", file = sys.stderr)
            ex          = 1
        pass
    else :
        fns = fns[ clipst : clipen ]
        if  not len(fns) :
            print("All images clipped so %s is not written." % make_pref_name(ofn, pref_nm), file = sys.stderr)
            ex  = 1
        else    :
            if  verbose     :
                if  img_fns :
                    print("%s images from %s" % ( awname, img_fns, ))
                else        :
                    print("%s %u images" % ( awname, len(fns), ))
                pass
            img = avg_rtn(fns, warpnts, verbose = verbose)
            if  img is not None :
                ofn         = make_pref_name(ofn, pref_nm)
                write_image(ofn, img)

                llfns       = list(fns)
                while ((lat is None) or (lon is None)) and len(llfns) :
                    lat, lon, alt   = show_image_where.image_location(llfns.pop(0))
                geotag_jpg(ofn, lat = lat, lon = lon, alt = alt)

                llfns       = list(fns)
                while (when is None) and len(llfns) :
                    when    = show_image_where.image_when(llfns.pop(0))
                set_jpg_create_modify_dates(ofn, when)
            pass
        pass

    if  img_fns :
        tzlib.whack_full_dir(os.path.dirname(img_fns))

    sys.exit(ex)


# eof
