#!/usr/bin/python

# box_compare.py
#       --copyright--                   Copyright 2013 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       May 19, 2013            bar
#       May 20, 2013            bar     allow 'info' in a_rect for caller's use
#                               bar     nicer printout
#       May 21, 2013            bar     -s
#                                       strip dupe files
#                                       --screen name
#       May 26, 2013            bar     --not_flat
#       June 3, 2013            bar     just ignore --info and 3 params to make life easier
#       July 11, 2013           bar     comment
#       July 23, 2013           bar     pass verbose to subroutine
#       July 25, 2013           bar     use pil_tostring
#       August 27, 2013         bar     Numeric replaced by numpy properly
#       September 13, 2013      bar     start on --post_choice
#       September 14, 2013      bar     --post_choice
#                                       use scipy for better speed
#       September 15, 2013      bar     experiment with looking for a sub-image
#                                       move the post_choice code to a lower rtn (where experiments can be done)
#       September 23, 2013      bar     correct namespace
#       October 12, 2013        bar     unused 'None' fixed in _dot_none_val()
#       April 21, 2014          bar     oooo, scipy can come back with a nan when we don't want it
#       May 5, 2014             bar     fix a bug that's accessed if the array is not a float, but rather a byte.
#       July 10, 2014           bar     able to find a black rectangle
#       January 19, 2016        bar     again fix the nan handling
#                                       speed up the sub-image search
#       February 27, 2016       bar     --screen_pattern
#       November 7, 2017        bar     maxint->maxsize
#       March 5, 2023           bar     future print
#       --eodstamps--
##      \file
#       \namespace              tzpython.box_compare
#
#
#       Given a 2-d array (presumably a gray scale image) and a
#       list of rectangles, find the rectangle likliest to fit
#       the high values in the array.
#
#       Assume that the top border and top sides are where the array values will be high.
#       On the bottom they may not be distinct from the rectangle's surroundings.
#
#

from    __future__  import  print_function

import  math
import  sys

try :
    xrange
except NameError :
    xrange  = range


import  numpy
from    PIL import  Image
import  scipy
import  scipy.spatial.distance


import  tzlib
tz_timer        = None
#       import  tz_timer        # uncomment to show speed



HZ_SAMPLES      = 16    #: number of samples to check at the top

VT_SAMPLES      = 32    #: number of samples to check along the sides (at the top, though, because of the data that this code is built to work with)
VT_SAMP_STEP    = 1     #: how far to step along a vertical pixel line

X_RANGE         = 24    #: find edges plus or minus this far from the given edges when looking for vertical edges
Y_RANGE         = 16    #: find edges plus or minus this far from the given edges when looking for horizontal edges


Y_MULT          =  1.0  #: weight the north side evaluation by this much in the final score
WEST_MULT       = 10.0  #: weight the west  side evaluation by this much in the final score
EAST_MULT       = 10.0  #: weight the east  side evaluation by this much in the final score


NEG_JUMP_MULT   = 2.0   #: multiply negative pixel changes by this much
POS_JUMP_POW    = 1.5   #: take positive pixel changes to this power

AVG_ZERO_INC    = 1.0   #: increase the difference in row/columns from the rectangle's edge by this much (to avoid divide by zeros)

CUTOFF_MULT     = 0.12  #: evalations less than this multiple of the best are thrown away

MAX_GRAY_DIFF   = 10    #: flat areas must average within this far from the amount the average is over "black"

POST_COMPARE_HALF       = 5         #: look for post_choice patterns plus/minus this far in pixels from where they should be if cmd line uses dots for the w h and/or NSEW values


PRINT_WHICH_FILTERED    = False     #: debug/test printout
PRINT_AFTER             = False     #: debug/test printout

PRINT_NORTH             = False     #: debug/test printout
PRINT_WEST              = False     #: debug/test printout
PRINT_EAST              = False     #: debug/test printout



class   an_avg(object) :
    def __init__(me, i, avg) :
        me.i    = i
        me.avg  = avg
        me.eval = None
    def __str__(me) :
        return("i:%u avg:%.1f %s" % ( me.i, me.avg, str(me.eval), ) )
    #   an_avg


def eval_jump(p_to, p_from) :
    """ Compute the gray jump from one pixel to another. """
    r   = float(p_to) - float(p_from)
    if  r < 0 :
        # return(math.pow(255 + r, POS_JUMP_POW))                 # Since we found that the values are floats to test_screen_settings and bytes to box_compare and box_compare is a better evaluator, we simulate the bug from before when we floated the two values given to us. Not that it makes any sense.
        return(r * NEG_JUMP_MULT)
    return(math.pow(r, POS_JUMP_POW))


def eval_avg(avg, i) :
    """ Compute the average evaluation for a row or column. """
    return(avg.avg * (X_RANGE + 1 - abs(avg.i - i)) / (X_RANGE + 1))
    return(avg.avg / (abs(avg.i - i) + AVG_ZERO_INC))



def eval_n(a, xyw, y_range = Y_RANGE, width = None, hite = None) :
    """ Return an array of an_avg's for the given array and xyw values. """

    try :
        x   = xyw.x
        y   = xyw.y
        w   = xyw.w
    except AttributeError :
        x   = xyw[0]
        y   = xyw[1]
        w   = xyw[2]
    x       = int(x)
    y       = int(y)
    w       = int(w)

    xw      = width or len(a[0])
    yh      = hite  or len(a)

    # print('@@@@@', xw, yh, a[yh - 1][xw - 1])

    ya      = range(max(0, y - y_range), min(yh - 1, y + y_range + 1))
    xa      = range(x, min(xw, x + w), max(1, w / HZ_SAMPLES))
    if  (len(ya) < 2) or (not len(xa)) :
        return([])

    # print("@@@@", width, hite, len(ya), len(xa), ya, xa)

    ay      = a[ya[0]]
    na      = [ ay[xi] for xi in xa ]
    avgs    = []
    for yi in ya :
        aa  = na
        ay  = a[yi + 1]
        na  = [ ay[xi] for xi in xa ]
        avgs.append(an_avg(yi + 1, sum([ eval_jump(v, aa[xi]) for xi, v in enumerate(na) ]) / float(len(xa))))
        if  PRINT_NORTH > 1 :
            print("@@@@", yi, avgs[-1].avg) # sum([ v - aa[xi] for xi, v in enumerate(na) ]), len(xa), aa, na, [ v - aa[xi] for xi, v in enumerate(na) ]
        pass

    return(avgs)



def eval_north(a, xyw, y_range = Y_RANGE, width = None, hite = None) :
    """ Evaluate how close the given edge is to a step-up in the values in the 2D array. Return the winning an_avg with its evaluation set. """

    avgs    = eval_n(a, xyw, y_range = y_range, width = width, hite = hite)
    if  not len(avgs) :
        return(None)

    try :
        y   = xyw.y
    except AttributeError :
        y   = xyw[1]
    evals   = [ eval_avg(avg, y) for avg in avgs ]

    ei              = tzlib.max_index(evals)
    avgs[ei].eval   = evals[ei]

    if  PRINT_NORTH :
        print("@@@@ north", avgs[ei], xyw.info)

    return(avgs[ei])





def eval_w(a, xyh, x_range = X_RANGE, width = None, hite = None) :
    """ Return an array of an_avg's for the given array and xyh values. """

    try :
        x   = xyh.x
        y   = xyh.y
        h   = xyh.h
    except AttributeError :
        x   = xyh[0]
        y   = xyh[1]
        h   = xyh[3]
    x       = int(x)
    y       = int(y)
    h       = int(h)

    xw      = width or len(a[0])
    yh      = hite  or len(a)

    # print('@@@@@', xw, yh, a[yh - 1][xw - 1])

    ya      = range(y, min(yh, y + VT_SAMPLES + VT_SAMPLES + VT_SAMPLES), 3)
    xa      = range(max(0, x - x_range), min(xw - 1, x + x_range))
    if  (not len(ya)) or (len(xa) < 2) :
        return([])

    # print("@@@@", width, hite, len(ya), len(xa), ya, xa)

    xi      = xa[0]
    na      = [ a[yi][xi] for yi in ya ]
    avgs    = []
    for xi in xa :
        aa  = na
        na  = [ a[yi][xi + 1] for yi in ya ]
        avgs.append(an_avg(xi + 1, sum([ eval_jump(v, aa[yi]) for yi, v in enumerate(na) ]) / float(len(ya))))
        if  PRINT_WEST > 1 :
            print("@@@@", xi, avgs[-1].avg)     # sum([ v - aa[yi] for yi, v in enumerate(na) ]), len(na), aa, na, [ v - aa[yi] for yi, v in enumerate(na) ]
        pass

    return(avgs)



def eval_west(a, xyh, x_range = X_RANGE, width = None, hite = None) :
    """ Evaluate how close the given edge is to a step-up in the values in the 2D array. Return the winning an_avg with its evaluation set. """

    avgs    = eval_w(a, xyh, x_range = x_range, width = width, hite = hite)
    if  not len(avgs) :
        return(None)

    try :
        x   = xyh.x
    except AttributeError :
        x   = xyh[0]
    evals   = [ eval_avg(avg, x) for avg in avgs ]

    ei              = tzlib.max_index(evals)
    avgs[ei].eval   = evals[ei]

    if  PRINT_WEST  :
        print("@@@@ west", avgs[ei], xyh.info)

    return(avgs[ei])



def eval_e(a, xyh, x_range = X_RANGE, width = None, hite = None) :
    """ Return an array of an_avg's for the given array and xyh values. """

    try :
        x   = xyh.x
        y   = xyh.y
        w   = xyh.w
        h   = xyh.h
    except AttributeError :
        x   = xyh[0]
        y   = xyh[1]
        w   = xyh[2]
        h   = xyh[3]
    x       = int(x)
    y       = int(y)
    w       = int(w)
    h       = int(h)

    x      += (w - 1)

    xw      = width or len(a[0])
    yh      = hite  or len(a)

    # print('@@@@@', xw, yh, a[yh - 1][xw - 1])

    ya      = range(y, min(yh, y + VT_SAMPLES * VT_SAMP_STEP), VT_SAMP_STEP)
    xa      = range(max(0, x - x_range), min(xw - 1, x + x_range))
    if  (not len(ya)) or (len(xa) < 2) :
        return([])

    # print("@@@@", width, hite, len(ya), len(xa), ya, xa)

    xi      = xa[0]
    na      = [ a[yi][xi] for yi in ya ]
    avgs    = []
    for xi in xa :
        aa  = na
        na  = [ a[yi][xi + 1] for yi in ya ]
        avgs.append(an_avg(xi, sum([ eval_jump(aa[yi], v) for yi, v in enumerate(na) ]) / float(len(ya))))
        if  PRINT_EAST > 1 :
            print("@@@@", xi, avgs[-1].avg)     # sum([ aa[yi] - v for yi, v in enumerate(na) ]), len(na), aa, na, [ v - aa[yi] for yi, v in enumerate(na) ]
        pass

    return(avgs)



def eval_east(a, xyh, x_range = X_RANGE, width = None, hite = None) :
    """ Evaluate how close the given edge is to a step-up in the values in the 2D array. Return the winning an_avg with its evaluation set. """

    avgs    = eval_e(a, xyh, x_range = x_range, width = width, hite = hite)
    if  not len(avgs) :
        return(None)

    try :
        x   = xyh.x + xyh.w   - 1
    except AttributeError :
        x   = xyh[0] + xyh[2] - 1
    evals   = [ eval_avg(avg, x) for avg in avgs ]

    ei              = tzlib.max_index(evals)
    avgs[ei].eval   = evals[ei]

    if  PRINT_EAST  :
        print("@@@@ east", avgs[ei], xyh.info)

    return(avgs[ei])



class   a_screen_pattern(object) :
    """ Define an pattern - a sub-image to be used by a a_screen_settings screen/probe-depth detector. """
    def __init__(me, w, h, pixels) :
        me.w    = w
        me.h    = h
        me.pix  = pixels
    #   a_screen_pattern



class   a_post_choice(object) :
    """
        Define a pattern that, when matched, tell the North, South, East, and West boundaries of a rectangle.

    """

    def __init__(me, x, y, w, h, pw, ph, pat, N, S, E, W) :
        """ Initialize/instantiate an object. """
        me.x    = x
        me.y    = y
        me.w    = w
        me.h    = h
        me.pw   = pw
        me.ph   = ph
        me.pat  = pat
        me.N    = N
        me.S    = S
        me.E    = E
        me.W    = W
        me.pat  = numpy.asarray(me.pat)

    def fix_defaults(me, x, y, w, h) :
        """ Fix default values for things that were dotted in the cmd line. """
        if  me.N   == '.' : me.N    = y     - me.y
        if  me.S   == '.' : me.S    = y + h - me.y
        if  me.E   == '.' : me.E    = x + w - me.x
        if  me.W   == '.' : me.W    = x     - me.x

        if  me.w   == '.' :
            me.w    = (POST_COMPARE_HALF * 2) + 1
            me.x   -=  POST_COMPARE_HALF
        if  me.h   == '.' :
            me.h    = (POST_COMPARE_HALF * 2) + 1
            me.y   -=  POST_COMPARE_HALF
        pass

    def find_best(me, pix, black = 0) :
        """ Find the best position in pix of our choice. """
        bv      = sys.maxsize
        bx      = me.x
        by      = me.y
        sa      = []
        pat     = me.pat.flatten()                                              # 6 minutes instead of 10 for a long test
        for y      in xrange(me.y, me.y + (me.h or 0)) :
            for x  in xrange(me.x, me.x + (me.w or 0)) :
                if  pat != None :
                    sm  = scipy.spatial.distance.correlation(pix[y : y + me.ph, x : x + me.pw].ravel(), pat)
                else        :
                    sm  = 0.0
                    i   = 0
                    for yi     in xrange(me.ph) :
                        if  False   :
                            for xi in xrange(me.pw) :
                                d   = pix[y + yi][x + xi] - black - me.pat[i]
                                # sm += abs(d)                                  # a few milliseconds faster - don't know if it's better or worse for our purposes
                                sm += d * d
                                i  += 1
                            pass
                        else :
                            c       = scipy.spatial.distance.correlation(pix[y + yi, x:x + me.pw], me.pat[i : i + me.pw])
                            if  math.isnan(c)   :
                                pix[y + yi, x]  = int(pix[y + yi, x]) ^ 1       # kludge, handles both 0's and 255's, I believe - if this routine is given an array of all zeros or all 255's, then it returns nan
                                me.pat[i + 1]   = int(me.pat[i + 1] ) ^ 1       # stops having to set c to 2.0 in Python 2.7, but the code is still there just in case - problem happens when the pixels are zero and the pattern is all one number, like 11 - u_b logo
                                c   = scipy.spatial.distance.correlation(pix[y + yi, x:x + me.pw], me.pat[i : i + me.pw])
                                if  math.isnan(c) :
                                    c           = 2.0
                                pix[y + yi, x]  = int(pix[y + yi, x]) ^ 1
                                me.pat[i + 1]   = int(me.pat[i + 1] ) ^ 1
                            sm += c
                            i  += me.pw
                        if  (me.h == 1) and (me.w == 1) :
                            print( " ".join([ "%03u" % (int(pix[y + yi][x + xi]) - black) for xi in xrange(me.pw) ]))
                        pass
                    pass
                if  bv  > sm :
                    bv  = sm
                    bx  = x
                    by  = y
                sa.append(sm)
                pass
            pass
        # print("@@@@", bx, by, '\n'.join([ str(s) for s in sa ]))
        return(bx, by, bv)

    #   a_screen_setting_post_choice


class   a_rect(object) :
    def __init__(me, x, y, w, h, black = None, flats = None, not_flats = None, post_choice = None, info = None, blk = 0.0) :
        me.x            = x
        me.y            = y
        me.w            = w
        me.h            = h
        me.black        = black
        me.flats        = flats     or []
        me.not_flats    = not_flats or []
        me.post_choice  = post_choice
        me.info         = info
        me._blk         = blk                           # used by post_choice - the average "black" pixel value
        if  post_choice :
            post_choice.fix_defaults(x, y, w, h)
        pass
    def __str__(me)     :
        return("%u:%u-%ux%u" % ( me.x, me.y, me.w, me.h, ) )
    #   a_rect





def find_rect(pix, rects, x_range = X_RANGE, y_range = Y_RANGE, width = None, hite = None, black = 0, find_2nd = False) :
    """ Given a 2-d array of values (gray pixels, for instance), return a partial copy of the rectangle, with extra information, that best fits around the "light" pixels. May return two rectangles for debug/test use. """

    ra          = [ a_rect(r.x, r.y, r.w, r.h, post_choice = r.post_choice, info = r.info, blk = r._blk) for r in rects ]
    for ri, r in enumerate(ra) :
        r.ri    = ri                                                                    # lead back to the original rectangle

    #
    #       Do the north side
    #
    evals   = [ eval_north(pix, r, y_range = y_range, width = width, hite = hite) for r in ra ]

    ei      = tzlib.max_index([ (e and e.eval) or -1 for e in evals ])
    if  (not len(evals)) or (ei < 0) or (not evals[ei]) or (evals[ei].eval < 0) :      # note: check < 0 to find black "rectangles"
        if  PRINT_WHICH_FILTERED :
            print("north filtered", [ str(e.eval) for e in evals ])
        return(None)

    for ri, r in enumerate(ra) :
        r.nv    = evals[ri].eval if evals[ri]   else -1
        r.ni    = evals[ri].i    if evals[ri]   else  0

    half_e  = ra[ei].nv * CUTOFF_MULT
    ra      = [ r for r in ra if r.nv >= half_e ]                                       # don't look at rects that didn't make the grade on the top border
    if  PRINT_AFTER :
        print("@@@@ after n", [ r.info for r in ra ])
        # print("@@@@", len(ra), [ str(r.nv) + " " + str(r.ri) for r in ra ])


    #
    #       Do the west side
    #
    evals   = [ eval_west(pix, r, x_range = x_range, width = width, hite = hite) for r in ra ]

    ei      = tzlib.max_index([ (e and e.eval) or -1 for e in evals ])
    if  (not len(evals)) or (ei < 0) or (not evals[ei]) or (evals[ei].eval < 0) :
        if  PRINT_WHICH_FILTERED :
            print("west filtered", [ str(e.eval) for e in evals ])
        return(None)

    for ri, r in enumerate(ra) :
        r.wv    = evals[ri].eval if evals[ri]   else -1
        r.wi    = evals[ri].i    if evals[ri]   else  0

    half_e  = ra[ei].wv * CUTOFF_MULT
    ra      = [ r for r in ra if r.wv >= half_e ]                                       # don't look at rects that didn't make the grade on the west border
    if  PRINT_AFTER :
        print("@@@@ after w", len(ra), [ r.info for r in ra ])


    #
    #       Do the east side
    #
    evals   = [ eval_east(pix, r, x_range = x_range, width = width, hite = hite) for r in ra ]

    ei      = tzlib.max_index([ (e and e.eval) or -1 for e in evals ])
    if  (not len(evals)) or (ei < 0) or (not evals[ei]) or (evals[ei].eval < 0) :
        if  PRINT_WHICH_FILTERED :
            print("east filtered", [ str(e.eval) for e in evals ])
        return(None)

    for ri, r in enumerate(ra) :
        r.ev    = evals[ri].eval if evals[ri]   else -1
        r.ei    = evals[ri].i    if evals[ri]   else  0

    half_e  = ra[ei].ev * CUTOFF_MULT
    ra      = [ r for r in ra if r.ev >= half_e ]                                       # don't look at rects that didn't make the grade on the east border
    if  PRINT_AFTER :
        print("@@@@ after e", len(ra), [ r.info for r in ra ])


    #
    #   Combine the sides for the final evaluation
    #
    for r in ra :
        r.eval  = (r.nv * Y_MULT) + (r.wv * WEST_MULT) + (r.ev * EAST_MULT)
        # print("@@@@", r.info, r.nv, r.wv, r.ev)

    ei      = tzlib.max_index([ r.eval for r in ra ])
    rect    = ra[ei]

    r       = rect
    if  r and  r.post_choice :
        if  tz_timer :
            t     = tz_timer.tick()
        x, y, v     = r.post_choice.find_best(pix, black = r._blk)
        if  r.post_choice.W != None :   r.wi    = x + r.post_choice.W
        if  r.post_choice.E != None :   r.ei    = x + r.post_choice.E - 1           # inside index
        if  r.post_choice.N != None :   r.ni    = y + r.post_choice.N
        if  r.post_choice.S != None :   r.h     = y + r.post_choice.S - r.ni        # bar: these name/values could be consistent, like si instead of h or w instead of ei
        if  tz_timer :
            tt    = tz_timer.tick()
            print("@@@@", tt - t)
        # print("@@@@", x, y, r.post_choice.N, r.post_choice.S, r.post_choice.E, r.post_choice.W, r._blk)

    if  find_2nd :
        ra.sort(lambda a, b : cmp(b.eval, a.eval))
        # print("@@@@", ra[0].eval, (len(ra) >= 2) and ra[1].eval)
        return(rect, ((len(ra) > 1) and ra[1]) or None)

    return(rect)



def find_best_rect(rects, pix, find_2nd = False, verbose = 0, fn = None) :
    """ Find the best match, from those a_rect's or equivalents in rects, for the given image. """

    nra             = []
    for r in rects  :
        r._blk       = 0.0
        if  hasattr(r, 'black') and r.black :
            rb      = r.black
            r._blk   = sum([ sum(pr[rb.x : rb.x + rb.w]) for pr in pix[rb.y : rb.y + rb.h ] ]) / float(rb.w * rb.h)             # note: r._blk is also used by post_choice

        if  hasattr(r, 'flats') :
            for f in r.flats :
                pv  = sum([ sum(pr[f.x : f.x + f.w]) for pr in pix[f.y : f.y + f.h ] ]) / float(f.w * f.h)
                if  verbose :
                    print("Flat", pv - r._blk - f.gray, "flat_raw", pv, "gray", f.gray, "black", r._blk, f, r.info, fn)         # , pix[f.y][f.x : f.x + f.w]
                if  abs(pv - r._blk - f.gray) > MAX_GRAY_DIFF :
                    r   = None                                                                                                  # filter rects that have an area that's not close enough in gray to the --flat area's gray pixel value
                    break
                pass
            pass
        if  hasattr(r, 'not_flats') :
            for f in r.not_flats :
                pv  = sum([ sum(pr[f.x : f.x + f.w]) for pr in pix[f.y : f.y + f.h ] ]) / float(f.w * f.h)
                if  verbose :
                    print("Notflat", pv - r._blk - f.gray, "flat_raw", pv, "gray", f.gray, "black", r._blk, f, r.info, fn)      # , pix[f.y][f.x : f.x + f.w]
                if  abs(pv - r._blk - f.gray) <= MAX_GRAY_DIFF :
                    r   = None                                                                                                  # filter rects that have an area that's too close in gray to the --not_flat area's gray pixel value
                    break
                pass
            pass
        if  r   :
            nra.append(r)
        pass

    rr          = find_rect(pix, nra, find_2nd = find_2nd)       # , width = img.size[0], hite = img.size[1])

    return(rr)




def do_it(fns, rects, verbose = 0) :
    """ Find and print the best match for each of the given files. """

    mxn = (len(rects) and max([ len(r.info) for r in rects ])) or 4
    for fn in fns :
        img         = Image.open(fn)
        img         = img.convert('L')
        pix         = numpy.asarray(numpy.fromstring(tzlib.pil_tostring(img), numpy.uint8))
        w           = img.size[0]
        h           = img.size[1]
        pix.shape   = h, w

        # import    tz_timer
        # t         = tz_timer.tick()

        ra          = [ r for r in rects if ((not r.s_width) or (r.s_width == w)) and ((not r.s_hite) or (r.s_hite == h)) ]         # only consider template rects that are the same size as the image's width and hite

        rr          = find_best_rect(ra, pix, find_2nd = True, verbose = verbose, fn = fn)

        # tt        = tz_timer.tick()
        # print(tt - t)

        if  rr     != None :
            r, r2   = rr

            def _eq_chr(i1, i2) : return(((abs(i1 - i2) > 3) and ' ') or '=')

            print("%4u %-*s %5.0f + %5.0f + %5.0f = %5.0f > %-10s | %4u%s%-4u %4u%s%-4u %4u%s%-4u %20s %s" % ( r.ri, mxn, str(r.info), r.nv, r.wv, r.ev, r.eval, ((r2 and ("%-5.0f:%-4u" % ( r2.eval, r2.ri, ) )) or "None"), r.ni, _eq_chr(r.ni, r.y), r.y, r.wi, _eq_chr(r.wi, r.x), r.x, r.ei, _eq_chr(r.ei, r.x + r.w - 1), r.x + r.w - 1, str(r), fn ))
        else    :
            print("None", fn)
        pass
    pass





help_str    = """
%s (options) ambiguous_image_file_name...

    Find which of given rectangles fit best around light pixels in an image.

Options:

    --rect      x y w h                         Define a rectangle. Multiple rectangles should be defined.
    --name      name width hite                 Name the next rectangle and give its width and hite.
    --black     x y w h                         Specify the black            area for the next --rect.
    --flat      gray_level x y w h              Specify a   flat-colored     area for the next --rect.
    --not_flat  gray_level x y w h              Specify a   not-flat-colored area for the next --rect.
    --post_choice  x y w h pw ph pv... N S E W  Specify a pattern to match near the found --rect and
                                                  where, relative to it to put the N S E and W borders.
                                                "pw" may be a screen pattern to replace pw ph pv values.
    --screen_pattern    name w h pixels         Define a pattern for --post_choice matching.
    --subdirs                                   Look for input files in sub-directories.
    --info  a b c                               Ignored.
    --verbose                                   Increase the verbosity level.

This program finds the given rectangle that fits best around lite pixels in an image.

"""



if  __name__ == '__main__' :

    import  os
    import  re

    import  TZCommandLineAtFile
    import  tz_os_priority


    program_name    = sys.argv.pop(0)

    TZCommandLineAtFile.expand_at_sign_command_line_files(sys.argv)


    verbose     = 0
    subdirs     = False
    rects       = []
    flats       = []
    not_flats   = []
    post_choice = None
    patterns    = {}
    black       = None
    name        = ""
    width       = None
    hite        = None

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--help", "-h", "-?", "/h", "/H", "/?" ] )
        if  oi < 0  :   break
        del sys.argv[oi]
        print(help_str % ( os.path.basename(program_name), ))
        sys.exit(254)


    while True :
        oi  = tzlib.array_find(sys.argv, [ "--info", ] )
        if  oi < 0  :   break
        del sys.argv[oi]
        del sys.argv[oi]
        del sys.argv[oi]
        del sys.argv[oi]

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--verbose", "-v", ] )
        if  oi < 0  :   break
        del sys.argv[oi]
        verbose    += 1

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--sub_dirs", "--subdirs", "--sub-dirs", "-s", ] )
        if  oi < 0  :   break
        del sys.argv[oi]
        subdirs     = True

    ai  = 0
    while ai < len(sys.argv) :
        a   = sys.argv[ai]

        if tzlib.array_find([ "--rate", "--crop", "-r", ], a) >= 0 :
            del(sys.argv[ai])
            x           = int(sys.argv.pop(ai))
            y           = int(sys.argv.pop(ai))
            w           = int(sys.argv.pop(ai))
            h           = int(sys.argv.pop(ai))
            r           = a_rect(x, y, w, h, black = black, flats = flats, not_flats = not_flats, post_choice = post_choice, info = name or str(len(rects)))
            r.s_width   = width
            r.s_hite    = hite
            rects.append(r)
            name        = ""
            width       = None
            hite        = None
            flats       = []
            not_flats   = []
            post_choice = None
            black       = None

        elif tzlib.array_find([ "--name", "--screen", ], a) >= 0 :
            del(sys.argv[ai])
            name    = os.path.basename(sys.argv.pop(ai))
            width   = int(sys.argv.pop(ai))
            hite    = int(sys.argv.pop(ai))

        elif tzlib.array_find([ "--flat", ], a) >= 0 :
            del(sys.argv[ai])
            g               = int(sys.argv.pop(ai))
            x               = int(sys.argv.pop(ai))
            y               = int(sys.argv.pop(ai))
            w               = int(sys.argv.pop(ai))
            h               = int(sys.argv.pop(ai))
            flats.append(a_rect(x, y, w, h))
            flats[-1].gray  = g

        elif tzlib.array_find([ "--not_flat", "--not-flat", "--notflat", ], a) >= 0 :
            del(sys.argv[ai])
            g               = int(sys.argv.pop(ai))
            x               = int(sys.argv.pop(ai))
            y               = int(sys.argv.pop(ai))
            w               = int(sys.argv.pop(ai))
            h               = int(sys.argv.pop(ai))
            not_flats.append(a_rect(x, y, w, h))
            not_flats[-1].gray  = g

        elif tzlib.array_find([ "--black", ], a) >= 0 :
            del(sys.argv[ai])
            x               = int(sys.argv.pop(ai))
            y               = int(sys.argv.pop(ai))
            w               = int(sys.argv.pop(ai))
            h               = int(sys.argv.pop(ai))
            black           = a_rect(x, y, w, h)

        elif tzlib.array_find([ "--screen_pattern", ], a) >= 0 :
            del(sys.argv[ai])
            nm  = sys.argv.pop(ai)
            pw  = int(sys.argv.pop(ai))
            ph  = int(sys.argv.pop(ai))
            pat = []
            for i in xrange(pw * ph) :
                pat.append(int(sys.argv.pop(ai)))
            patterns[nm] = a_screen_pattern(pw, ph, pat)

        elif tzlib.array_find([ "--post_choice", ], a) >= 0 :

            def _dot_none_val(v) :
                if  v not in [ '.', 'None', ] :
                    v       = int(v)
                return(v)

            del(sys.argv[ai])
            x               = int(sys.argv.pop(ai))
            y               = int(sys.argv.pop(ai))
            w               = _dot_none_val(sys.argv.pop(ai))
            h               = _dot_none_val(sys.argv.pop(ai))
            pw              = sys.argv.pop(ai)
            if  pw in patterns :
                ph          = patterns[pw].h
                pat         = patterns[pw].pix
                pw          = patterns[pw].w
            else            :
                pw          = int(pw)
                ph          = int(sys.argv.pop(ai))
                pat         = []
                for i in xrange(pw * ph) :
                    pat.append(int(sys.argv.pop(ai)))
                pass
            N               = _dot_none_val(sys.argv.pop(ai))
            S               = _dot_none_val(sys.argv.pop(ai))
            E               = _dot_none_val(sys.argv.pop(ai))
            W               = _dot_none_val(sys.argv.pop(ai))
            post_choice     = a_post_choice(x, y, w, h, pw, ph, pat, N, S, E, W)

        else :
            ai += 1

        pass


    fns = []
    while len(sys.argv) :
        afn         = sys.argv.pop(0)
        if  (not afn.startswith('-')) and (not re.search(r"^" + tzlib.float_regx_str + "$", afn)) :
            fa      = tzlib.ambiguous_file_list(afn, do_sub_dirs = subdirs)
            if  not len(fa) :
                print("No files for", afn)
            fns    += fa
        pass

    fns = tzlib.without_dupes(fns)

    if  not len(fns) :
        print("Tell me files to evaluate!")
        sys.exit(101)

    if  not len(rects) :
        print("Tell me rects to search!")
        sys.exit(101)

    tz_os_priority.set_proc_to_idle_priority(0)

    fns.sort(  lambda a, b : cmp(a.lower(), b.lower()))                                                                                     # so equal results are consistent in printout
    rects.sort(lambda a, b : cmp(a.info.lower(), b.info.lower()) or cmp(a.y, b.y) or cmp(a.x, b.x) or cmp(a.h, b.h) or cmp(a.w, b.w))       # so equal results are consistent in printout

    do_it(fns, rects, verbose = verbose)



#
#
# eof
