#!/usr/bin/python

# combine_files.py
#       --copyright--                   Copyright 2012 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       May 6, 2012             bar
#       May 27, 2012            bar     doxygen namespace
#       March 5, 2023           bar     future print
#       --eodstamps--
##      \file
#       \namespace              tzpython.combine_files
#
#
#       Combine multiple files in to a given number of files.
#
#


from    __future__  import  print_function

HOW_MANY    = 255                       # how many files to create, max


help_str    = """
%s      (options)   ambiguous_input_file_names...

        Combine files in to a given number of files.

Options:

    --output            output_template     Output file name template     (default: first_input_file_####.first_input_file_extension)
    --how_many          N                   Set how many files to create. (default: %u)
    --subdir                                Search subdirectories for input files.

"""




if  __name__ == '__main__' :
    import  os
    import  sys

    import  TZCommandLineAtFile
    import  tzlib


    program_name    = sys.argv.pop(0)

    TZCommandLineAtFile.expand_at_sign_command_line_files(sys.argv)


    how_many        = HOW_MANY
    ofile_name      = None
    sub_dirs        = False

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--help", "-h", "-?", "/h", "/H", "/?" ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        print(help_str % ( os.path.basename(program_name), how_many ))

        sys.exit(254)


    while True :
        oi  = tzlib.array_find(sys.argv, [ "--how_many", "--how-many", "--howmany", "--count", "-m", "-c", ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        how_many        = max(1, int(sys.argv.pop(oi)))

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--output", "-o", ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        ofile_name      = sys.argv.pop(oi)

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--sub_dirs", "--sub-dirs", "--subdirs", "-s", ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        sub_dirs        = True



    fls     = {}
    while len(sys.argv) :
        fn  = sys.argv.pop(0)
        fns = tzlib.ambiguous_file_list(fn, do_sub_dirs = sub_dirs)
        if  not len(fns) :
            print("No files found for [%s]!" % fn)
            sys.exit(101)

        ofile_name  = ofile_name or fns[0]

        for fn in fns :
            fls[fn] = fn

        pass

    fns = fls.keys()
    fns.sort(lambda a, b : cmp(os.path.basename(a), os.path.basename(b)) or cmp(a.lower(), b.lower()) or cmp(a, b))

    def get_ofn(n) :
        return(os.path.join(os.path.dirname(ofile_name), os.path.basename(os.path.splitext(ofile_name)[0]) + ("_%04u" % n) + os.path.splitext(ofile_name)[1]))

    n   = -1
    fd  = ""
    cnt = (len(fns) + how_many - 1) / how_many          # how many files to put in each output file
    while len(fns) :
        if  (not cnt) and len(fd) :
            n  += 1
            tzlib.write_whole_binary_file(get_ofn(n), fd)
            fd  = ""
            how_many    = max(1, how_many - 1)
            cnt = (len(fns) + how_many - 1) / how_many
        cnt    -= 1
        fd     += tzlib.read_whole_binary_file(fns.pop(0))

    if  len(fd) :
        n  += 1
        tzlib.write_whole_binary_file(get_ofn(n), fd)

    pass


#
#
#
# eof
