#!/usr/bin/python

# hikify.py
#       --copyright--                   Copyright 2008 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       October 25, 2007        bar
#       October 27, 2007        bar     put the source track names in the output tracks
#       October 28, 2007        bar     spin routine out to tz_gps
#       May 11, 2008            bar     handle no tracks found
#                                       show_info
#                                       process input in way that allows lots of files
#       May 11, 2008            bar     change name from hikes to hikify and put in tzpython dir
#       May 12, 2008            bar     combine
#       May 13, 2008            bar     more combine
#       May 14, 2008            bar     force no_time even if cmd line logic doesn't, if we whack the speeds
#       May 16, 2008            bar     nmea input
#       May 17, 2008            bar     email adr
#       May 20, 2008            bar     map_them
#       May 31, 2008            bar     fix a couple of cmd line error msgs
#       September 2, 2008       bar     spin the include/exclude logic to a routine
#       September 3, 2008       bar     fix a blown-edit typo
#       September 7, 2008       bar     compress runs of __ to one of 'em
#                                       --sparsify
#       September 8, 2008       bar     comment
#       October 26, 2008        bar     set split output files' date/time
#       November 2, 2008        bar     note about settings and big_points
#                                       start code to do start_point and end_point
#       November 5, 2008        bar     put distance and altitude info in track descriptions
#       January 25, 2009        bar     cachew
#                                       put LF after description
#                                       move hike count printout over to right if non-zero
#       February 19, 2009       bar     --bike
#       April 30, 2009          bar     fix non-caching logic
#       November 22, 2009       bar     help typo
#       January 9, 2010         bar     cache date-time, file name, input/output crcs, and use date-time/file-name to filter before reading an input file for crc
#       March 23, 2010          bar     forget_file
#       May 8, 2010             bar     allow s on end of --forget_file
#       June 20, 2010           bar     try to put the track in time order if there are name collisions
#       April 24, 2011          bar     --merge
#                                       --output
#       July 11, 2011           bar     --join
#       July 14, 2011           bar     mess with --start_point and --end_point but don't test 'em
#                                       --snip and --snip_near
#       August 17, 2011         bar     commented-out debug printout
#       November 29, 2011       bar     pyflake cleanup
#       February 3, 2012        bar     snip 'em before doing the splits, etc. so that shortened, snipped tracks are filtered out
#       May 27, 2012            bar     doxygen namespace
#       April 16, 2016          bar     set proper output file time to last point if that's what's asked for
#       May 11, 2016            bar     --bikify_file
#       July 23, 2016           bar     um. use the bikify_file settings on the first look at the file
#       May 12, 2018            bar     comment about merging
#       May 11, 2019            bar     --unforget_file
#       June 3, 2019            bar     --verbose (jeez, how many of these things do we need?)
#       July 28, 2019           bar     tell the user what file(s) we write
#       August 8, 2019          bar     when merging, toss out big points that don't have all the tracks being merged involved, sort of - can smooth things by avoiding partial matches in time - so we only have samples at times all tracks had a sample for
#       October 26, 2019        bar     print when the tracks were done
#       December 5, 2019        bar     verbose stuff
#                                       max_max param to only_big_points_from_max_tracks()
#       July 18, 2020           bar     don't let him put a file name with an extension with a --bikify_file option
#       April 30, 2021          bar     verbosely print forgotten files and other things
#                                       finally make the forget_files regexs work properly
#       July 12, 2021           bar     work with --snip logic
#       August 10, 2021         bar     comment
#       October 1, 2022         bar     --eot_strip_file
#                                       more verbose printing
#       February 25, 2023       bar     future the print statements
#       March 2, 2023           bar     use tzlib for pickling
#       March 4, 2023           bar     get rid of unused cmp def
#       April 13, 2023          bar     some py3.10 things
#       April 25, 2023          bar     spin code to tz_gps.set_tracks_names_from_labels()
#                                       spin code to tz_gps.add_tracks_metric_descriptions()
#                                       sparsify only when told to do so
#                                       modernize argv
#       June 3, 2023            bar     fix_unpickle() rtn where it's notably needed
#       July 22, 2023           bar     --zap
#       July 23, 2023           bar     another helpful verbosity-ness
#       August 8, 2023          bar     log()
#       September 16, 2023      bar     print some hike counts when verbose
#                                       code cmd line handling of file_max_speed, but don't do it
#       --eodstamps--
##      \file
#       \namespace              tzpython.hikify
#
#
#       TODO:
#
#           implement --file_max_speed by putting the extraction call in a per-input-file loop
#           bikify/hikify settings should be able to have separate uphill/downhill/flat values (or calculate them from the exact angle in some way)
#           --include lat lon               to make sure the point is included in hikes - that point and all neighboring, normally-excluded points (for small segments that are too fast)
#           --join_points lat lon lat lon   when two big-points near these two locations are at the ends of two tracks, join the tracks
#           Instead of doing dumb, time-value based merge, align the tracks in case their clocks are slightly off,
#             and interpolate points when merging them in,
#             in case the merged-to track doesn't have points for each second/time-tick of the merging track's points.
#             Test with roam and the two Canmores together to see that hikes don't have squiggles that can double the milage.
#
#           Default to --first_time or --last_time (I seem to use --first_time, though the latter seems better)
#
#           Loses the first, uphill track in split Poo Poo point (_01) hike if the hike is passed through again. (sparsified)
#
#           Sparsify is too aggressive. Strips significant number of points from tracks already sparsified.
#
#           Option to set the output file's explicit name, as a last resort before using the track name, the geoname of the first/furthest-away/both points.
#
#           Test the include/exclude/filter-value options i've not used (all but --exclude_near)
#
#           Use labels or whatever to combine tracks that are same place in same split files.
#           Get t.name in to combined tracks.
#
#           output gpx file(s)
#               update a .ini file with gpx file info for thumbnail_htm.py use
#               one track per file allowed if .ini files can't have sub-sections (i forget)
#               [file_name]
#                   date_time=
#                   md5=
#                   found=(not) date/time
#
#                   north=
#                   south=
#                   east=
#                   west=
#                   high=
#                   low=
#                   start_time=
#                   end_time=
#                   tags= word word ...
#               and/or put this stuff at the top of the gpx file in custom fields
#
#

from    __future__  import  print_function

import  copy
import  sys
import  os
import  re
import  time

import  tz_gps
import  latlon
import  tz_parse_time


try :
    xrange
except NameError :
    xrange  = range


help_str    = """
Extract hike tracks from .gpx or .nmea track files.

python  hikify.py   (options)   ambiguous_input_files.gpx_nmea   output_file_name.gpx_kml_kmz_nmea

Options:
    --strip_spaces                          Strip leading spaces in XML type output files to make them smaller.
    --show_info                             Print extra debugging information.
    --quiet                                 Do not print the file names.
    --verbose                               Add one to verbosity (for informative printouts).

    --split                                 Output each track to separate files using labels and etc. (numbered _0000...####)
    --creator        name                   Set the file/tracks creator name to the given name.    (default: "Tranzoa, Co.")

    --first_time                            Set --split files' date/time to first point's.
    --last_time                             Set --split files' date/time to last  point's.

    --cache          file_name              Read/write cache of previous computations from given file.
    --recompute                             Recompute the hikes from files the cache says have hikes. (Do no use cached hikes.)

  Filtering:
    --forget_file    "regx"                 Forget from cache and don't do hikes from matching files.
    --unforget_file  "regx"                 Never mind, don't --forget_file these matching files.

    --file_since     date/time              Only input files dated since the given date/time  (default: year 1970).
    --file_before    date/time              Only input files dated before the given date/time (default: year 2038).

    --exclude        latlon latlon          Ignore any hike with a point inside the given NW/SE or NE/SW rectangle.
    --exclude_near   latlon nautical_miles  Ignore any hike with a point within the given distance to the given point.
    --include        latlon latlon          Only output hikes with a point inside any such given NW/SE or NE/SW rectangle.
    --include_near   latlon nautical_miles  Only output hikes with a point near any such given point.
    --snip           latlon latlon          Split tracks in to separate tracks at points inside any such given NW/SE or NE/SW rectangle.
    --snip_near      latlon nautical_miles  Split tracks in to separate tracks at points near any such given point.

    --zap            from_when to_when      Don't include points inside these date/times in any hike.
                                               Note: The from or to time may be a duration under 10 days.

    Unimplemented:
    --file_max_speed    "regx"  kph         Set a max_speed for matching file(s).
    --paz   "regx"   from_when to_when      Include points from matching input files, no matter their speed, inside these date/times.
    --start_point    latlon                 Points before first point near (--ends_distance) this point are stripped from output tracks..
    --end_point      latlon                 Points after  last  point near (--ends_distance) this point are stripped from output tracks..
                                            If only one start_ end_ point is given, both points are that point.
                                            If only one of these points is in the output track, the shortest end is clipped.

  Output Processing:
    --output         file_name              Set the output file name (instead of the last name on cmd line).
    --smooth                                Smooth the output tracks (default)
    --rough                                 Do not smooth output tracks.
    --no_times                              Strip all times, speeds, durations and heart rates from output.
    --label          latlon latlon label    Add label to tracks containing a point inside given NW/SW or NE/SW rectangle.
    --label_near     latlon NMI    label    Add label to tracks containing a point within given nautical miles from given point.
    --merge                                 Merge points recorded at same time/place.
    --combine                               Merge points so there and back become one track. (implies --smooth and --no_times).
    --sparsify                              Take out points that are midway between surrounding points.
    --join                                  Join tracks whose beginnings and ends match in time and space.
                                            (Uses --ends_distance and --max_sleep settings.)

  Hikify Filter Values (Note: These apply to 'big_points' not to the detailed points.):
    --bike                                  Use bikify settings.
    --bikify_file       "regx"              Use --bike filtering for matching files.
    --eot_strip_file    "regx"  kph         For matching files, strip contiguous end points as fast or faster than given speed.
    --ends_distance     meters              End-point resolution detection (--smooth to 1/4 of this value). default:  40 meters
    --min_distance      meters              Hike must go at least this far from starting point this far.    default: 450 or 1200 meters
    --max_jump          meters              Track cannot have more than this distance between points        default: unlimited
    --min_duration      hh:mm:ss            Hike must be for at least this long in time.                    default: 15 minutes
    --max_sleep         hh:mm:ss            Track cannot have more than this time between points.           default:  3 minutes
    --max_speed         kilometers_per_hour Hike cannot have speeds faster than this.                       default: 15 or 47 kph
    --cutoff_distance   meters              Used for low level filtering and for sparsify.                  default: --ends_distance/4


Any number of --exclude... and --include... options may be given.

If any --include... point or rectangle is given,
only hikes with at least one point inside any of the --include... areas are output.

e.g. python hikify.py --include_near "47.0473, -121.4594" 0.9 --label_near "47.0473, -121.4594" 0.9 "Lost Lake" *.gpx i.kmz

"""


def log(s)      :
    if  False   :
        fo      = open("zap.tmp", "a")
        fo.write(s)
        fo.write('\n')
        fo.close()
    pass



def include_exclude(hikes, includes, excludes, verbose = 0) :
    """     Strip points and tracks from the given hikes as a function of the include and exclude lists.    """

    delcnt  = 0

    for ti in xrange(len(hikes) - 1, -1, -1) :

        t   = hikes[ti]

        if  not t.points :

            del(hikes[ti])

        else :

            #
            #                       First, whack any tracks that have an excluded point
            #
            for p in t.points :
                for r in excludes :
                    if  r.is_inside(p) :
                        del(hikes[ti])
                        delcnt += 1
                        if  verbose :
                            print("Excluding point", p, "which is inside exclude", r)
                        t   = None
                        break
                    pass
                if  not t :
                    break
                pass

            #
            #                       Then, whack any tracks with no points in the 'includes' list (if there are any points in the 'includes' list).
            #
            if  t and includes :
                ok  = False
                for p in t.points :
                    for r in includes :
                        if  r.is_inside(p) :
                            ok  = True
                            break
                        pass
                    pass
                if  not ok :
                    del(hikes[ti])
                    t       = None
                    delcnt += 1
                    if  verbose :
                        print("Excluding point that is not inside includes", p)
                    pass
                pass
            pass
        pass

    return(delcnt)




def snip_tracks(tracks, snips, verbose = 0) :
    """     Break the tracks at the given snip points. Count on --join to put 'em back together again. """

    if  len(snips) :
        tracks  = tz_gps.make_array_of_tracks(tracks)

        tlen    = len(tracks)
        for ti in xrange(tlen) :

            t   = tracks[ti]

            si  = 0
            pi  = 0
            while pi < len(t.points) :
                p   = t.points[pi]
                for r in snips :
                    if  r.is_inside(p) :
                        ppi     = pi
                        tracks.append(tz_gps.a_track(t.points[si:ppi], id_num = t.id_num, name = t.name, description = t.description, file_name = t.file_name, color = t.color, opacity = t._opacity))
                        while pi < len(t.points) :              # skip over any points near the snip point
                            if  not r.is_inside(t.points[pi]) :
                                break
                            pi += 1
                        if  verbose :
                            print("Snipping %u points off %s at %s, saving %u points at track point %s" % ( pi - ppi, t.file_name or "", r, ppi - si, p, ))
                        si      = pi                            # the next track we snip off will start here
                        break
                    pass
                pi             += 1
            if  verbose and (pi - si != len(t.points)) :
                print("After snip(s), %u remaining points in %s" % ( pi - si, t.file_name or "", ))
            t.points            = t.points[si:pi]               # we've snipped off tracks, so change the existing track to whatever is left outside the snip points

        tracks  = tz_gps.make_array_of_tracks(tracks)           # whack any empty tracks

    return(tracks)




def clip_at_start_end_point(tracks, distance, start_point = None, end_point = None) :
    start_point = start_point or end_point
    end_point   = end_point   or start_point

    if  start_point and end_point :
        tracks  = tz_gps.make_array_of_tracks(tracks)

        for t in tracks :
            si          = 0
            for pi in xrange(len(t.points)) :
                if  t.points[pi].flat_distance_from(start_point) < distance :
                    while pi < len(t.points) :
                        if  t.points[pi].flat_distance_from(start_point) >= distance :
                            break
                        pi  += 1
                    si  = pi - 1
                    break
                pass

            ei          = len(t.points)
            for pi in xrange(len(t.points) - 1, -1, -1) :
                if  t.points[pi].flat_distance_from(end_point) < distance :
                    while pi >= 0 :
                        if  t.points[pi].flat_distance_from(end_point) >= distance :
                            break
                        pi  -= 1
                    ei  = pi + 1
                    break
                pass

            if  si or (ei < len(t.points)) :
                #
                #   Things to handle !!!!
                #       Only one point is found     (clip the shorter end of the hike)
                #       There are points, but no real hike points, between si and ei (clip the shorter of len(t.points)-si (from the tail) or ei (from the beginning))
                #       After clipping, the hike becomes too short, too brief, or doesn't get far enough away from the starting point (whack the hike)
                #

                if  si  < ei :
                    t.points    = t.points[si:ei]
                pass
            pass
        tracks      = tz_gps.make_array_of_tracks(tracks)

    return(tracks)




def zap_zaps(tracks, zaps) :
    i       = len(tracks) - 1
    while   i >= 0 :
        trk = tracks[i]
        i  -= 1
        if  len(trk.points) :
            for zf, zt in zaps :
                if  (trk.points[0].when <= zt) and (trk.points[-1].when >= zf) :                        # is this zap inside this track?
                    trk1    = tz_gps.a_track([ p for p in trk.points if p.when < zf ], name = trk.name, description = trk.description, file_name = trk.file_name, color = trk.color_gbr_val(), opacity = trk.get_opacity())
                    trk2    = tz_gps.a_track([ p for p in trk.points if p.when > zt ], name = trk.name, description = trk.description, file_name = trk.file_name, color = trk.color_gbr_val(), opacity = trk.get_opacity())
                    i      += 1
                    ln      = len(tracks[i].points)
                    del(tracks[i])
                    tracks.insert(i, trk1)
                    i      += 1
                    tracks.insert(i, trk2)
                    log("zapped %6u points from %s through %s in %s" % (
                                                                        ln - len(trk1.points) - len(trk2.points),
                                                                        time.asctime(time.gmtime(zf)),
                                                                        time.asctime(time.gmtime(zt)),
                                                                        trk1.file_name or trk1.name or "",
                                                                       )
                       )
                    log("              hole   from %s to      %s"    % (
                                                                        (len(trk1.points) and time.asctime(time.gmtime(trk1.points[-1].when))) or (" " * 24),
                                                                        (len(trk2.points) and time.asctime(time.gmtime(trk2.points[ 0].when))) or (" " * 24),
                                                                       )
                       )
                    break
                pass
            pass
        pass
    tracks  = [ t for t in tracks if len(t.points) ]
    for i, trk in enumerate(tracks) :
        trk.id_num  = i

    return(tracks)



class   a_cache_item(object) :
    def __init__(me, fn, crc = 0, hikes = None) :
        me.fn       = os.path.splitext(os.path.basename(fn))[0]
        me.fwhen    = os.path.getmtime(fn)
        me.fsize    = os.path.getsize(fn)
        me.crc      = None
        me.hikes    = hikes or None             # None is unknown. [] means there are no hikes in this input file
        me.settings = None

    def fix_unpickle(me)    :
        if  me.settings     :
            me.settings.fix_unpickle()
        pass

    def forget(me)          :
        me.hikes            = None

    def clear(me)           :
        me.hikes            = []

    def append(me, hike)    :
        if  hike            :
            me.hikes        = me.hikes or []
            me.hikes.append(hike)
        pass

    def __str__(me)         :
        return("%s %d %d 0x%08x %s" % ( me.fn, int(me.fwhen), me.fsize, me.crc or 0, ((me.hikes == None) and "Unk") or str(len(me.hikes)) ) )

    pass            # a_cache_item



def is_in_regx_list(ra, fn) :
    if  len(ra) :
        if  hasattr(ra[0], 'search') :              # compiled regexs in this list, probably
            for r in ra :
                if  r.search(fn) :
                    return(r)
                pass
            pass
        else :
            for r in ra :
                if  re.search(r, fn) :              # uncompiled regex strings in the list
                    return(r)
                pass
            pass
        pass

    return(None)




def _chk_tracks(trks) :
    trks.sort(key = lambda a : a.file_name)
    for ti in range(len(trks)) :
        try :
            a_cache_item(trks[ti].file_name)
        except OSError :
            print("--------------------------------------------------------------")
            print("     " + trks[ti - 1].file_name)
            print("     ... No file name here...")
            print("     " + trks[ti + 1].file_name)
            print("--------------------------------------------------------------")
            raise
        pass
    pass


if  __name__ == '__main__' :

    import  glob

    import  replace_file
    import  TZCommandLineAtFile
    import  tzlib


    argv            = sys.argv[1:]
    TZCommandLineAtFile.expand_at_sign_command_line_files(argv)


    strip_spaces    = False
    settings        = tz_gps.a_track_segmenter_settings().hike()
    bsettings       = tz_gps.a_track_segmenter_settings().bike()
    file_max_speeds = {}                    # key = regexs_to_match_file_name  value = max_speed_in_kph
    bikify_files    = []
    eot_strip_files = {}                    # key = regexs_to_match_file_name  value = speed        we strip the end points from matching hikes/bikes as fast or faster than the speed in kph
    ofile_name      = None
    smooth          = True
    sparse          = False
    show_info       = False
    quiet           = False
    verbose         = 0
    cutoff_distance = None
    no_times        = False
    merge           = False
    combine         = False
    join_tracks     = False
    excludes        = []
    includes        = []
    zaps            = []
    snips           = []
    labels          = []
    split           = False
    map_them        = False
    creator         = "Tranzoa, Co."
    file_time       = 0
    start_point     = None
    end_point       = None
    recompute       = False
    cache_file_name = None
    cache           = []                    # array of a_cache_item's
    forget_files    = []
    unforget_files  = []

    file_before     = float(0x7fffFFFF)
    file_since      = float(0)


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--help", "-h", "-?", "/h", "/H", "/?" ] )
        if  oi < 0 :    break
        print(help_str)
        sys.exit(254)


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--strip_spaces" ] )
        if  oi < 0 :    break
        strip_spaces    = True


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--output", "-o", ] )
        if  oi < 0 :    break
        ofile_name      = argv.pop(oi)


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--no_times" ] )
        if  oi < 0 :    break
        no_times        = True


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--split" ] )
        if  oi < 0 :    break
        split           = True


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--first_time" ] )
        if  oi < 0 :    break
        file_time       = 1

    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--last_time" ] )
        if  oi < 0 :    break
        file_time       = -1


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--cache_file", "--cache", ] )
        if  oi < 0 :    break
        cache_file_name = argv.pop(oi)
        if  os.path.isfile(cache_file_name) :
            cache      += tzlib.unpickle_file(cache_file_name)
        for c in cache  :
            c.settings  = getattr(c, 'settings', None)
        pass

    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--recompute" ] )
        if  oi < 0 :    break
        recompute       = True


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--creator" ] )
        if  oi < 0 :    break
        creator         = argv.pop(oi)


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--show_info" ] )
        if  oi < 0 :    break
        show_info       = True

    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--quiet" ] )
        if  oi < 0 :    break
        quiet           = True

    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--verbose", "-v", ] )
        if  oi < 0 :    break
        verbose        += 1


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--sparse", "--sparsify", ] )
        if  oi < 0 :    break
        sparse          = True


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--bike", "--bikify", "-b", ] )
        if  oi < 0 :    break
        settings        = bsettings


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--cutoff_distance", "-d"] )
        if  oi < 0 :    break
        cutoff_distance = float(argv.pop(oi)) / latlon.metersPerNauticalMile


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--smooth" ] )
        if  oi < 0 :    break
        smooth          = True


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--rough", "-r" ] )
        if  oi < 0 :    break
        smooth          = False


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--combine", "-c" ] )
        if  oi < 0 :    break
        combine         = True
        smooth          = True
        no_times        = True


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--merge", ] )
        if  oi < 0 :    break
        merge           = True


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--join", ] )
        if  oi < 0 :    break
        join_tracks     = True


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--map_them", ] )
        if  oi < 0 :    break
        combine         = True
        smooth          = True
        no_times        = True
        map_them        = True


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--ends_distance", "-e" ] )
        if  oi < 0 :    break
        settings.ends_distance  = float(argv.pop(oi)) / latlon.metersPerNauticalMile
        bsettings.ends_distance = settings.ends_distance


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--min_distance", "-d" ] )
        if  oi < 0 :    break
        settings.min_distance   = float(argv.pop(oi)) / latlon.metersPerNauticalMile
        bsettings.min_distance  = settings.min_distance

    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--max_jump", "-j" ] )
        if  oi < 0 :    break
        settings.max_jump       = float(argv.pop(oi)) / latlon.metersPerNauticalMile
        bsettings.max_jump      = settings.max_jump


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--min_duration", "-t" ] )
        if  oi < 0 :    break
        settings.min_duration   = tz_parse_time.parse_time_zone(argv.pop(oi))
        if  settings.min_duration == None :
            print("I cannot understand the minimum duration!")
            sys.exit(102)
        bsettings.min_duration  = settings.min_duration

    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--max_sleep", ] )
        if  oi < 0 :    break
        settings.max_sleep      = tz_parse_time.parse_time_zone(argv.pop(oi))
        if  settings.max_sleep == None :
            print("I cannot understand what the maximum lack-of-points time is!")
            sys.exit(102)
        bsettings.max_sleep     = settings.max_sleep

    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--max_speed", "-s" ] )
        if  oi < 0 :    break
        settings.max_speed      = float(argv.pop(oi))
        bsettings.max_speed     = settings.max_speed

    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--file_max_speed", ] )
        if  oi < 0 :    break
        rx                      = argv.pop(oi)
        sp                      = float(argv.pop(oi))
        file_max_speeds[rx]     = sp
        if  re.search(r'\.[a-zA-Z0-9]*$', rx) :
            print("I won't --eot_strip_files that appear to have file name extensions [%s]!" % rx)
            sys.exit(101)
        pass


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--exclude" ] )
        if  oi < 0 :    break
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand exclude of %s" % s)
            sys.exit(103)
        p   = tz_gps.a_rectangle(lat, lon)
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand exclude lat/lon of %s" % s)
            sys.exit(103)
        p.include_latlon(lat, lon)
        excludes.append(p)


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--include" ] )
        if  oi < 0 :    break
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand include of %s" % s)
            sys.exit(103)
        p   = tz_gps.a_rectangle(lat, lon)
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand include lat/lon of %s" % s)
            sys.exit(103)
        p.include_latlon(lat, lon)
        includes.append(p)


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--snip" ] )
        if  oi < 0 :    break
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand snip of %s" % s)
            sys.exit(103)
        p   = tz_gps.a_rectangle(lat, lon)
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand snip lat/lon of %s" % s)
            sys.exit(103)
        p.include_latlon(lat, lon)
        snips.append(p)


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--exclude_near", "-x" ] )
        if  oi < 0 :    break
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand exclude_near of %s" % s)
            sys.exit(103)
        p   = tz_gps.a_circle(lat, lon, float(argv.pop(oi)))
        excludes.append(p)


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--include_near", "-i" ] )
        if  oi < 0 :    break
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand include_near of %s" % s)
            sys.exit(103)
        p   = tz_gps.a_circle(lat, lon, float(argv.pop(oi)))
        includes.append(p)


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--snip_near" ] )
        if  oi < 0 :    break
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand snip_near of %s" % s)
            sys.exit(103)
        p   = tz_gps.a_circle(lat, lon, float(argv.pop(oi)))
        snips.append(p)



    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--zap", "-z" ] )
        if  oi < 0 :    break
        zts = argv.pop(oi)
        zf = tz_parse_time.parse_time(zts)
        if  zf is None :
            zf  = tz_parse_time.parse_time_of_day(zts)
            if  zf is None :
                print('I cannot understand the --zap of "%s" !' % zts)
                sys.exit(102)
            pass
        zts = argv.pop(oi)
        zt  = tz_parse_time.parse_time(zts)
        if  zt is None :
            zt  = tz_parse_time.parse_time_of_day(zts)
            if  zt is None :
                print('I cannot understand the --zap of "%s" !' % zts)
                sys.exit(102)
            pass
        max_zap_duration    = 20 * 24 * 60 * 60
        azf = abs(zf)
        azt = abs(zt)
        if  azf + azt < max_zap_duration :
            print('I cannot understand --zap of two durations!')
            sys.exit(102)
        if  azf     < max_zap_duration :
            zf      = zt - zf
        if  azt     < max_zap_duration :
            zt     += zf                        # now zf and zt are from and to times
        if  zf > zt :
            zf, zt  = zt, zf
        zaps.append(( zf, zt, ))


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--start_point" ] )
        if  oi < 0 :    break
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand start_point of %s" % s)
            sys.exit(103)
        start_point             = tz_gps.a_point(lat = lat, lon = lon)


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--end_point" ] )
        if  oi < 0 :    break
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand end_point of %s" % s)
            sys.exit(103)
        end_point               = tz_gps.a_point(lat = lat, lon = lon)




    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--label" ] )
        if  oi < 0 :    break
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand label of %s" % s)
            sys.exit(103)
        p                       = tz_gps.a_rectangle(lat, lon)
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand label lat/lon of %s" % s)
            sys.exit(103)
        p.include_latlon(lat, lon)
        p.name                  = argv.pop(oi).strip()
        labels.append(p)


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--label_near", "-l" ] )
        if  oi < 0 :    break
        s                       = argv.pop(oi)
        ( lat, lon )            = latlon.parse_lat_lon(s)
        if  (lat == None) or (lon == None) :
            print("I cannot understand label_near of %s" % s)
            sys.exit(103)
        p                       = tz_gps.a_circle(lat, lon, float(argv.pop(oi)))
        p.name                  = argv.pop(oi).strip()
        labels.append(p)



    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--file_since" ] )
        if  oi < 0 :    break
        file_since              = tz_parse_time.parse_time(argv.pop(oi))
        if  file_since         == None :
            print("I cannot understand the file_since!")
            sys.exit(102)
        pass


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--file_before" ] )
        if  oi < 0 :    break
        file_before             = tz_parse_time.parse_time(argv.pop(oi))
        if  file_before        == None :
            print("I cannot understand the file_before!")
            sys.exit(102)
        pass


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--forget_file", "--forget_files", "--forget", ] )
        if  oi < 0 :    break
        forget_files.append(argv.pop(oi))

    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--unforget_file", "--unforget_files", "--unforget", ] )
        if  oi < 0 :    break
        unforget_files.append(argv.pop(oi))


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--list_settings" ] )
        if  oi < 0 :    break
        hs  = tz_gps.a_track_segmenter_settings().hike()
        bs  = tz_gps.a_track_segmenter_settings().bike()
        cache.sort(key = lambda a : ( a.fwhen, a.fsize, ))
        for c in cache  :
            s   = "hike"
            if  c.settings != hs :
                if  c.settings == bs :
                    s   = "Bike"
                elif c.settings != None :
                    s   = "Path"
                else    :
                    pc  = 0
                    ct  = 0.0
                    for trk in c.hikes :
                        pa  = list(trk.points)
                        tz_gps.fix_points_speeds(pa)
                        pc += len(pa)
                        ct += len([ pnt for pnt in pa if pnt.speed > hs.max_speed + 10 ])
                    if  pc and (ct / pc >= 0.01) :
                        s   = "BIKE"
                    pass
                pass
            if  s[0] != 'h' :
                print(s, "Hikes:", len(c.hikes), "in", os.path.basename(c.fn))
            pass
        sys.exit(0)


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--bikify_file", "--bikify_files", ] )
        if  oi < 0 :    break
        bikify_files.append(argv.pop(oi))
        if  re.search(r'\.[a-zA-Z0-9]*$', bikify_files[-1]) :
            print("I won't --bikify_files that appear to have file name extensions [%s]!" % bikify_files[-1])
            sys.exit(101)
        pass


    while True :
        oi  = tzlib.find_argi_and_del(argv, [ "--end_strip_speed", "--eot_strip_files", "--eot_strip_file", ] )
        if  oi < 0 :    break
        rx  = argv.pop(oi)
        sp  = float(argv.pop(oi))
        eot_strip_files[rx] = sp
        if  re.search(r'\.[a-zA-Z0-9]*$', rx) :
            print("I won't --eot_strip_files that appear to have file name extensions [%s]!" % rx)
            sys.exit(101)
        pass


    if  verbose > 3 :
        print("Regexs for forgotten files:")
        ffls        = list(forget_files)
        ffls.sort(key = lambda fn : fn.upper())
        for fn in ffls :
            print("    %s" % fn)
        print('')
        if  False :
            c   = a_cache_item('mv_post_office_real_summit_bike_trails_safeway_01y.gpx')        # why did this thing get filtered from the cache when I finally got the correct forget_files logic in here? April 30, 2021
            print(is_in_regx_list(forget_files, c.fn), is_in_regx_list(ffls, c.fn))
            sys.exit(1) # @@@@
        pass
    forget_files    = [ re.compile(r) for r in   forget_files ]         # speed things up
    unforget_files  = [ re.compile(r) for r in unforget_files ]

    c_by_file_name  = {}
    c_by_crc        = {}
    for c in cache  :
        if  (not is_in_regx_list(forget_files, c.fn)) or is_in_regx_list(unforget_files, c.fn) :
            if False and (verbose > 3) :
                print("Keep in cache: %s" % c.fn)
            c_by_file_name[c.fn]    = c
            c_by_crc[c.crc]         = c
        elif verbose > 3 :
            print("Forgot from cache: %s" % c.fn)
        if  not c.settings :                                            # note: that we None'd the cache file's settings causes us to update them to the current values - which doesn't really matter if the cache isn't recomputed
            c.settings  = copy.deepcopy(bsettings if is_in_regx_list(bikify_files, c.fn) else settings)
            rx          = is_in_regx_list(list(eot_strip_files.keys()), c.fn)
            if  rx      :
                c.settings.end_strip_speed  = eot_strip_files[rx]       # remember how fast points must be to be stripped from the ends of this hike/bike track
            pass
        pass


    if  cutoff_distance == None :
        cutoff_distance = 5.0 / latlon.metersPerNauticalMile            # (the original sparsify value - which we emulate by dividing cutoff_distance by two below)
        cutoff_distance = settings.ends_distance / 4.0

    labels.sort(key = lambda a : a.name)


    settings.max_jump           = min(settings.max_jump or settings.min_distance, settings.ends_distance * 10.0)
    bsettings.max_jump          = settings.max_jump


    if  not len(argv) :
        print("Must have input .gpx/.nmea file(s)%s!" % (((not ofile_name) and " and output file") or ""))
        sys.exit(101)


    if  not ofile_name :
        ofile_name  = argv.pop(-1)
    odir        = os.path.dirname(ofile_name)
    if  odir and (not os.path.isdir(odir)) :
        os.makedirs(odir)

    fnames      = {}
    while len(argv) :
        fn      = argv.pop(0)
        fns     = glob.glob(fn)
        if  not fns :
            print("Found no file named %s!" % ( fn ))
            sys.exit(101)
        for fn in fns :
            c   = a_cache_item(fn)
            if  (not is_in_regx_list(forget_files, c.fn)) or is_in_regx_list(unforget_files, c.fn) :
                fnames[fn]  = True
            elif verbose > 3 :
                print("Forget (ignored) from cmd line: %s" % fn)
            pass
        pass

    if  not len(fnames) :
        print("Found no input files!", file = sys.stderr)
        sys.exit(101)


    if  os.path.splitext(ofile_name)[1].lower() == '.nmea' :
        if  no_times or (combine and smooth) :
            print("I cannot write NMEA files with no date/times!")
            sys.exit(109)
        pass


    delcnt      = 0
    tracks      = []
    chikes      = []
    fnames      = [ fn for fn in fnames.keys() if file_since <= os.path.getmtime(fn) <= file_before ]
    fnames.sort()

    print("%u files." % len(fnames))

    for fn in fnames :
        c       = a_cache_item(fn)
        if  cache_file_name :
            c   = c_by_file_name.get(c.fn, c)
            c.fix_unpickle()

            if  c.crc      == None :
                crc         = tzlib.blkcrc32(0xffffFFFF, tzlib.read_whole_binary_file(fn))
                c           = c_by_crc.get(crc, c)
                c.crc       = crc
                c.fsize     = os.path.getsize(fn)
                c.fwhen     = os.path.getmtime(fn)
            elif (os.path.getsize(fn) != c.fsize) or (os.path.getmtime(fn) != c.fwhen) :
                crc         = tzlib.blkcrc32(0xffffFFFF, tzlib.read_whole_binary_file(fn))
                if  crc    != c.crc :
                    # print("      %s CRC mismatch. Re-computing hikes." % fn)
                    c.forget()                              # input file has changed so forget what the cache knew about hikes
                else        :
                    # print("      %s CRC match. Using cache: %s." % ( fn, ((c.hikes == None) and "Unk") or str(len(c.hikes)) ))
                    pass
                c.crc       = crc
                c.fsize     = os.path.getsize(fn)
                c.fwhen     = os.path.getmtime(fn)
            else            :
                # print("      %s date/size match. Using cache: %s." % ( fn, ((c.hikes == None) and "Unk") or str(len(c.hikes)) ))
                pass
            pass

        if  c.settings is None  :                                       # note: this is essentially an if the file is not in cache if the code above set the cache files' settings or it left them as they were read from cache
            c.settings  = copy.deepcopy(bsettings if is_in_regx_list(bikify_files, c.fn) else settings)
            rx          = is_in_regx_list(list(eot_strip_files.keys()), c.fn)
            if  rx      :
                c.settings.end_strip_speed  = eot_strip_files[rx]       # remember how fast points must be to be stripped from the ends of this hike/bike track
            pass

        c_by_file_name[c.fn]    = c
        if   cache_file_name and (c.hikes != None) and (not recompute) and (not len(c.hikes)) :
            s               = "File: %s" % fn
            s              += (" " * max(0, 70 - len(s))) + " cache-skipping."
            if  not quiet   :
                print(s)
            pass
        elif cache_file_name and (c.hikes != None) and (not recompute) :
            chikes         += c.hikes
            s               = "File: %s" % fn
            s              += (" " * max(0, 70 - len(s))) + " %u hike%s cached." % ( len(c.hikes), tzlib.s_except_1(c.hikes) )
            if  not quiet   :
                print(s)
            _chk_tracks(c.hikes)
        else                :
            trks            = tz_gps.tracks_from_file(fn)               # read the file
            for t in trks   :
                t.file_name = fn                                        # override any super-duper logic that picks up the original file name from the data ('cause that original name may be from some file somewhere else. Or it may be wrong.)

            if  not trks    :
                print("No tracks in %s!" % fn)
                c.clear()                                               # just in case of what? duped files? yep. But, the cache doesn't like files with the same base name.
            else :
                _chk_tracks(trks)
                if  verbose :
                    print("%u points in hikes." % tz_gps.count_points_in_tracks(trks))
                if  len(zaps)   :
                    trks    = zap_zaps(trks, zaps)
                    if  verbose :
                        print("After %u zaps, %u points in %u hikes." % ( len(zaps), tz_gps.count_points_in_tracks(trks), len(trks), ))
                    _chk_tracks(trks)

                trks        = tz_gps.sorted_tracks_by_when(trks)
                _chk_tracks(trks)
                trks        = tz_gps.remove_duplicate_tracks(trks)
                _chk_tracks(trks)

                hikes       = snip_tracks(trks, snips, verbose = verbose)

                hikes       = tz_gps.extract_track_segments(hikes, c.settings, show_info = show_info)       # this does what we're running for
                _chk_tracks(hikes)
                if  verbose :
                    print("After extract, %u points in %u hikes." % ( tz_gps.count_points_in_tracks(hikes), len(hikes), ))

                hikes       = clip_at_start_end_point(hikes, c.settings.ends_distance, start_point, end_point)
                _chk_tracks(hikes)
                if  verbose :
                    print("After clip start/end, %u points in %u hikes." % ( tz_gps.count_points_in_tracks(hikes), len(hikes), ))

                if  False and len(zaps) :
                    hikes       = zap_zaps(hikes, zaps)
                    if  verbose :
                        print("After %u zaps %u points in %u hikes." % ( len(zaps), tz_gps.count_points_in_tracks(hikes), len(hikes), ))
                    pass

                # print("@@@@", len(hikes), [ "from " + str(h.points[0]) + " to " + str(h.points[-1]) for h in hikes ])

                delcnt     += include_exclude(hikes, includes, excludes, verbose = verbose)

                _chk_tracks(hikes)
                if  verbose :
                    print("After include/exclude, %u points in %u hikes." % ( tz_gps.count_points_in_tracks(hikes), len(hikes), ))

                tracks     += hikes

                s           = "File: %s %u track%s" % ( fn, len(trks), tzlib.s_except_1(trks) )

                if  hikes   :
                    s      += (" " * max(0, 70 - len(s))) + " %u hike%s" % ( len(hikes), tzlib.s_except_1(hikes) )

                s          += "."
                if  not quiet   :
                    print(s)
                pass
            pass
        pass

    chikes  = tz_gps.remove_duplicate_tracks(chikes)

    cnt     = len(tracks)
    tracks  = tz_gps.remove_duplicate_tracks(tracks)
    dupecnt = (cnt - len(tracks))
    delcnt += dupecnt
    if  verbose and dupecnt :
        print("Duplicate tracks removed:", dupecnt)


    _chk_tracks(tracks + chikes)


    if  delcnt :
        print("Excluded %u (snipped?) track%s." % ( delcnt, tzlib.s_except_1(delcnt) ))

    if  not tracks and not chikes :
        print("No tracks remaining.")
    else :
        if  tracks              :
            cnt                 = tz_gps.count_points_in_tracks(tracks)
            print("Processing %u point%s in %u track%s..." % ( cnt, tzlib.s_except_1(cnt), len(tracks), tzlib.s_except_1(tracks) ))

            if  join_tracks     :
                fnd             = True
                while fnd       :
                    fnd         = False
                    deltrks     = []
                    bwhens      = [ ( t.points[ 0], ti ) for ti, t in enumerate(tracks) ]
                    ewhens      = [ ( t.points[-1], ti ) for ti, t in enumerate(tracks) ]
                    for ep, eti in ewhens :
                        for bp, bti in bwhens :
                            td  = bp.start_when - ep.end_when
                            if  (verbose > 3) and (td >= 0) :
                                print("Joining if time from end to beginning of %s is under max_sleep of %s and distance of %s is less than ends_distance %s" % ( td, settings.max_sleep, ep.flat_distance_from(bp), settings.ends_distance, ))
                                if  (verbose > 4) and (td >= 0) :
                                    print("   ", ep)
                                    print("   ", bp)
                                pass
                            if  (td >= 0) and (td < settings.max_sleep) and (ep.flat_distance_from(bp) < settings.ends_distance) :
                                if  verbose > 3 :
                                    print("  joined:")
                                    print("   ", ep)
                                    print("  to:")
                                    print("   ", bp)
                                tracks[eti].append_track(tracks[bti])
                                del(tracks[bti])                                            # note: because fnd is true, we'll do the whole thing from the top, so it's ok that we whack this track
                                fnd = True
                                break
                            pass
                        if  fnd :
                            break
                        pass
                    pass
                if  verbose     > 1 :
                    cnt         = tz_gps.count_points_in_tracks(tracks)
                    print("After join, %u track%s with %u point%s" % ( len(tracks), tzlib.s_except_1(tracks), cnt, tzlib.s_except_1(cnt), ))
                pass

            if  merge           :
                tracks.sort(key = lambda a : a.when)
                ti              = 0
                while ti < len(tracks) - 1 :
                    tti         = ti + 1
                    when        = tracks[ti].points[-1].when                                # the merge-to track's ending time
                    while tti   < len(tracks) :
                        if  when < tracks[tti].when :                                       # run tti past all tracks that start before the end of the ti'th track (remember the tracks are sorted by beginning times)
                            break
                        tti    += 1
                    if  tti     > ti + 1    :
                        pa      = tz_gps.merge_tracks_to_points(tracks[ti : tti])           # get the merged points
                        if  len(pa)         :
                            pa              = tz_gps.only_big_points_from_max_tracks(pa, tti - ti)      # this avoids tiny eddies in the track at the risk of getting rid of all the points
                        if  len(pa)         :
                            trk             = tz_gps.a_track(pa, id_num = tracks[ti].id_num, name = tracks[ti].name, description = tracks[ti].description, file_name = tracks[ti].file_name, color = tracks[ti].color, opacity = tracks[ti]._opacity)
                            tracks[ti:tti]  = [ trk ]                                       # replace the separate tracks with the merged track
                        pass
                    ti         += 1
                if  verbose     > 1 :
                    cnt         = tz_gps.count_points_in_tracks(tracks)
                    print("After merge, %u track%s with %u point%s" % ( len(tracks), tzlib.s_except_1(tracks), cnt, tzlib.s_except_1(cnt), ))
                pass
            # print(" @@@@", len(tracks), len(tracks) and len(tracks[0].points))

            if  combine         :
                cnt             = tz_gps.count_points_in_tracks(tracks)
                tracks          = tz_gps.combine_near_points_in_tracks(tracks, settings.ends_distance / 4.0, map_them, show_info)
                if  map_them    :
                    no_times    = True
                if  smooth      :
                    ncnt        = tz_gps.count_points_in_tracks(tracks)
                    spp         = 2.0
                    if  ncnt    :
                        spp     = max(spp, float(cnt) / float(ncnt))
                    tracks      = tz_gps.set_equal_durations_in_tracks(tracks, spp)
                    tracks      = tz_gps.smooth_tracks(tracks)
                    no_times    = True                                          # since we've whacked the times, we may as well whack 'em good
                if  verbose     > 1 :
                    cnt         = tz_gps.count_points_in_tracks(tracks)
                    print("After combine, %u track%s with %u point%s" % ( len(tracks), tzlib.s_except_1(tracks), cnt, tzlib.s_except_1(cnt), ))
                pass
            elif  smooth        :
                tracks          = tz_gps.make_big_points(tracks, cutoff_distance)
                # print(" @@@@", len(tracks), len(tracks) and len(tracks[0].points), len(tracks) and tracks[0].points[0], len(tracks) and tracks[0].points[-1])
                _chk_tracks(tracks)
                tracks          = tz_gps.smooth_tracks(tracks)
                # print(" @@@@", len(tracks), len(tracks) and len(tracks[0].points), len(tracks) and tracks[0].points[0], len(tracks) and tracks[0].points[-1])

            _chk_tracks(tracks)

            if  no_times        :
                if  os.path.splitext(ofile_name)[1].lower() == '.nmea' :
                    print("I cannot write NMEA files with no date/times!")
                    sys.exit(109)

                tracks          = tz_gps.set_no_whens_in_tracks(tracks)         # must be done after smoothing, as smoothing uses 'when' values for interpolation
                _chk_tracks(tracks)

            tz_gps.set_tracks_names_from_labels(tracks, labels)

            if  sparse      :
                tracks      = tz_gps.remove_redundant_points_from_tracks(tracks, cutoff_distance / 2.0)
                # print(" @@@@", len(tracks), len(tracks) and len(tracks[0].points), len(tracks) and tracks[0].points[0], len(tracks) and tracks[0].points[-1])
                _chk_tracks(tracks)

            tz_gps.add_tracks_metric_descriptions(tracks)

        tracks             += chikes

        cnt                 = tz_gps.count_points_in_tracks(tracks)
        if  not cnt         :
            print("No points/tracks to write.")
        else                :
            print("Writing %u point%s to %u track%s to..." % ( cnt, tzlib.s_except_1(cnt), len(tracks), tzlib.s_except_1(tracks) ), end = ' ')

            if  not split :
                print(ofile_name)
                tz_gps.write_gps_file(ofile_name, "hikify.py", creator, [], tracks, strip_spaces)
            else :
                ands            = ""
                tracks.sort(key = lambda a : a.when)
                ofns            = {}
                for t in tracks :
                    ( fn, ext ) = os.path.splitext(ofile_name)

                    fn         += "_" + tzlib.file_name_able((t.name or "").replace(" ", "_"))          # add the track name to the file name
                    fn          = re.sub(r"_+", "_", fn)
                    ( pth, fn ) = os.path.split(fn)
                    fn          = fn.strip("_")
                    fn          = os.path.join(pth, fn)

                    ofns[fn]    = ofns.get(fn, 0) + 1
                    if  fn :
                        fn     += "_"
                    fn          = replace_file.find_available_N_file_name(fn + replace_file.NNNNN + ext, replace_file.NNNNN, 10, True)

                    print(ands, fn, time.asctime(time.localtime(t.when)))
                    ands        = "  and "
                    tz_gps.write_gps_file(fn,     "hikify.py", creator, [], [ t ],  strip_spaces)

                    if  file_time :
                        p   = t.points[0]
                        if  file_time < 1 :
                            p   = t.points[-1]
                        if  p.when :
                            os.utime(fn, ( os.path.getatime(fn), int(p.when) ) )                        # set the file date/time to that of the 1st point
                        pass
                    pass
                pass
            pass
        pass

    _chk_tracks(tracks)

    if  cache_file_name :
        for c in c_by_file_name.values() :
            c.clear()                                   # forget old tracks for this file - we're going to reset them to however they've changed a couple lines of code down
        for t in tracks :
            c   = c_by_file_name[a_cache_item(t.file_name).fn]
            c.append(t)                                 # add this track to the file's cache hikes. note: the files that were merged in to this one will be cache-skipped in the future
        pass

    if  cache_file_name :
        r   = tzlib.pickle_file(cache_file_name, list(c_by_file_name.values()), tzlib.BEST_PICKLE_PROTOCOL)

    pass


#
#
# eof
