#!/usr/bin/python

# httppost.py
#       --copyright--                   Copyright 2007 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       August 31, 2005         bar
#       November 18, 2007       bar     turn on doxygen
#       November 27, 2007       bar     insert boilerplate copyright
#       May 15, 2008            bar     --show_response
#                                       change to FF 2
#       May 17, 2008            bar     email adr
#       April 11, 2011          bar     add the multipart code snagged from the net
#       May 27, 2012            bar     doxygen namespace
#       August 7, 2012          bar     explicitely call time.time()
#       June 2, 2013            bar     expose the routine to upload to asuka
#       June 3, 2013            bar     check (but not definately) https in upload
#       August 19, 2013         bar     a_bar_uploader
#       November 27, 2013       bar     frienlier unicode bar upload
#       March 10, 2014          bar     handle unicode problems with a zip file being uploaded
#       May 28, 2014            bar     name the thread and get its thread id (htop's PID)
#       March 5, 2015           bar     able to take url to upload_to_bar() and a_bar_uploader()
#       November 15, 2015       bar     do not upload .tmp files
#                                       randomly choose which unsent file to upload
#                                       clean up the uploader code - backoff when no files to upload
#       December 16, 2015       bar     be graceful when the outsider sets our UPLOAD_TIMEOUT behind our back to a long time, but forgets to set UPLOAD_MAX_DELAY to a longer time
#       February 28, 2016       bar     trigger_upload() allows the uploader to be kicked off by an empty file name
#       May 21, 2016            bar     niceties in upload_to_bar()
#       March 5, 2023           bar     future print
#       --eodstamps--
##      \file
#       \namespace              tzpython.httppost
#
#
#       Do a POST for a given URL.
#
#           e.g. http://wap.melodeo.com/verdi/tmestore/download/songnotify?r=25792&t=43162a7b&a=S3EE5HaFY3qauivb74If
#
#

from    __future__  import  print_function

import  cStringIO
import  glob
import  httplib
import  itertools
import  mimetools
import  mimetypes
import  os
import  Queue
import  random
import  socket
import  sys
import  threading
import  time
import  urllib2

import  tzlib


opener            = urllib2.build_opener()
opener.addheaders = []                  # get rid of 'User-agent' the only way that seems to work (yes, I tried lower-casing 'Agent')
urllib2.install_opener(opener)


#
#
#   Code from http://uthcode.sarovar.org/pythonsnippets.html
#   urllib2-binary-upload.py - On 19 Dec,2010.
#
#
class   a_multipart_form(object):
    """Accumulate the data to be used when posting a form."""

    def __init__(me)    :
        me.form_fields  = []
        me.files        = []
        me.boundary     = mimetools.choose_boundary()

    def get_content_type(me) :
        return('multipart/form-data; boundary=%s' % me.boundary)

    def add_field(me, name, value) :
        """Add a simple field to the form data."""
        me.form_fields.append((name, value))

    def add_file_data(me, fieldname, filename, file_data, mimetype = None) :
        """Add file data to be uploaded."""
        mimetype        = mimetype or 'application/octet-stream'
        me.files.append((fieldname, filename, mimetype, file_data))

    def add_file(me, fieldname, filename, fileHandle, mimetype = None) :
        """Add a file to be uploaded."""
        body            = fileHandle.read()
        if  mimetype   is None :
            mimetype    = mimetypes.guess_type(filename)[0] or 'application/octet-stream'
        me.add_file_data(fieldname, filename, body, mimetype)

    def __str__(me) :
        """Return a string representing the form data, including attached files."""
        # Build a list of lists, each containing "lines" of the
        # request.  Each part is separated by a boundary string.
        # Once the list is built, return a string where each
        # line is separated by '\r\n'.
        parts           = []
        part_boundary   = '--' + me.boundary

        # Add the form fields
        parts.extend(
                        [
                            part_boundary,
                            'Content-Disposition: form-data; name="%s"' % name,
                            '',
                            value,
                        ]
                        for name, value in me.form_fields
                    )
        # Add the files to upload
        parts.extend(
                        [
                            part_boundary,
                            'Content-Disposition: file; name="%s"; filename="%s"' % \
                               (field_name, filename),
                            'Content-Type: %s' % content_type,
                            '',
                            body,
                        ]
                        for field_name, filename, content_type, body in me.files
                    )

        # Flatten the list and add closing boundary marker,
        # then return CR+LF separated data
        flattened   = list(itertools.chain(*parts))
        flattened.append('--' + me.boundary + '--')
        flattened.append('')
        try :
            return('\r\n'.join(flattened))
        except UnicodeDecodeError :
            pass
        return((u'\r\n'.join([ f.decode('latin1') for f in flattened ])).encode('latin1'))

    #   a_multipart_form


UPLOAD_URL  =   'https://www.tranzoa.net/~alex/cgi-bin/upload.pl'


def upload_to_bar(file_name, file_data_or_open_file = None, url = None, verbose = 0) :
    """ Upload a file to asuka/tranzoa.net. Return the request response or None if the upload failed. """

    #       Create the form with simple fields
    form    = a_multipart_form()

    #       Add a fake file
    #       note: upload.pl expects to see this 'mode: upload' field and the field name for the file must be 'upload_file'
    form.add_field('mode', 'upload')

    ofh     = None
    fh      = file_data_or_open_file
    if  not hasattr(fh, 'read') :
        if  fh is None :
            fh  = open(tzlib.expand_user_vars(file_name), 'rb')
            ofh = fh
        else    :
            if  type(fh) == type(u'') :
                fh  = fh.encode('utf8')
            fh  = cStringIO.StringIO(fh)
        pass
    fw          = fh.tell()

    form.add_file('upload_file', os.path.basename(file_name or 'httppost_py_upload.dat'), fileHandle = fh)

    # Build the request
    request = urllib2.Request(url or UPLOAD_URL)
    request.add_header('User-agent',    'tzlib.httppost.a_multipart_form')
    body    = str(form)
    request.add_header('Content-type',   form.get_content_type())
    request.add_header('Content-length', len(body))
    request.add_data(body)

    if  verbose > 1 :
        print('')
        print('OUTGOING DATA:')
        print(repr(request.get_data()))

    if  verbose :
        print('')
        print('SERVER RESPONSE:')
    try :
        r   = urllib2.urlopen(request)
    except ( socket.error, socket.herror, socket.gaierror, OSError, IOError, MemoryError, AttributeError, urllib2.URLError, urllib2.HTTPError, httplib.HTTPException, ValueError, ) :
        r   = None
    if  verbose :
        if  r :
            print(r.geturl(), "headers", r.headers.keys(), "code:", r.getcode())
        else :
            print("Upload failed!")
        pass

    if  ofh :
        ofh.close()
        del(ofh)
    else :
        fh.seek(fw)

    return(r)



if  False :
    print(upload_to_bar('bio.txt', cStringIO.StringIO('Python developer and blogger. %s' % time.asctime(time.localtime(time.time()))), verbose = 2) != None)
    print(upload_to_bar('./httppost.py') != None)
    print(upload_to_bar('httppost.py', "blah, lbah" + u" with unicode  ", verbose = 2) != None)
    exit(1)



UPLOAD_TIMEOUT      = 3.5                               #: shortest wait between send attempts
UPLOAD_BACKOFF_MULT = 1.2                               #: how much to back off on failed attempts or when idle
UPLOAD_MAX_DELAY    = 60.0 * 7.0                        #: maximum back-off duration

class   a_bar_uploader(threading.Thread) :
    """ Class to make something that can send files to bar - upload them to a properly decked out server. """

    def __init__(me, dir_name = None, url = None, idle_backoff_mult = 0, *args, **kwargs) :
        """ Make a BAR uploaded, and queue things in files in the given directory. """

        me.dir_name = dir_name or os.path.join(os.path.dirname(tzlib.get_ini_or_cfg_file_name("upload.ini", app_name = 'bar_uploader')), 'feedback')
        if  not os.path.exists(me.dir_name) :
            os.makedirs(me.dir_name)

        me.url              = url                           #: url to hit
        me.idle_mult        = idle_backoff_mult or 0        #: (0 ... 1] means don't back off looking in the directory when there's nothing to send. 0 means back off in the same way as when the upload failed

        if  len(args) >= 3  :
            args[2]         = args[2]     or     me.__class__.__name__
        else                :
            kwargs['name']  = kwargs.get('name', me.__class__.__name__)
        me.tid              = None
        super(a_bar_uploader, me).__init__(*args, **kwargs)

        me.q        = Queue.Queue()
        me.setDaemon(True)                              #: allow our thread to be zapped at shutdown time


    def run(me) :
        """ Run the thread to get everything uploaded, eventually. """
        me.tid              = tzlib.get_tid()

        tmout               = UPLOAD_TIMEOUT
        while True          :
            q               = me.q
            if  not q       :
                break

            try             :

                fn          = ""
                try         :
                    tmout   = min(max(UPLOAD_MAX_DELAY, UPLOAD_TIMEOUT), tmout)
                    fn_fd   = q.get(timeout = tmout)
                    if  fn_fd is None :
                        break                               # None in queue tells us to stop

                    fn, fd          = fn_fd
                    if  fn          :                       # allow an empty file name string to kick us off
                        fn          = os.path.join(me.dir_name, os.path.basename(fn))
                        bfn, ext    = os.path.splitext(fn)
                        while os.path.exists(fn) :
                            fn      = bfn + '_' + str(random.randint(0, 999999)) + ext
                        tzlib.safe_write_whole_binary_file(fn, fd)
                    tmout           = UPLOAD_TIMEOUT        # start over, seeing as how the human is probably driving this and we don't want him to wait
                except Queue.Empty :
                    pass                                    # nothing in queue, let's just check for old stuff that hasn't gone out yet

                if  not fn      :                                                                                           # if we've not just been told something to upload,
                    todo        = [ ffn for ffn in glob.glob(os.path.join(me.dir_name, '*')) if os.path.isfile(ffn) ]       #    find a random file in the directory to send
                    todo        = [ ffn for ffn in todo if not ( ffn.startswith(".") or ffn.lower().endswith("~") or os.path.splitext(ffn)[1].lower() in [ ".tmp", ".bak", ] ) ]
                    if  len(todo) :
                        fn      = random.choice(todo)
                    pass

                if  fn          :
                    fd          = tzlib.safe_read_whole_binary_file(fn) or ''                                               # if there's a read error, we'll upload a zero length file and whack the file
                    r           = upload_to_bar(os.path.basename(fn), file_data_or_open_file = fd, url = me.url)
                    if  r       :
                        tmout   = UPLOAD_TIMEOUT
                        tzlib.whack_file(fn)
                    else        :
                        tmout  *= UPLOAD_BACKOFF_MULT                                                                       # the upload failed - the net's probably down or disconnected - let's not be obnoxious
                    pass
                else            :
                    tmout      *= (me.idle_mult and max(1.0, me.upload_backoff_mult)) or UPLOAD_BACKOFF_MULT                # we've had nothing to do, let's back off hitting the directory - for our purposes, a few minute delay won't hurt anything
                pass

            except KeyboardInterrupt :
                raise

            except Exception    :
                tzlib.print_exception()
                tmout          *= UPLOAD_BACKOFF_MULT                                                                       # catastrophic failure. Back off.

            pass
        pass


    def send_file(me, fn, fd) :
        """ Queue up a file or feedback message for later sending. It failed to send now, probably (if the sender wants to give his user feedback. """
        q   = me.q
        if  q :
            q.put([ fn, fd])
        pass


    def trigger_upload(me) :
        """ Trigger any uploads currently ready to go. """
        me.send_file("", "")


    def stop(me) :
        """ Stop the thread and wait for it to finish. """
        q       = me.q
        me.q    = None
        if  q   :
            q.put(None)
            try :
                me.join(1.5)
            except RuntimeError :
                pass
            pass
        pass


    #   a_bar_uploader


if  False   :
    me      = a_bar_uploader()
    me.start()
    me.send_file('x.y', 'lksjlfkjsldkjf slf')
    while True :
        try :
            time.sleep(1.0)
        except KeyboardInterrupt :
            break
        pass
    me.stop()
    exit(1)



#
#
#
if  __name__ == '__main__' :
    """

        Get some URLs to a directory named for the current Unix time.
        Put the time in the output file names, too.

    """

    import  re
    import  urlparse

    import  TZCommandLineAtFile
    import  url_getter


    del(sys.argv[0])

    TZCommandLineAtFile.expand_at_sign_command_line_files(sys.argv)


    cookies         = {}
    show_response   = False

    while True :
        oi  = tzlib.array_find(sys.argv, "--cookie")
        if  oi < 0 :    break
        del sys.argv[oi]
        ck_name = sys.argv.pop(oi)
        ck_val  = sys.argv.pop(oi)
        cookies[ck_name] = ck_val


    while True :
        oi  = tzlib.array_find(sys.argv, "--show_response")
        if  oi < 0 :    break
        del sys.argv[oi]
        show_response   = True



    cks = ""
    for c in cookies.keys() :
        v = cookies[c]
        cks += " " + c + "=" + v + ";"

    # print("adding cookie", cks)


    while   len(sys.argv) > 0 :

        url     = sys.argv.pop(0)

        t       = []
        t      += urlparse.urlsplit(url)

        v       = t[3]
        t[3]    = ""

        if  not v :
            g   = re.match(r"(.+?)\?(.+)", t[1])
            if  g :
                v       = g.group(2)
                t[1]    = g.group(1)
            pass

        url     = urlparse.urlunsplit(t)

        req     = urllib2.Request(url)

        req.add_data(v)

        if  len(cks) > 0 :
            req.add_header('Cookie', cks)

        req.add_header('User-Agent',       'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.14) Gecko/20080404 Firefox/2.0.0.14')
        req.add_header('Accept',           'text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,image/jpeg,image/gif;q=0.2,*/*;q=0.1')
        req.add_header('Accept-Language',  'en-us,en;q=0.5')
        # req.add_header('Accept-Encoding',  'gzip,deflate')
        req.add_header('Accept-Charset',   'ISO-8859-1,utf-8;q=0.7,*;q=0.7')
        req.add_header('Keep-Alive',       '300')
        req.add_header('Connection',       'keep-alive')

        r       = url_getter.url_open_read_with_timeout(req)
        if  r  == None :
            print("Failed to POST", url)
            sys.exit(101)

        if  show_response :
            print(r)

        pass

    pass

#
#
#
# eof
