#!/usr/bin/python

# number_bullet_wx_widget.py
#       --copyright--                   Copyright 2007 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       December 11, 2007       bar     spin off from camera_time.py
#       January 2, 2008         bar     flesh out an_examiner's talk to its owner
#       May 17, 2008            bar     email adr
#       June 1, 2008            bar     prompt param and PROMPT
#       August 28, 2008         bar     linux version crashes flakey ways with code adjusted in random ways with respect to the timer
#       May 27, 2012            bar     doxygen namespace
#       March 5, 2023           bar     future print
#       --eodstamps--
##      \file
#       \namespace              tzpython.number_bullet_wx_widget
#
#
#       Display a number bullet screen filling window using wxWidgets.
#
#

from    __future__  import  print_function

import  time

import  wx

import  number_bullet
import  number_bullet_scanner
import  tz_parse_time
import  tz_wx_widgets
import  tzlib



BULLET_FRAC     = 10
BULLET_TIME     = 3600 * 24 * BULLET_FRAC


PROMPT          = """Take some full-frame color pictures of the flashing patterns.
Avoid glare from room or sun lighting!
If the lighting is bad, take several pictures, each from slightly different angles and distances from the screen."""


class   a_time_flasher(wx.Frame) :
    """
        Flash number bullets with the GMT time in a screen-filling window.
        Stop on any key or mouse movement or whatever.
    """

    def __init__(me, parent, ID, title, prompt = None) :
        """ Constructor. """

        prompt  = prompt or PROMPT

        swidth  = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_X)
        shite   = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_Y)

        # swidth  = 640
        # shite   = 480

        me.captured = 0

        wx.Frame.__init__(me, parent, ID, title, style = wx.CLOSE_BOX | wx.SYSTEM_MENU | wx.STAY_ON_TOP, size = [ swidth, shite ] )

        me.SetBackgroundColour(wx.Colour(255, 255, 255))
        me.SetFont(wx.Font(14, wx.DEFAULT, wx.NORMAL, wx.NORMAL))


        tw  = wx.Window(me, -1)
        tw.SetBackgroundColour(wx.Colour(0, 0, 255))
        tw.SetForegroundColour(wx.Colour(255, 255, 128))

        tb  = wx.BoxSizer(wx.VERTICAL)

        ht  = wx.StaticText(tw, -1, prompt, pos = wx.Point(20, 20)
                            )
        tb.Add(ht, 1, wx.EXPAND)

        tb.AddSpacer(60)

        tw.SetSizer(tb)
        tw.SetAutoLayout(True)


        me.pat_win          = wx.Window(me, -1)

        mb                  =  wx.BoxSizer(wx.VERTICAL)
        mb.Add(tw,          0, wx.EXPAND)
        mb.Add(me.pat_win,  1, wx.EXPAND)


        me.SetSizer(mb)
        me.SetAutoLayout(True)

        me.Fit()
        me.SetDimensions(0, 0, swidth, shite)

        ht.Wrap(me.GetSizeTuple()[0] - 30)


        me.SetFocus()

        wx.EVT_CLOSE(       me, me.on_close_window)
        wx.EVT_SIZE(        me, me.on_size)
        wx.EVT_PAINT(       me, me.on_paint)

        wx.EVT_CHAR(        me, me.on_human_activity)
        wx.EVT_KEY_DOWN(    me, me.on_human_activity)
        wx.EVT_SIZE(        me, me.on_human_activity)
        wx.EVT_KILL_FOCUS(  me, me.on_human_activity)
        wx.EVT_MOUSE_CAPTURE_LOST(me, me.on_human_activity)

        me.mouse_x          = 0
        me.mouse_y          = 0
        me.SetCursor(wx.StockCursor(wx.CURSOR_BLANK))
        me.captured         = 1
        me.CaptureMouse()
        wx.EVT_MOUSE_EVENTS(me, me.on_mouse)

        me.ID_timer         = wx.NewId()
        me.timer            = wx.Timer(me, me.ID_timer)
        wx.EVT_TIMER(me, me.ID_timer, me.on_timer)
        me.cal  = 0
        me.timer.Start(97)


    def on_timer(me, evt = None) :
        t   = int(time.time() * BULLET_FRAC) % ( BULLET_TIME )

        brr = number_bullet.a_number_bullet(BULLET_TIME - 1)

        ( ww, wh )  = me.pat_win.GetSizeTuple()

        marg    = 5
        wwh     = min(ww - marg - marg, wh - marg - marg)
        r       = wwh / 4
        r3      = r * 3

        calxy   = [
                    [ marg + r,  marg + r  ],
                    [ marg + r,  marg + r3 ],
                    [ marg + r3, marg + r  ],
                    [ marg + r3, marg + r3 ],
                  ]

        if  not hasattr(me, 'cal') :
            me.cal  = 0
        if  me.cal >= len(calxy) :
            me.cal  = 0



        class   a_td :
            def __init__(me, thang, dc, p, r) :
                me.thang    = thang
                me.dc       = dc
                me.p        = p
                me.r        = r

            def hite(me) :
                return(600)

            def width(me) :
                return(800)

            def draw_ring(me, ring, of_rings, rgb) :
                me.dc.SetBackground(wx.Brush(wx.Colour(rgb[0], rgb[1], rgb[2])))
                me.p.SetColour(wx.Colour(rgb[0], rgb[1], rgb[2]))
                me.dc.SetPen(me.p)
                r           = int(round(me.r - ((ring + 0.5) * (float(me.r) / of_rings))))
                me.dc.DrawCircle(calxy[me.thang.cal][0], calxy[me.thang.cal][1], r)

            pass


        dc  = wx.ClientDC(me.pat_win)
        p   = wx.Pen(wx.Colour(0, 0, 0), (r / brr.rings + 2))
        dc.SetPen(p)
        dc.SetTextBackground(me.pat_win.GetBackgroundColour())
        dc.SetTextForeground(wx.Colour(255, 0, 255))
        dc.SetBackgroundMode(wx.SOLID)
        dc.DrawText("%06u" % t, calxy[me.cal][0] - r, calxy[me.cal][1] - r)

        ptr = a_td(me, dc, p, (r * brr.rings) / (brr.rings + 1))

        brr.paint(ptr, t)

        me.cal     += 1

        del(dc)
        del(p)


        if  me.captured :
            me.captured += 1

        pass



    def on_size(me, evt = None) :
        me.Layout()
        me.Refresh()


    def on_close_window(me, evt = None) :
        win = wx.Window.FindFocus()
        if  win != None:
            win.Disconnect(-1, -1, wx.wxEVT_KILL_FOCUS)    # avoids windows calling on_human_activity (from KILL_FOCUS) after we're long gone

        if  me.captured :
            me.captured = 0
            try :
                me.ReleaseMouse()
            except wx._core.PyAssertionError :
                # e   = sys.exc_info()
                # print(e[0], e[1], e[2])
                pass
            pass

        if  hasattr(me, 'timer') :
            me.timer.Stop()

        me.Destroy()



    def on_paint(me, evt = None) :
        dc = wx.PaintDC(me)
        dc.Clear()


    def on_human_activity(me, evt = None) :
        evt.Skip()                                  # keep processing this event
        me.Close()


    def on_mouse(me, evt = None) :
        if  me.captured > 10 :
            if  not evt.Moving() or (abs(me.mouse_x - evt.GetX()) >= 5) or (abs(me.mouse_y - evt.GetY()) >= 5) :
                me.Close()
            pass
        else :
            me.mouse_x  = evt.GetX()
            me.mouse_y  = evt.GetY()
        pass


    pass        # a_time_flasher



class   an_examiner :
    """
        Give the user a prompt to examine number bullet jpg files and calculate and tell him the time offset.
    """

    def __init__(me, owner, fnames) :
        me.fnames       = fnames
        me.owner        = owner
        me.examiner     = None
        me.time_diff    = None

        if  fnames      :
            me.examiner = number_bullet_scanner.a_number_bullet_file_scanner(me, fnames, max_wait = 5.0)
            me.examiner.start()
        pass



    def stop(me) :
        ex          = me.examiner
        me.examiner = None

        if  ex :
            ex.stop()

        return(ex)


    def number_bullet_scanner_done(me, tcnt, td_tot, bad_fnames) :
        me.examiner = None

        if  tcnt :
            me.time_diff    = float(td_tot) / float(tcnt)

        show    = not me.owner or me.owner.examiner_time_difference(me.time_diff)

        if  show :
            if  not tcnt :
                ss      = "these files"
                if  not bad_fnames :
                    ss  = "any files"
                elif len(bad_fnames) == 1 :
                    ss  = "this file"
                tz_wx_widgets.show_msg("I did not find a time in " + ss + ":\r\n\r\n" + "\r\n".join(bad_fnames) + "\r\n", wx.OK)
            else :
                s               = "The average of %u sample%s is %s"    % ( tcnt, tzlib.s_except_1(tcnt), tz_parse_time.time_diff_str(0, round(me.time_diff)) )
                s              += "\r\n and to local time is %s."       % ( tz_parse_time.time_diff_str(0, round(me.time_diff) + time.timezone) )
                tz_wx_widgets.show_msg(s)
            pass
        pass


    pass        # an_examiner



if __name__ == '__main__' :
    print(  "Imported from wxWidgets python scripts/programs.")



#
#
#
# eof
