#!/usr/bin/python

# pushfile.py
#       --copyright--                   Copyright 2007 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       January 12, 2007        bar
#       November 18, 2007       bar     turn on doxygen
#       November 20, 2007       bar     comments
#                                       next()
#       November 27, 2007       bar     insert boilerplate copyright
#       May 17, 2008            bar     email adr
#       May 27, 2012            bar     doxygen namespace
#       March 5, 2023           bar     future print
#       April 3, 2023           bar     python3, kinda
#       --eodstamps--
##      \file
#       \namespace              tzpython.pushfile
#
#       File input whose readline() understands pushline().
#
#

from    __future__  import  print_function


class   a_pushfile(object) :
    """ Class that overloads built in 'file' class to allow text lines to be pushed back in to a file that the owner wants to call readline() on. """

    def __init__(me, name, mode = "r") :
        """ Constructor. """

        me.fi               = open(name, mode)

        me.pushed_lines     = []
        me.line_nm          = 0


    def next(me) :
        """ Overload routine to get the next text line starting with lines pushed. """

        if  len(me.pushed_lines) :
            return(me.pushed_lines.pop())

        me.line_nm += 1

        return(me.fi.next())


    def readline(me) :
        """ Overload routine to get the next text line starting with lines pushed. """

        if  len(me.pushed_lines) :
            return(me.pushed_lines.pop())

        me.line_nm += 1

        return(me.fi.readline())


    def pushline(me, li) :
        """ Push a text line back in to the file. """

        if  li :
            me.pushed_lines.append(li)
            # me.pushed_lines.insert(0, li)     # completely different logic, eh?

        return(li)


    def read(me, *args, **kwargs) :
        return(me.fi.read(*args, **kwargs))

    def fileno(me, *args, **kwargs) :
        return(me.fi.fileno(*args, **kwargs))

    def flush(me, *args, **kwargs) :
        return(me.fi.flush(*args, **kwargs))

    def isatty(me, *args, **kwargs) :
        return(me.fi.isatty(*args, **kwargs))

    def tell(me, *args, **kwargs) :
        return(me.fi.tell(*args, **kwargs))

    @property
    def closed(me) :
        return(me.fi.closed)

    @property
    def encoding(me) :
        return(me.fi.encoding)

    @property
    def mode(me) :
        return(me.fi.mode)

    @property
    def name(me) :
        return(me.fi.name)

    def close(me) :
        rv  = me.fi.close()
        return(rv)


    pass        # a_pushfile


#
#
#
if __name__ == '__main__' :
    import  sys

    import  tzlib

    name        = "x.y"
    if  len(sys.argv) > 1 :
        name    = sys.argv[1]           # for tz_lib_test.py
    mode        = "r"
    if  len(sys.argv) > 2 :
        mode    = sys.argv[2]           # for tz_lib_test.py

    f           = a_pushfile(name, mode)
    li          = f.readline()
    print(li)
    if  tzlib.is_byteish(li) :
        p1      = b"p1 "
        p2      = b"p2 "
    else        :
        p1      =  "p1 "
        p2      =  "p2 "
    f.pushline(p1 + li)
    f.pushline(p2 + li)
    print(f.readline())
    print(f.readline())
    print(f.readline())
    print(f.readline())

    f.close()

#
#
#
# eof
