#!/usr/bin/python

# recolor_image.py
#       --copyright--                   Copyright 2014 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       June 4, 2014            bar
#       March 5, 2023           bar     future print
#       --eodstamps--
##      \file
#       \namespace              tzpython.recolor_image
#
"""
    Recolor image(s), given a source color and destination color, both of which are the "full" color.

"""


from    __future__  import  print_function

import  os
import  sys

from PIL    import  Image
from PIL    import  ImageColor

import  tzlib
import  replace_file



def pv(v, m) :
    """ Return a colorized pixel color. """
    return(max(0, min(255, int(v * m))))


def colorize_image(img, from_c, to_r, to_g, to_b) :
    """ Return an RGBA image, colored to a new shade. """
    dat = img.getdata()

    if  (from_c is None) or (from_c == "") :
        if  not hasattr(dat[0], '__getitem__') :
            dat = [ ( p, p, p, 255  ) for p in dat ]
        elif  len(dat[0]) == 2 :
            dat = [ ( p[0], p[0], p[0], p[1] ) for p in dat ]
        elif  len(dat[0]) == 3 :
            dat = [ ( p[0], p[1], p[2], 255  ) for p in dat ]

        a   = max([ p[3] for p in dat ])
        r   = max([ p[0] for p in dat if p[3] == a ])
        g   = max([ p[1] for p in dat if p[3] == a ])
        b   = max([ p[2] for p in dat if p[3] == a ])
    else :
        r, g, b = ImageColor.getrgb(from_c)

    rm      = float(to_r) / r
    gm      = float(to_g) / g
    bm      = float(to_b) / b

    timg    = Image.new("RGBA", img.size)
    timg.putdata([ ( pv(p[0], rm), pv(p[1], gm), pv(p[2], bm), p[3] ) for p in dat ])

    return(timg)




help_str    = """
%s (options)    ambiguous_image_file_name...

Options:
    --to        PIL_color       The new color the source color should become.
    --outdir    directory       Directory to put output images in as the input image file names.
    --output    image_file_name Output file name (for one image).
    --from      PIL_color       The source color. (default: the brightest color in the highest alpha)
    --sub_dirs                  Include matching files in subdirectories.

Re-color the given images given the basic color of the source image(s) and the desired color they should be.

"""

#
#
if __name__ == '__main__' :

    import  TZCommandLineAtFile


    program_name    = sys.argv.pop(0)

    TZCommandLineAtFile.expand_at_sign_command_line_files(sys.argv)

    sub_dirs        = False
    from_c          = None
    to_c            = None
    ofile_name      = None
    out_dir         = None

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--help", "-h", "-?", "/?", "/h", "/H" ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        print(help_str % ( os.path.basename(program_name) ))
        sys.exit(254)


    while True :
        oi  = tzlib.array_find(sys.argv, [ "--from", "--from_color", "--from-color", "--fromcolor", "-f", ] )
        if  oi < 0  : break
        del sys.argv[oi]
        from_c      = sys.argv.pop(oi)

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--to", "--to_color", "--to-color", "--tocolor", "-t", ] )
        if  oi < 0  : break
        del sys.argv[oi]
        to_c        = sys.argv.pop(oi)


    while True :
        oi  = tzlib.array_find(sys.argv, [ "--sub_dirs", "--subdirs", "--sub-dirs", "--sub_dir", "--subdir", "--sub-dir", "--sd", "-s", ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        sub_dirs    = True                  # search sub-directories


    if  (to_c is None) or (to_c == "") :
        print("No to-color!", file = sys.stderr)
        sys.exit(101)
    to_r, to_g, to_b    = ImageColor.getrgb(to_c)

    while len(sys.argv) :
        a   = sys.argv.pop(0)
        if  tzlib.array_find([ "--outdir", ], a) >= 0 :
            outdir      = sys.argv.pop(0)
        elif  tzlib.array_find([ "--output", "-o", ], a) >= 0 :
            ofile_name  = sys.argv.pop(0)
            outdir      = None
        else    :
            fns = tzlib.ambiguous_file_list(a, do_sub_dirs = sub_dirs)
            if  not len(fns) :
                print("No file found: %s!" % a, file = sys.stderr)
                sys.exit(111)

            for fn in fns :
                img = Image.open(fn)
                dat = img.getdata()
                if  not len(dat) :
                    print("No image data found in: %s!" % fn, file = sys.stderr)
                    sys.exit(110)

                omg = colorize_image(img, from_c, to_r, to_g, to_b)

                if  ofile_name :
                    ofn = ofile_name
                    if  outdir :
                        if  not os.path.dirname(ofn) :
                            ofn = os.path.join(outdir, ofn)
                        pass
                    pass
                else    :
                    if  not outdir :
                        print("Tell an output name or directory for: %s!" % fn, file = sys.stderr)
                        sys.exit(102)

                    ofn = os.path.join(outdir, os.path.basename(fn))
                    if  tzlib.is_same_file(ofn, fn) :
                        print("I won't write over: %s!" % fn, file = sys.stderr)
                        sys.exit(103)
                    pass

                obn, ext    = os.path.splitext(ofn)
                tfn         = obn + "_tmp" + ext
                omg.save(tfn)
                replace_file.replace_file(ofn, tfn, ofn + ".bak")

                ofile_name  = None
            pass
        pass

    pass

#
#
#
# eof
