#!/usr/bin/python

# rename.py
#       --copyright--                   Copyright 2014 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       December 2, 2014        bar
#       March 5, 2023           bar     future print
#       --eodstamps--
##      \file
#       \namespace              tzpython.rename
#
#
#       Convert all matching file names to lower case.
#
#

from    __future__  import  print_function

import  glob
import  os
import  re
import  sys

import  TZCommandLineAtFile
import  tzlib



class a_thang :


    def __cmd_line_help(me) :
        print("""
%s regx_from regx_to files_or_dirs...

    I rename files using regular a expression.

    Tell me (ambiguous) file names or directory names.

Options:

    --from              regx    'From' regular expression string. Matching strings are re.sub()'d to 'To' string.
    --to                string  'To' string.
    --subdirs or  -s            Do sub-directories.
    --verbose                   Print changed file names.

""") % os.path.basename(me.program_name)

        pass



    def _parse_cmd_line(me) :
        """
            Parse command line parameters
        """


        me.program_name = sys.argv.pop(0)

        TZCommandLineAtFile.expand_at_sign_command_line_files(sys.argv)

        while True :
            oi  = tzlib.array_find(sys.argv, [ "--help", "-h", "-?", "/?" ] )
            if  oi < 0 :    break
            del sys.argv[oi]
            me.__cmd_line_help()
            sys.exit(254)


        while True :
            oi  = tzlib.array_find(sys.argv, [ "--verbose", "-v" ] )
            if  oi < 0 :    break
            del sys.argv[oi]
            me.verbose += 1


        while True :
            oi  = tzlib.array_find(sys.argv, [ "--from", "-f" ] )
            if  oi < 0 :    break
            del sys.argv[oi]
            me.regx = sys.argv.pop(oi)

        while True :
            oi  = tzlib.array_find(sys.argv, [ "--to", "-t" ] )
            if  oi < 0 :    break
            del sys.argv[oi]
            me.torx = sys.argv.pop(oi)

        while True :
            oi  = tzlib.array_find(sys.argv, [ "--subdirs", "-s" ] )
            if  oi < 0 :    break
            del sys.argv[oi]
            me.do_subdirs   = True


        if  not len(sys.argv) :
            me.__cmd_line_help()
            sys.exit(254)
        if  me.regx is None :
            me.regx = sys.argv.pop(0)
        if  not len(sys.argv) :
            me.__cmd_line_help()
            sys.exit(254)
        if  me.torx is None :
            me.torx = sys.argv.pop(0)

        me.regx = re.compile(me.regx)


    def __init__(me) :
        me.regx             = None
        me.torx             = None

        me.do_subdirs       = False

        me.verbose          = 0

        me.base_amb_name    = ""

        me._parse_cmd_line()



    def print_dir_entry(me, fname) :
        fn  = me.regx.sub(me.torx, fname)
        if  fn != fname :
            if  me.verbose :
                print("%s -> %s" % ( fname, fn, ))
            os.rename(fname, fn)
        pass


    pass





def print_em(me, s) :
    for fname in glob.glob(s) :
        me.print_dir_entry(fname)
    pass



def print_dir(me, s, names) :
    if  me.base_amb_name :
        print_em(me, os.path.abspath(os.path.join(s, me.base_amb_name)))
    else :
        print_em(me, os.path.abspath(os.path.join(s, "*.*")))
    pass



#
#
if __name__ == '__main__' :
    me = a_thang()

    for s in sys.argv :

        if me.do_subdirs :
            if  not os.path.isdir(s) :
                ( s, me.base_amb_name ) = os.path.split(os.path.normpath(s))
                if  s == "" :   s = "."
                if  not os.path.isdir(s) :
                    print_dir(me, s, [] )
                else :
                    os.path.walk(s, print_dir, me)
                pass
            else :
                os.path.walk(s, print_dir, me)
            pass

        elif    not os.path.isdir(s) :
            print_em(me, s)

        else :
            print_dir(me, s, [])

        pass

    pass

#
#
#
# eof
