#!/usr/bin/python

# rip_multi_cd.py
#       --copyright--                   Copyright 2009 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       January 10, 2009        bar
#       January 12, 2009        bar     run "eject" at the end
#                                       lower lame quality
#       January 29, 2009        bar     --never-skip=2 for cdparanoia
#       February 12, 2009       bar     leave a numeric space between cds (so we can inject a "this is the last disc msg")
#       February 13, 2009       bar     try to avoid double-ripping CDs
#                                       make lame more quiet
#                                       nocd
#                                       dupecd
#       February 14, 2009       bar     indir
#       February 21, 2009       bar     eject immediately after ripping, before converting to mp3
#       March 8, 2009           bar     strip silence from both ends of the wave files (because 99-track albums have too much silence on the shoulders)
#       August 4, 2009          bar     spin the strip silence logic to a separate script so that it won't eat memory
#                                       --span
#       October 6, 2009         bar     --ignore_errors
#       April 10, 2010          bar     update
#                                       --cdparanoia_options
#       April 18, 2010          bar     help cdparanoia find the cdrom
#       September 29, 2010      bar     better help str
#       November 29, 2011       bar     pyflake cleanup
#       May 27, 2012            bar     doxygen namespace
#       July 14, 2012           bar     --loop
#       July 15, 2012           bar     crank delay up to 15 seconds from 10
#       July 16, 2012           bar     --delay
#       October 29, 2014        bar     more option synonyms
#       November 11, 2014       bar     is_audio_cd_inserted()
#       August 19, 2015         bar     pyflakes
#       January 25, 2018        bar     --loop in --help
#       August 1, 2018          bar     able to use lshw besides lshal
#       October 11, 2019        bar     prompt with the disk number
#       March 5, 2023           bar     future print
#       March 8, 2023           bar     python3
#       September 19, 2023      bar     for-debug code
#                                       tell user to install programs we use
#       --eodstamps--
##      \file
#       \namespace              tzpython.rip_multi_cd
#
#
#       Shell-ish script to rip audio book cds so that they end up with MP3 files named in sequencial manner.
#
#       Requires:
#           cdparanoia
#           lame
#       Wants:
#           eject
#
#       I convert audio books to mp3 files with this:
#
#           cd  tmp
#           nd  BOOK_TITLE
#           python /home/alex/flight/tzpython/rip_multi_cd.py --cdparanoia "-d /dev/cdrom2" --prename BOOK_TITLE_ . --ignore_errors
#
#
#       TODO:
#
#           figure out PRE_NAME from current directory and, if any .mp3 files are in the dir, that they begin with current dir name
#
#           run the ripper as a sub-process. Verify that it's proceeding OK. If not, then bail and restart, cleaning up the .wav files appropriately.
#               especially on the 1st track
#               binary search around bad spots?
#
#               If ripping is going too slowly, auto-apply --ignore_errors
#
#
#           --strip causes a big memory leak and I don't know why!
#
#           Run the mp3 conversions while ripping.
#               NO. Swapping the CD takes as long as the conversions, anyway.
#               use the herd logic in tz_multi_run.py for conversions
#
#           Use the TOC length * 2352 (+ 44) for the wav file length to detect wav files already ripped to pick up in the middle of a disc.
#
#

from    __future__  import  print_function

import  glob
import  multiprocessing
import  os
import  re
import  subprocess
import  sys
import  time

import  strip_wave_silence
import  TZKeyReady
import  tzlib


PRE_NAME        = "rmcd_"

LAME_OPTIONS    = "-V 9 -b 64 --nohist -S "                 # note: -S causes parellel conversions

DELAY           = 15.0                                      # how long to delay after sensing CD, before ripping

nmp3_re         = re.compile(r"(\d+)\.mp3$", re.DOTALL + re.IGNORECASE)

fn_format       = "%04u.mp3"




def get_wave_file_names(indir, outdir) :
    if  indir   :
        tfls    = glob.glob(os.path.join(indir,  "*.wav"))
    else :
        tfls    = glob.glob(os.path.join(outdir, "track*.wav"))
        tfls   += glob.glob(os.path.join(outdir, "??.Track_*.wav"))
        tfls   += glob.glob(os.path.join(outdir, "*AudioTrack *.wav"))
    tfls.sort()

    return(tfls)



def run_lame(cmd, fn) :
    if  os.system(cmd) :
        print("lame failure on %s!" % fn)
        sys.exit(104)
    try     :
        os.unlink(fn)
    except ( OSError, IOError ) :
        tzlib.print_exception()
        sys.exit(105)
    pass


def convert_to_mp3(indir, outdir, pre_name = None, lame_options = None, strip_silence = False) :
    pre_name            = pre_name or PRE_NAME
    if  lame_options   == None :
        lame_options    = LAME_OPTIONS

    tfls        = get_wave_file_names(indir, outdir)

    if  tfls :

        mfls    = glob.glob(os.path.join(outdir, pre_name + "*.mp3"))
        mfls.sort()

        n   = 1
        if  mfls :
            g   = nmp3_re.search(mfls[-1])
            if  not g :
                print("Unnumbered mp3 file in directory, %s\n%s\n----------!" % ( outdir, "\n".join(mfls) ))
                sys.exit(103)
            n   = int(g.group(1)) + 2

        pa      = []
        for fn in tfls :
            ofn = os.path.join(outdir, (pre_name + (fn_format % n)))

            if  strip_silence :
                os.system("python %s %s" % ( strip_wave_silence.__file__, fn ) )

            if  sys.platform == 'win32' :
                cmd = 'lame %s "%s" "%s"'   % ( lame_options, fn.replace('"', r'\"'), ofn )
            else :
                cmd = 'lame %s  %s  "%s"'   % ( lame_options, re.sub(r"([^a-zA-Z0-9\/\.])", r"\\\1", fn), ofn )           # fn.replace(")", r"\)").replace("(", r"\(").replace("'", r"\'").replace('"', r'\"').replace(' ', r'\ ')

            print("cmd", cmd)

            if  lame_options.find("-S") >= 0 :
                p           = multiprocessing.Process(target = run_lame, args = ( cmd, fn, ))
                p.daemon    = True
                p.start()
                pa.append(p)
            else    :
                run_lame(cmd, fn)

            n  += 1

        for p in pa :
            p.join()

        pass

    pass



def inject_last_cd(outdir, lame_options, file_name, where) :
    if  file_name :
        if  lame_options   == None :
            lame_options    = LAME_OPTIONS

        tfls    = [ fn for fn in glob.glob(os.path.join(outdir, "*.mp3")) if nmp3_re.search(fn) ]
        tfls.sort()

        while tfls :
            pre_name    = nmp3_re.sub("", tfls[0])
            na          = []

            while tfls :
                if  not tfls[0].startswith(pre_name) :
                    break
                na.append(int(nmp3_re.search(tfls.pop(0)).group(1)))

            if  na :
                w   = where
                while na and (w < 0) :
                    n       = na.pop() - 1
                    while na and (na[-1] == n) :
                        n   = na.pop() - 1
                    w  += 1
                if (w  >= 0) and (n >= 0) :
                    ofn = pre_name + (fn_format % n)
                    if  os.path.isfile(ofn) :
                        print("%s is already a file! BUG!" % ofn)
                        sys.exit(105)

                    # print(ofn, file_name)

                    if  file_name.endswith(".wav") :
                        cmd = "lame %s '%s' '%s'" % ( lame_options, file_name, ofn )
                        if  os.system(cmd) :
                            print("lame failure on %s%s!" % ( pre_name, file_name ))
                            sys.exit(104)
                        pass
                    else :
                        tzlib.write_whole_binary_file(ofn, tzlib.read_whole_binary_file(file_name))
                    pass
                pass
            pass
        pass
    pass


def file_name_ok_to_inject(file_name) :
    if  not file_name :
        return(True)

    return(file_name.endswith(".wav") or file_name.endswith(".mp3"))





def is_audio_cd_inserted() :
    """ Return True if an audio disk is inserted. """
    r, rs   = tzlib.run_program('lshw -short')
    if  not r :
        if  re.search(r'(?m)/dev/cdrom\s+disk[ \t]*$', rs) :                # also, not -short we could look for two instances of "logical name: /dev/cdrom" or "status=ready" under "*-cdrom", maybe using the json output

            return(True)

        return(False)

    r,  rs  = tzlib.run_program('lshal')                                    # this program has been taken out of Debian's repo. September 19, 2023
    if  not r :
        if  re.search(r"volume\.disc\.has_audio\s*=\s*true\s", rs) :        # really this is a pretty dumb and cheap way to sense it, but whatever

            return(True)

        return(False)

    return(-1)                  # well, we really don't know


tracks_re   = re.compile(r'^\s*(\d+)\.\s+\d+', re.MULTILINE)
def get_disk_number_and_track_count(dupe_file_name) :
    ds  = tzlib.safe_read_whole_text_file(dupe_file_name) or ""
    dcs = re.split(r'Table of contents', ds)
    tc  = 0
    for s  in dcs :
        tc += len(tracks_re.findall(s))
    return(( len(dcs), tc ))


def print_prompt(dupe_file_name, outdir) :
    dsn, tc     = get_disk_number_and_track_count(dupe_file_name)
    mp3fls      = [ fn for fn in glob.glob(os.path.join(outdir, "*.mp3")) if nmp3_re.search(fn) ]
    print('')
    print("%u track%s recorded. %u should be. Ready for CD number %u." % ( len(mp3fls), tzlib.s_except_1(mp3fls), tc, dsn, ))


def is_dupe_or_no_disk(dupe_file_name, ask, options = "") :
    """
        Return False if the disc is there and wants to be ripped.
        Return True  if there disk is a dupe of one we've ripped and doesn't want to be ripped.
        Return       -1 if there's no disc.
    """
    if  is_audio_cd_inserted() :
        p           = subprocess.Popen("cdparanoia -Q %s" % ( options or "" ), shell=True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.STDOUT, close_fds = True)
        # ( si, so )  = ( p.stdin, p.stdout )
        sos         = p.stdout.read()
        p.stdout.close()
        p.stdin.close()
        #
        #   Pipe cdparanoia 2>&1  so you can see that it's not installed (but don't leave it that way or the same-disc sensor will not work)
        #   print("@@@@", sos, "<<<<<<<<<<<<<<<<<<")
        #

        sos         = tzlib.convert_to_unicode(sos)

        if  sos.find("Table of contents (audio tracks only):") > 0 :
            ds  = ""
            if  os.path.isfile(dupe_file_name) :
                ds  = tzlib.read_whole_text_file(dupe_file_name)

                if  ds.find(sos) >= 0 :
                    if  not ask :
                        return(True)                            # just tell the caller to eject the disc

                    print("This disk appears to be already ripped.")
                    print("Do you want to rip it again (y/N)?", end = ' ')
                    while True :
                        k   = TZKeyReady.key_ready()
                        if  k :
                            print('')
                            k   = k.lower()
                            if  k == 'y' :
                                print("OK. Here we go...")
                                sos = ""
                                break

                            if  (k == 'n') or (k == '\r') or (k == '\n') or (k == chr(3)) or (k == 'q') :
                                print("Then I will eject the disc.")

                                return(True)

                            print("Well (y/N)?")

                        time.sleep(0.1)
                    pass
                pass

            if  sos :
                ds += sos
                tzlib.write_whole_text_file(dupe_file_name, ds)

            return(False)

        pass

    return(-1)





if  __name__ == '__main__' :
    import  TZCommandLineAtFile
    import  onpath


    help_str    = """
%s (options)

    I rip multiple CDs to sequentially numbered MP3 files.
    The MP3 numbering spans multiple CDs, so that CDs can be ripped in order and then a player will play them in order.

    To use:
        Make an empty directory.
        cd to the directory.
        Put the 1st/next CD in the CD drive and run me for each CD.
        Run me with the --lastcd option to inject a .wav file of your choice before the last CD.

    I convert all track*.wav files in the outdir to sequentially numbered .mp3 files (skipping a number between CDs).
    I erase the   track*.wav files as I do this.
    Then I rip whatever CD is in the CD drive.
    Then I do the track*.wav to mp3 conversion operation again.
    Then, if the rip took long enough (10 seconds), I eject the CD.

    I do the sequential numbering like this:
        prename + "%%04.mp3"
    I pick up on currently existing files to continue numbering beyond them, skipping a number.

    I can inject a .wav file in the numeric holes before the presumed last 2 CDs.
      Run me with the --lastcd and/or --2ndlastcd options once on a directory, though,
      as the holes will move back a CD for each of these options having been done.

Options:

    --indir     dir             Set the input directory.                (default: none (If set, read ./*.wav files)
    --outdir    dir             Set the output directory.               (default: .)
    --prename                   Set the output file prefix name.        (default: %s)
    --ignore_errors             Set cdparanoia to ignore errors.        (default: retry twice)
    --loop                      Keep reading CDs forever.
    --span      option          Set the cdparanoia 'span' option.       (default: none)
    --cdparanoia    "options"   Set cdparanoia options.                 (default: "")
    --lame      "options"       Set the lame options.                   (default: %s)
    --lastcd    wave_file       Set the "last CD" wave file.            (default: none)
    --2ndlastcd wave_file       Set the "second to last CD" wave file.  (default: none)
    --dupefile  file_name       Set the duplicate CD detection file.    (default: rip_multi_cd_discs.txt)
    --nocd                      Do not access the CD (only do wav to mp3 conversions).
    --listwaves                 Just list the .wav files in order.
    --strip                     Strip silence from ends of the .wav files.
    --delay     seconds         Set how long to delay after CD sense.   (default: %.1f)

    """ % ( os.path.basename(sys.argv[0]), PRE_NAME, LAME_OPTIONS, DELAY )

    program_name        = sys.argv.pop(0)
    TZCommandLineAtFile.expand_at_sign_command_line_files(sys.argv)

    errs                = []
    for pn in [ 'lame', 'cdparanoia', 'lshw', 'eject', ] :
        if  not onpath.on_path(pn) :
            errs.append("Install %s!" % pn)
        pass
    if  errs            :
        print("\n".join(errs))
        sys.exit(1)


    indir               = None
    outdir              = "."
    pre_name            = None
    lame_options        = None
    cdparanoia_options  = None
    last_cd             = None
    second_to_last_cd   = None
    dupe_file_name      = None
    do_cd               = True
    strip_silence       = False
    span                = ""
    ignore_errors       = False
    loop                = False
    delay               = DELAY

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--help", "-h", "-?", "/?", "/h" ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        print(help_str)
        sys.exit(254)

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--indir", "--in", "--in_dir", "--in-dir", "-i" ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        indir           = sys.argv.pop(oi)

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--outdir", "--out", "--out_dir", "--out-dir", "-o" ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        outdir          = sys.argv.pop(oi)

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--prename", "--pre_name", "--pre-name", "--precede", "--pre", "-p", "--name", "--file_name", "--file-name", "--filename", "--file_names", "--file-names", "--filenames", ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        pre_name        = sys.argv.pop(oi)

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--span", ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        span            = sys.argv.pop(oi)

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--loop",  ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        loop            = True

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--ignore_errors", "--ignoreerrors", "--ignore-errors", "--ie", ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        ignore_errors   = True

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--lame", "--lameoptions", "--lame_options", "--lame-options", "-l" ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        lame_options    = sys.argv.pop(oi)

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--cdparanoia", "--cdparanoiaoptions", "--cdparanoia_options", "--cdparanoia-options", "--cd_paranoia_options", "--cd-paranoia-options", ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        cdparanoia_options  = sys.argv.pop(oi)

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--lastcd", "--last_cd", "--last-cd", "-1" ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        last_cd             = sys.argv.pop(oi)

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--2ndlastcd", "--secondtolastcd", "--2ndtolastcd", "--2nd2lastcd", "--second2lastcd", "--2nd_last_cd", "--second_to_last_cd", "--2nd_to_last_cd", "--2nd_2_last_cd", "--second_2_last_cd", "--second", "-2" ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        second_to_last_cd   = sys.argv.pop(oi)

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--dupefile", "--dupe_file", "--dupe-file", "-d" ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        dupe_file_name      = sys.argv.pop(oi)

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--nocd", "--no_cd", "--no-cd", "-n" ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        do_cd               = False

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--strip", "--stripsilence", "--strip_silence", "--strip-silence", ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        strip_silence       = True

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--delay", ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        delay               = float(sys.argv.pop(oi))

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--listwaves", "--listwavs", "--list_waves", "--list_wavs", "--list-waves", "--list-wavs", "--lw", ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        tfls                = get_wave_file_names(indir, outdir)
        for fn in tfls :
            print(fn)
        sys.exit(0)



    if  not file_name_ok_to_inject(last_cd) :
        print("Injection file name must be .wav or .mp3: %s" % last_cd)
        sys.exit(109)
    if  not file_name_ok_to_inject(second_to_last_cd) :
        print("Injection file name must be .wav or .mp3: %s" % second_to_last_cd)
        sys.exit(109)


    if  not dupe_file_name  :
        dupe_file_name      = os.path.join(outdir, "rip_multi_cd_discs.txt")

    convert_to_mp3(indir, outdir, pre_name, lame_options, strip_silence)

    print_prompt(dupe_file_name, outdir)

    et      = bt    = 0

    ask     = True
    while do_cd :
        while True :
            d   = is_dupe_or_no_disk(dupe_file_name, ask, options = cdparanoia_options)
            if  d :
                if  d != True       :
                    cdopts          = cdparanoia_options or ""
                    if  cdopts.find("-d ") < 0 :
                        cd_names    = glob.glob("/dev/cdrom*")
                        for cdn in cd_names  :
                            cdopts  = "%s -d %s" % ( cdopts, cdn )
                            d       = is_dupe_or_no_disk(dupe_file_name, ask, options = cdopts)
                            if  not d :
                                cdparanoia_options  = cdopts
                                break
                            if  d == True   :
                                os.system("eject")
                                time.sleep(2)
                            pass
                        pass
                    pass
                if  d == True   :
                    os.system("eject")
                    time.sleep(2)
                if  not loop :
                    break
                pass
            if  not d :
                break
            pass

        if  d       :
            if  not loop :
                break
            while True :
                time.sleep(delay or 5)
                if  is_audio_cd_inserted() :
                    break
                pass
            pass
        else        :
            time.sleep(delay)                       # give a new CD a chance to settle

            print("copts", cdparanoia_options)
            bt      = tzlib.elapsed_time()
            if  not ignore_errors :
                os.system("cdparanoia -B --never-skip=2 %s %s" % ( span, cdparanoia_options or "" ) )           # rip the CD
            else    :
                os.system("cdparanoia -B -Z             %s %s" % ( span, cdparanoia_options or "" ) )           # rip the CD blindly
            et      = tzlib.elapsed_time()
            if  et - bt > 10.0 :
                os.system("eject")
                et  = bt    = 0

            convert_to_mp3(indir, outdir, pre_name, lame_options, strip_silence)
        print_prompt(dupe_file_name, outdir)

    sz      = 0.0
    mfls    = glob.glob(os.path.join(outdir, (pre_name or PRE_NAME) + "*.mp3"))
    for fn in mfls :
        sz += os.path.getsize(fn)

    # print("Total megabytes so far: %.0f" % ( (sz + (1 << 20) - 1) // (1 << 20) ))
    print(  "Total megabytes so far: %.0f" % ( (sz +  1000000  - 1) // 1000000  ))


    inject_last_cd(outdir, lame_options, second_to_last_cd, -2)
    inject_last_cd(outdir, lame_options, last_cd,           -1)

    if  et - bt > 10.0 :
        os.system("eject")

    pass


#
#
# eof
