#!/usr/bin/python

# show_image_where.py
#       --copyright--                   Copyright 2021 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       April 29, 2021          bar
#       May 1, 2021             bar     print the lat/lon/altitude, too
#                                       better URL
#                                       image_when()
#       May 16, 2021            bar     print the file name, too
#       July 22, 2021           bar     print the latlon strings how geocaching.com prints them: Deg Min_dot_...
#       March 5, 2023           bar     future print
#       August 4, 2023          bar     --print
#       --eodstamps--
##      \file
#       \namespace              tzpython.show_image_where

from    __future__  import  print_function

import  glob
import  os
import  re
import  sys

import  latlon
import  tz_browser
import  tz_jpg
import  tz_parse_time
import  tzlib


def exiftool_gps(fn) :
    """ Return (None,None,None) or ( lat, lon, altitude_in_meters, ) from exiftool's printout for the given file. Altitude may be None if the file has no altitude information. """
    lat     = None
    lon     = None
    alt     = None

    r, rs   = tzlib.run_program('exiftool "%s"' % fn)
    if  not r :
        g   = re.search(r'GPS Position +: +(.*?)\r?\n', rs)
        if  not g :
            g   = re.search(r'GPS Coordinates +: +(.*?)\r?\n', rs)
        if  g   :
            ll  = g.group(1)
            lat, lon    = latlon.parse_lat_lon(ll)
            if  lon is None :
                lat = None
            if  lat is None :
                lon = None
            pass

        g       = re.search(r'GPS Altitude +: +(\d+) m ', rs)
        if  g   :
            alt = int(g.group(1))

        pass

    return(lat, lon, alt)


def image_location(fn) :
    """ return the latitude, longitude, and altitude of the given image, or None, None, None. """
    lat, lon, alt   = exiftool_gps(fn)
    if  (lat is None) or (lon is None) :
        try             :
            e   = tz_jpg.a_jpg(fn).get_exif()
            if  not e :
                raise ValueError
            lat = e.get_latitude()
            lon = e.get_longitude()
            alt = e.get_altitude()
        except ValueError :
            pass
        pass
    return(lat, lon, alt)


def exiftool_when(fn) :
    """ Return None or the UTC time of a file as given by exiftool's printout. """
    when    = None
    r, rs   = tzlib.run_program('exiftool "%s"' % fn)
    if  not r       :
        for vs in   [ 'Date/Time Original', 'Create Date', 'Modify Date', 'GPS Date/Time', 'Track Create Date', 'Track Modify Date', 'Media Create Date', 'Media Modify Date', ] :
            g       = re.search(vs + r' +: +(.*?)\r?\n', rs)
            if  g   :
                ts  = g.group(1)
                t   = tz_parse_time.parse_time(ts)
                if  t > 1000000 :
                    when    = min(when or t, t)                 # find the earliest date (which if there's a bug in the file's exif info, we're in trouble - and really we should be using the datetime() info and allowing 19th century dates, right?)
                pass
            pass
        if  not when    :
            vs          = 'GPS Date Stamp'
            g           = re.search(vs + r' +: +(.*?)\r?\n', rs)
            if  g       :
                ts      = g.group(1)
                vs      = 'GPS Time Stamp'
                g       = re.search(vs + r' +: +(.*?)\r?\n', rs)
                if  g   :
                    ts += ' ' + g.group(1)
                    t   = tz_parse_time.parse_time(ts)
                    if  t > 631152000.0 :                       # let's not get excited about GPS time before 1990, dude
                        when    = min(when or t, t)
                    pass
                pass
            pass
        pass
    return(when)


def image_when(fn) :
    """ Return the UTC create or modify or whatever time of this image. """
    when            = exiftool_when(fn)
    if  not when    :
        try         :
            jpg     = tz_jpg.a_jpg(fn)
            when    = jpg.picture_taken_time()
            when    = when or tzlib.safe_file_create_time(fn)
        except ValueError :
            pass
        pass
    return(when)


if  __name__ == '__main__' :
    helpstr = """
    %s input_image

    Show the given image's location on Google Maps.

    Or --print  The image locations and times.

    """ % os.path.basename(sys.argv[0] or __file__)

    ex      = 0                 # default exit code

    argv    = sys.argv[1:]

    fnames  = []
    verbose = 0
    print_t = 0

    if  tzlib.array_find([ a.lower() for a in argv ], [ "--help", "/help", "-h", "-?", "/?", "-?", "/h", ] ) >= 0 :
        print(helpstr)
        sys.exit(254)

    while len(argv) :
        a   = argv.pop(0)
        la  = a.lower()
        if  not a.startswith('-') :
            fnames.append(a)
        elif tzlib.find_argi(la, [ "--verbose", "-v", ]) >= 0 :
            verbose    += 1
        elif tzlib.find_argi(la, [ "--print",   "-p", ]) >= 0 :
            print_t    += 1
        else            :
            print("I cannot understand the option %s" % a, file = sys.stderr)
            sys.exit(1)
        pass

    if  not len(fnames) :
        print(helpstr)
        sys.exit(254)

    fns         = []
    for fn     in fnames :
        fns    += glob.glob(fn)
    fns         = tzlib.without_dupes(fns)
    fns.sort()

    for fn in fnames    :
        lat, lon, alt   = image_location(fn)
        if  lat is None :
            print("I cannot find GPS information in", fn, file = sys.stderr)
            ex          = 1
        else            :
            es          = "" if alt is None else " Elevation:%5s meters" % alt
            print("At %11.6f, %11.6f%-23s %15s,%15s  %s"  % ( lat, lon, es, latlon.lat_degrees_minutes_string(lat), latlon.lon_degrees_minutes_string(lon), fn, ))
            if  not print_t :
                # tz_browser.start_file(    'https://www.google.com/maps/dir/?api=1&map_action=map&center=%s,%s&zoom=15&origin=%s,%s&destination=%s,%s&basemap=terrain' % ( lat, lon, lat, lon, lat, lon,   ))      # note: GM doesn't show terrain or satellite view - only street map when we have an origin or destination
                # tz_browser.start_file(    'https://www.google.com/maps/dir/?api=1&map_action=map&center=%s,%s&zoom=15&waypoints=%s,%s&basemap=terrain'                % ( lat, lon, lat, lon,             ))      # note: shows street view zoomed way out (to include current location?)
                # tz_browser.start_file(    'https://www.google.com/maps/search/?api=1&query=%s,%s'                                                                     % ( lat, lon,                       ))      # note: no zoom and map-type control
                # tz_browser.start_file(    'https://www.google.com/maps/@?api=1&map_action=map&center=%s,%s&zoom=15&basemap=terrain'                                   % ( lat, lon,                       ))      # note: no marker, but otherwise OK
                tz_browser.start_file(      'https://www.google.com/maps/place/%s,%s/@%s,%s,15.75z/data=!4m5!3m4!1s0x0:0x0!8m2!3d%s!4d%s!5m1!1e4'                       % ( lat, lon, lat, lon, lat, lon,   ))      # note: google's search result URL, including a marker and the text of the decimal lat/lon
            pass
        pass

    sys.exit(ex)


# eof
