#!/usr/bin/python

# stretch_image.py
#       --copyright--                   Copyright 2014 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       January 1, 2014         bar
#       February 19, 2014       bar     allow none parameters
#       March 5, 2023           bar     future print
#       --eodstamps--
##      \file
#       \namespace              tzpython.stretch_image
#
#
#       Stretch an image.
#
#

from    __future__  import  print_function

try :
    xrange
except NameError :
    xrange  = range


try :
    from    PIL     import  Image
except ImportError :
    Image               = None


def stretch_image(img, width = None, hite = None, idx = None) :
    """ Stretch the given image using the parameters from the command line, returning the resultant image. """
    w, h        = img.size

    nw          = w
    wi          = w
    if  width  != None :
        if  width < 0 :
            nw  = max(nw, int(-width * nw / 100.0))
        else    :
            nw  = max(nw, int(width))
        wi      = idx
        if  wi is None :
            wi  = w / 2
        pass

    if  width  != None :
        nimg    = Image.new(img.mode, (nw, h))
        nimg.paste(img, ( 0, 0, w, h))
        simg    = img.crop(( wi, 0, w, h))
        nimg.paste(simg,   ( nw  - (w - wi), 0))
        simg    = img.crop(( wi, 0, wi + 1,  h))
        for i in xrange(wi,  nw  - (w - wi)) :
            nimg.paste(simg, ( i, 0 ))
        nimg.load()
        img     = nimg
        w       = nw

    nh          = h
    hi          = h
    if  hite   != None :
        if  hite  < 0 :
            nh  = max(nh, int(-hite  * nh / 100.0))
        else    :
            nh  = max(nh, int(hite))
        hi      = idx
        if  hi is None :
            hi  = h / 2
        pass

    if  hite   != None :
        nimg    = Image.new(img.mode, (nw, nh))
        nimg.paste(img, ( 0, 0, w, h))
        simg    = img.crop(( 0, hi, w, h))
        nimg.paste(simg, (   0, nh  - (h - hi)))
        simg    = img.crop(( 0, hi, w, hi + 1))
        for i in xrange(hi,     nh  - (h - hi)) :
            nimg.paste(simg, ( 0, i ))
        nimg.load()
        img     = nimg

    return(img)


#
#
#
if __name__ == '__main__' :

    import  os
    import  sys

    import  TZCommandLineAtFile
    import  tzlib
    import  replace_file


    if  not Image :
        print("Cannot find PIL Image module!")
        sys.exit(250)



    program_name    = sys.argv.pop(0)

    TZCommandLineAtFile.expand_at_sign_command_line_files(sys.argv)


    help_str    = program_name + """ (options)   image_file output_file_name

    Stretch and image file to be wider or taller by replicating a middle column or row of pixels.

Options:

    --width     new_width               Specify the new width (image is stretched if wider  than image - may be new_width%).
    --hite      new_hite                Specify the new hite  (image is stretched if higher than image - may be new_hite% ).
    --index     column_or_row           Specify the column or row to duplicate (1-based).

"""

    if  tzlib.array_find(sys.argv, [ "--help", "-h", "-?", "/h", "/H", "/?" ] ) >= 0 :
        print(help_str)
        sys.exit(254)

    wh          = []
    width       = None
    hite        = None
    idx         = None

    while True  :
        oi      = tzlib.find_argi(sys.argv, [ "--index", "--idx", "-i", ] )
        if  oi  < 0 :   break
        del sys.argv[oi]
        idx     = int(sys.argv.pop(oi)) - 1

    while True  :
        oi      = tzlib.find_argi(sys.argv, [ "--width", "-w", ] )
        if  oi  < 0 :   break
        del sys.argv[oi]
        v       = sys.argv.pop(oi).strip()
        if  v.endswith('%') :
            width   = -abs(float(v[:-1]))
        else        :
            width   = float(v)
        pass

    while True  :
        oi      = tzlib.find_argi(sys.argv, [ "--hite", "-h", ] )
        if  oi  < 0 :   break
        del sys.argv[oi]
        v       = sys.argv.pop(oi).strip()
        if  v.endswith('%') :
            hite    = -abs(float(v[:-1]))
        else        :
            hite    = float(v)
        pass


    if  not len(sys.argv) :
        print("Please tell me an image to stretch and the name of an output image file to store the stretched image in.")
        sys.exit(101)

    ifile_name  = sys.argv.pop(0)
    if  ifile_name.startswith('-') :
        print("Please put a dot/slash or something before the input file name, %s!" % ifile_name)
        sys.exit(104)

    if  not len(sys.argv) :
        print("Please tell me an output image file to store the stretched image in.")
        sys.exit(102)

    ofile_name  = sys.argv.pop(0)
    if  ofile_name.startswith('-') :
        print("Please put a dot/slash or something before the output file name, %s!" % ofile_name)
        sys.exit(104)

    if  len(sys.argv) :
        print("I don't understand %s!" % str(sys.argv))
        sys.exit(103)

    if  (width is None) and (hite is None) :
        width   = -200.0            # go with 200% width gain by default

    img     = Image.open(ifile_name)

    nimg    = stretch_image(img, width, hite, idx)

    tfn = ofile_name + ".tmp" + os.path.splitext(ofile_name)[1]
    nimg.save(tfn)
    replace_file.replace_file(ofile_name, tfn, ofile_name + ".bak")


#
#
#
# eof
