#!/usr/bin/python

# svn_diff_all_revs.py
#       --copyright--                   Copyright 2014 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       June 9, 2014            bar
#       March 24, 2016          bar     make the pluses and minues in the right direction
#       March 5, 2023           bar     future print
#       April 13, 2023          bar     listize keys values items
#       --eodstamps--
##      \file
#       \namespace              tzpython.svn_diff_all_revs
#
#
"""
    Print the diffs in the given files for all revisions in the range given.

"""

from    __future__  import  print_function

import  os
import  re
import  sys

import  tzlib




help_str    = """
%s (options) files_to_diff...

    Prints SVN diffs for the given files by revision.

Options:

    --revision      rev(:rev)   Set revisions for SVN.

"""

if  __name__ == '__main__' :

    import  TZCommandLineAtFile


    program_name    = sys.argv.pop(0)
    TZCommandLineAtFile.expand_at_sign_command_line_files(sys.argv)


    sub_dirs        = False
    rev             = 'HEAD:0'
    verbose         = 0

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--help", "-h", "-?", "/h", "/H", "/?" ] )
        if  oi < 0  : break
        del sys.argv[oi]
        print(help_str % ( os.path.basename(program_name), ))
        sys.exit(254)


    while True :
        oi  = tzlib.array_find(sys.argv, [ "--verbose", "-v", ] )
        if  oi  < 0 :   break
        del sys.argv[oi]
        verbose    += 1

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--revision", "-r", ] )
        if  oi  < 0 :   break
        del sys.argv[oi]
        rev         = sys.argv.pop(oi)

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--sub_dirs", "--subdirs", "--sub-dirs", "--sub_dir", "--subdir", "--sub-dir", "--sd", "-s", ] )
        if  oi < 0  : break
        del sys.argv[oi]
        sub_dirs    = True


    if  not len(sys.argv) :
        print(help_str % ( os.path.basename(program_name), ))
        sys.exit(254)


    afns            = {}
    while len(sys.argv) :
        fn          = sys.argv.pop(0)
        fns         = tzlib.ambiguous_file_list(fn, do_sub_dirs = sub_dirs)

        if  not len(fns) :
            print("I cannot find input file(s) %s !" % fn, file = sys.stderr)

        afns.update(tzlib.make_dictionary([ os.path.abspath(fnm) for fnm in fns ]))

    fns = list(afns.keys())
    fns.sort(lambda a, b : cmp(os.path.basename(a).lower(), os.path.basename(b).lower()) or cmp(a.lower(), b.lower()) or cmp(a, b))

    log_split_re    = re.compile(r'^-+$', re.MULTILINE)
    rev_re          = re.compile(r'^r(\d+)')


    fn_diffs        = {}            # keyed by file name, valued by an array of revision numbers
    for fn in fns   :
        cmd         = 'svn log -v -v -r %s %s' % ( rev, fn, )
        r, rs       = tzlib.run_program(cmd)
        if  r       :
            print("Cannot get log for -r %s   %s errno %i!" % ( rev, fn, r, ), file = sys.stderr)
            sys.exit(101)

        rsa         = log_split_re.split(rs)
        for r in rsa :
            r       = r.strip()
            if  r   :
                g   = rev_re.match(r)
                if  not g :
                    print("BUG! No revision number found in %s !" % r)
                    sys.exit(111)

                fn_diffs[fn]    = fn_diffs.get(fn, []) + [ int(g.group(1)) ]
            pass

        pass

    rev_diffs       = {}            # keyed by revision number, valued by dictionary { file_name : svn_diff_output, ... }

    for fn, ra in fn_diffs.items() :
        for r in ra :
            rev_diffs[r]        = rev_diffs.get(r, {})
            rev_diffs[r][fn]    = r
        pass

    revs    = list(rev_diffs.keys())
    revs.sort()
    revs.reverse()
    for rv in revs :
        fns = list(rev_diffs[rv].keys())
        fns.sort(lambda a, b : cmp(os.path.basename(a).lower(), os.path.basename(b).lower()) or cmp(a.lower(), b.lower()) or cmp(a, b))

        print('')
        print("------------------------------------------------------")
        print("r%u" % rv)
        for fn in fns :
            cmd         = 'svn diff -r %u:%u %s' % ( rv - 1, rv, fn, )
            r, rs       = tzlib.run_program(cmd)
            if  r       :
                print("Cannot get diff for -r %u:%u   %s errno %i!" % ( rv, rv - 1, fn, r, ), file = sys.stderr)
                sys.exit(101)

            print(rs)

        pass

    pass

#
#
#
# eof
