#!/usr/bin/python

# tz_hardware.py
#       --copyright--                   Copyright 2012 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       February 7, 2012        bar
#       May 27, 2012            bar     doxygen namespace
#       January 9, 2013         bar     another sensor program output
#       January 11, 2013        bar     fix doxygen namespace error
#       March 5, 2023           bar     future print
#       --eodstamps--
##      \file
#       \namespace              tzpython.tz_hardware
#
#
#       Do hardware things.
#
#

from    __future__  import  print_function

import  os
import  random
import  re
import  sys
import  time

import  tzlib



TOO_HOT_F       = 6.0
TOO_HOT_C       = TOO_HOT_F / 1.8


class   a_temperature(object) :

    def __init__(me, name = None, temp = None, hot = None, blasing = None, fahrenheit = None) :
        me.name         = name  or "Temperature"
        me.temp         = None  if temp    is None else     float(temp)
        me.hot          = None  if hot     is None else     float(hot)
        me.blasing      = None  if blasing is None else     float(blasing)
        me.fahrenheit   = fahrenheit        or None

    def __str__(me)     :
        f               = ''
        if  me.fahrenheit is not None :
            f           = 'F' if me.fahrenheit else 'C'
        ts              = 'Unk'     if  me.temp    is None else     '%5.1f%s' % ( me.temp,    f )
        hs              = ''        if  me.hot     is None else     '%5.1f%s' % ( me.hot,     f )
        bs              = ''        if  me.blasing is None else     '%5.1f%s' % ( me.blasing, f )
        return("%s: %s    Hot:%s    Blasing:%s" % ( me.name, ts, hs, bs ) )

    #   a_temperature


sensor_re           = re.compile(r"^([^\:]+)\:\s*(" + tzlib.float_regx_str + ")[^FC]+[FC]\s*\(high\s*=\s*(" + tzlib.float_regx_str + ")[^FC]+[FC]\s*,\s*crit\s*=\s*(" + tzlib.float_regx_str + ")[^FC]+[FC]\)", re.DOTALL + re.IGNORECASE + re.MULTILINE)

#                               temp1:       +51.5<deg>C  (crit = +104.0<deg>C)
sensor2_re          = re.compile(r"^([^\:]+)\:\s*(" + tzlib.float_regx_str + ")[^FC]+[FC]\s*\((?:high\s*=\s*(" + tzlib.float_regx_str + ")[^FC]+[FC]\s*,\s*)?crit\s*=\s*(" + tzlib.float_regx_str + ")[^FC]+[FC]\)", re.DOTALL + re.IGNORECASE + re.MULTILINE)

def get_temperatures(fahrenheit = False) :
    """ Get an array of whatever temperatures we can get. """

    temps           = []

    if  sys.platform.find('linux') >= 0 :
        scl         = ''
        if  fahrenheit :
            scl     = ' -f'
        ( r, rs )   = tzlib.run_program('sensors %s' % scl)
        if  not r   :
            rs      = rs.decode('utf8')                 # decode the degree character
            tmps    = sensor_re.findall(rs)
            if  not len(tmps) :
                tmps    = sensor2_re.findall(rs)
            if  len(tmps)   :
                mxl         = max([ len(t[0]) for t in tmps ])
                for t in tmps :
                    blaz    = t[((len(t) >= 4) and 3)  or 2]
                    hot     = ((len(t) >= 4) and t[2]) or blaz
                    temps.append(a_temperature(("%*s" % ( mxl, t[0] )), t[1], hot, blaz, fahrenheit))
                pass
            pass
        pass

    return(temps)



can_get_temps   = True
def is_hot()    :
    global  can_get_temps

    if  can_get_temps :
        temps   = get_temperatures()
        if  not len(temps) :
            can_get_temps   = False
        else    :
            for t in temps :
                if  t.temp >= t.hot + (TOO_HOT_F if t.fahrenheit else TOO_HOT_C) :
                    return(True)
                pass
            pass
        pass

    return(False)


def is_blasing()    :
    global  can_get_temps

    if  can_get_temps :
        temps   = get_temperatures()
        if  not len(temps) :
            can_get_temps   = False
        else    :
            for t in temps :
                if  t.temp >= t.blasing :
                    return(True)
                pass
            pass
        pass

    return(False)



def slow_down_if_hot(max_time_to_slow = None) :
    max_time_to_slow    = max_time_to_slow  or .089
    r   = is_hot()
    if  r :
        time.sleep(random.random() * max_time_to_slow)
    if  is_blasing() :
        time.sleep(max(0.5, max_time_to_slow) + (random.random() * max_time_to_slow))

    return(r)




##      Command line help string.
help_str    = """
%s (options)

Do hardware things.

Options:

    --fahrenheit                Show temperatures in fahrenheit.

"""


#
#
#
if __name__ == '__main__' :
    import  TZCommandLineAtFile


    program_name    = sys.argv.pop(0)
    TZCommandLineAtFile.expand_at_sign_command_line_files(sys.argv)


    if  tzlib.array_find(sys.argv, [ "--help", "-h", "-?", "/?", ] ) >= 0 :
        print(help_str % ( os.path.basename(program_name), ))
        sys.exit(254)

    fahrenheit      = False

    while True :
        oi  = tzlib.array_find(sys.argv, [ "--fahrenheit", "-f", ])
        if  oi < 0  :   break
        del sys.argv[oi]
        fahrenheit  =   True


    temps   = get_temperatures(fahrenheit = fahrenheit)

    for t in temps :
        print(t)

    if  slow_down_if_hot() :
        print("Slowed down")

    pass


#
#
# eof
