#!/usr/bin/python

# tz_rss.py
#       --copyright--                   Copyright 2008 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       October 21, 2008        bar
#       November 29, 2011       bar     pyflake cleanup
#       May 27, 2012            bar     doxygen namespace
#       March 1, 2023           bar     python3
#       --eodstamps--
##      \file
#       \namespace              tzpython.tz_rss
#
#
#       RSS stuff.
#
#       Program creates an rss file.
#
#       Besides w3schools there is this, which is handy: http://cyber.law.harvard.edu/rss/rss.html
#
#
#


from    __future__  import  print_function

try :
    import  email.utils as email_utils
except ImportError :
    import  email.Utils as email_utils
import  time

try :
    basestring
except NameError :
    basestring  = str



class   a_thing(object) :

    @staticmethod
    def date_str(t = None) :
        return(email_utils.formatdate(t or time.time()))            # rfc822 date time in string form


    def __init__(me, title = "", link = "", description = "") :

        me.title        = title         or "tz_rss RSS title"
        me.link         = link          or "http://www.tranzoa.net/~alex/"
        me.description  = description   or "tz_rss RSS description"


    def maybe_cdata_element(me, name) :
        if  (not hasattr(me, name)) or (not getattr(me, name)) or (not isinstance(getattr(me, name), basestring)) :
            return("")

        text        = getattr(me, name)
        if  text.find("<") >= 0 :
            text    = "<![CDATA[%s]]>" % text

        return("<%s>%s</%s>\r\n" % ( name, text, name ) )


    def str_all_names(me) :
        s           = ""
        for name in dir(me) :
            if  not name.startswith("_") :
                s  += me.maybe_cdata_element(name)
            pass
        return(s)


    #   a_thing




class   an_image(a_thing) :

    def __init__(me, title = "", link = "", url = "") :
        super(an_image, me).__init__(title, link)

        me.url          = url or me.link

        delattr(me, "description")



    def __str__(me) :
        s   = ""

        s  +=   "<image>\r\n"
        s  +=   me.str_all_names()
        s  +=   "</image>\r\n"

        return(s)


    #   an_item



class   an_item(a_thing) :

    def __init__(me, title = "", link = "", description = "") :
        super(an_item, me).__init__(title, link, description)

        me.author       = None
        me.category     = None
        me.comments     = None
        me.enclosure    = None
        me.guid         = None
        me.pubDate      = None
        me.source       = None



    def __str__(me) :
        me.guid         = me.guid or me.link

        s   = ""

        s  +=   "<item>\r\n"
        s  +=   me.str_all_names()
        s  +=   "</item>\r\n"

        return(s)


    #   an_item



class   a_channel(a_thing) :

    def __init__(me, title = "", link = "", description = "", items = None, url = None) :
        super(a_channel, me).__init__(title, link, description)

        me.image            = None
        me._url             = url or None

        me.items            = items or []

        me.category         = None
        me.cloud            = None
        me.copyright        = None
        me.docs             = None
        me.generator        = None
        me.language         = None
        me.lastBuildDate    = me.date_str()
        me.pubDate          = None
        me.managingEditor   = None
        me.rating           = None
        me.skipDays         = None
        me.skipHours        = None
        me.textInput        = None
        me.ttl              = None
        me.webMaster        = None



    def __str__(me) :
        s       = ""

        s      +=   '<?xml version="1.0" encoding="ISO-8859-1" ?>\r\n'
        s      +=   '<rss version="2.0" xmlns:atom="http://www.w3.org/2005/Atom">\r\n'
        s      +=   "<channel>\r\n"

        if  me._url :
            s  +=   '<atom:link href="%s" rel="self" type="application/rss+xml" />\r\n' % me._url

        s      +=   me.str_all_names()

        if  me.image :
            s  += str(me.image)

        for i in me.items :
            s  += str(i)

        s      +=   "</channel>\r\n"
        s      +=   "</rss>\r\n"

        return(s)


    #   a_channel

#
#
#
if __name__ == '__main__' :

    import  sys

    import  TZCommandLineAtFile
    import  replace_file


    del(sys.argv[0])

    TZCommandLineAtFile.expand_at_sign_command_line_files(sys.argv)

    if  len(sys.argv) < 1 :
        print("Tell me an output file name!")
        sys.exit(101)

    rss = a_channel()
    rss.description = 'This is a <A HREF="http://www.tranzoa.com">feed</A>'         # test CDATA


    ofile_name  = sys.argv.pop(0)

    tfn = ofile_name + ".tmp"

    fo  = open(tfn, "w")

    fo.write(str(rss))

    fo.close()

    replace_file.replace_file(ofile_name, tfn, ofile_name + ".bak")


#
#
#
# eof
