#!/usr/bin/python

# tz_src.py
#       --copyright--                   Copyright 2012 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       September 12, 2012      bar
#       September 14, 2012      bar     svn dir and help
#       September 20, 2012      bar     fix a scope problem in the svn logic
#       March 5, 2023           bar     future print
#       April 13, 2023          bar     listize keys values items
#       --eodstamps--
##      \file
#       \namespace              tzpython.tz_src
#
#
#       Do what I need to do with source files.
#
#


from    __future__  import  print_function

import  re
import  time

import  tzlib
import  tz_parse_time


copyright_re    = re.compile("--copyright--\s*([^\r\n]+)",                  re.DOTALL)
url_re          = re.compile("--url--\s*([^\r\n]+)",                        re.DOTALL)
email_re        = re.compile("--email--\s*([^\r\n]+)",                      re.DOTALL)
bodstamps_re    = re.compile("--bodstamps--\s*?([^\r\n]*)",                 re.DOTALL)
eodstamps_re    = re.compile("--eodstamps--\s*?([^\r\n]*)",                 re.DOTALL)
comments_re     = re.compile("--bodstamps--(.*?)\n[;#]?\s*--eodstamps--",   re.DOTALL)


class   a_dated_comment(object) :
    def __init__(me, when = None, who = None, comments = []) :
        me.when     = when
        me.who      = who
        me.comments = comments
    def append(me, cmt) :
        me.comments.append(cmt)
    def __str__(me) :
        ts          = (me.when and time.strftime("%B %d, %Y", time.gmtime(me.when))) or ""
        return("Comment:%s by{%s}%s" % ( ts, (me.who or "UNK"), ((len(me.comments) and (" " + me.comments[0] + (((len(me.comments) > 1) and "...") or ""))) or "") ) )
    #   a_dated_comment


class   a_source_file(object) :

    def __init__(me, file_name) :
        fd  = tzlib.read_whole_text_file(file_name)

        for nm in [ 'copyright', 'url', 'email', "comments", "bodstamps", "eodstamps", ] :
            g   = globals()[nm + "_re"].search(fd)
            setattr(me, nm, (g and g.group(1)) or None)

        me.all_comments     = []
        if  me.comments     :
            me.all_comments = [ cmt.strip() for cmt in re.split(r"\r?\n", me.comments) if cmt.strip() ]

        me.dated_comments   = []
        cmt                 = None
        for c in me.all_comments :
            when            = tz_parse_time.parse_time(c)
            if  when        :
                if  cmt     :
                    me.dated_comments.append(cmt)
                who         = None
                g           = re.search(r".*?,\s+\d\d\d\d\s+(\S{1,4})(\s+.*|$)", c)
                if  g       :
                    who     = g.group(1)
                    if  g.lastindex == 2 :
                        c   = g.group(2).strip()
                    else    :
                        c   = ""
                    pass
                cmt         = a_dated_comment(when, who, [ c ])
            elif cmt        :
                c           = re.sub(r"[;#]?", "", c).strip()
                cmt.append(c)
            else            :
                me.dated_comments.append(a_dated_comment(0, "", [ c  ]))
            pass
        if  cmt             :
            me.dated_comments.append(cmt)

        pass


    def __str__(me) :
        return("Comment day count = %u" % len(me.dated_comments))


    #   a_source_file




def get_svn_when(svn, dr = None) :
    svn         = re.escape(svn)

    ( r, rs )   = tzlib.run_program("svn log -r HEAD:1 %s" % (dr or ""))

    if  r       :
        raise OSError("Cannot run svn")

    ra  = [ s.strip() for s in rs.split("------------------------------------------------------------------------") if re.search(svn, s) ]

    for e in ra :
        g       = re.search(r"( \| \d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d [^(]*)", e)
        if  g   :
            w   = tz_parse_time.parse_time(g.group(1))
            if  w :
                return(w)
            pass
        pass

    return(None)



help_str    = """
%s (options) ambiguous_files...

Options:

    --sub_dirs                  Include matching files in subdirectories.
    --output    file_name       Print to given file.

Print comments from source files.

OR:

%s --svn  string_to_search_for  directory_for_log  (options)  ambiguous_files...

Look for the given string in the log file and print only comments from that entry's day, on.
"""

if  __name__ == '__main__' :
    import  os
    import  sys

    import  TZCommandLineAtFile


    program_name    = sys.argv.pop(0)
    TZCommandLineAtFile.expand_at_sign_command_line_files(sys.argv)


    svn             = None
    svn_dir         = None
    ofile_name      = None
    sub_dirs        = False


    while True :
        oi  = tzlib.array_find(sys.argv, [ "--help", "-h", "-?", "/?", "/h", "/H" ] )
        if  oi < 0 :    break
        del sys.argv[oi]
        print(help_str % ( os.path.basename(program_name), os.path.basename(program_name), ))

        sys.exit(254)


    while True :
        oi          = tzlib.array_find(sys.argv, [ "--svn", ])
        if  oi < 0  : break
        del(sys.argv[oi])
        svn         = sys.argv.pop(oi)
        svn_dr      = sys.argv.pop(oi)

    while True :
        oi          = tzlib.array_find(sys.argv, [ "--output", "-o" ])
        if  oi < 0  : break
        del(sys.argv[oi])
        ofile_name  = sys.argv.pop(oi)

    while True :
        oi          = tzlib.array_find(sys.argv, [ "--sub_dirs", "--subdirs", "--sub-dirs", "--do_sub_dirs", "--dosubdirs", "--do-sub-dirs", "-s", ])
        if  oi < 0  : break
        del(sys.argv[oi])
        sub_dirs    = True


    fo              = sys.stdout
    if  ofile_name  :
        fo          = open(ofile_name, "wt")


    if  svn :

        try         :
            when    = get_svn_when(svn, dr = svn_dr)
        except OSError :
            print("Cannot run svn!", file = sys.stderr)
            sys.exit(101)

        if  not when :
            print("No svn time found!", file = sys.stderr)
            sys.exit(101)

        fns = {}
        for fn in sys.argv :
            fa      = tzlib.ambiguous_file_list(fn, do_sub_dirs = sub_dirs)
            for fn in fa :
                fns[os.path.abspath(fn)]    = fn
            pass

        ca  = []
        fns = list(fns.values())
        fns.sort(lambda a, b : cmp(a.lower(), b.lower()))
        for fn in fns :
            f   = a_source_file(fn)
            for ci, c in enumerate(f.dated_comments) :
                if  c.when + 24 * 60 * 60 >= when :
                    print("    %s" % fn, file = fo)
                    for c in f.dated_comments[ci:] :
                        print("        %-24s%-8s%s" % ( ((c.when and time.strftime("%B %d, %Y", time.gmtime(c.when))) or ""), (c.who or ""), c.comments[0], ), file = fo)
                        for cs in c.comments[1:] :
                            print("                                        %s" % cs, file = fo)
                        pass
                    break
                pass
            pass

        pass

    else    :

        while len(sys.argv) :
            fn  = sys.argv.pop(0)

            im  = a_source_file(fn)

            if  im.dated_comments :
                print(fn, im, "First [", im.dated_comments[0], "] Last [", im.dated_comments[-1], "]", file = fo)
            else :
                print(fn, im, file = fo)
            pass
        pass

    if  ofile_name :
        fo.close()

    pass


#
#
# eof
