/*
tz_stream_graph_client.js
        --copyright--                   Copyright 2011 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
        --url--                         http://www.tranzoa.net/tzpython/
        --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
        --bodstamps--
        September 29, 2011      bar
        September 30, 2011      bar     continue
        October 3, 2011         bar     let's get painting
        October 4, 2011         bar     handle tab being closed
        October 10, 2011        bar     play with server hit timeout (epiphany doesn't paint - it seems too busy polling the server now matter what)
        October 29, 2011        bar     don't over-send on the post
        February 22, 2012       bar     fix the stream path
        May 28, 2012            bar     doxygen
        March 8, 2015           bar     jsl
        March 15, 2015          bar     fixed the awful confusion about the stream.nam, including the typo in remove_stream()
        September 30, 2017      bar     nmrate and ticks
                                        able to turn painting on and off
        October 1, 2017         bar     proper, longer ticks and on top, too, and blackish
        --eodstamps--

*/
/** \file
    \namespace                          tzpython.tz_stream_graph_client

        Client side code to paint a streaming point/line graph in an HTML5 browser.

*/


var TICK_HITE   = 10;


function    a_stream_graph_stream(nam, url, id)
{
    this.nam    = nam;                  /* a unique id for this stream here on the client */
    this.ci     = '';                   /* server's client ID - set by the stream's downloaded code. We just pass it back to server. */
    this.url    = url;                  /* the url */
    this.sid    = id;                   /* stream ID that server knows it by */
    this.vc     = 0;                    /* how many values we've gotten - set by the streams's downloaded code and passed back to the server next hit */
    this.ttl    = '';                   /* title / display name */
    this.nms    = [];                   /* array of numbers to graph */
    this.nmrate = 0;                    /* number of numbers per displayed tick - <= 1.0 means unknown, do not show any ticks */
    this.cls    = [];                   /* colors of each number - numbers beyond end of this array take on the last color in the array */
    this.wds    = [];                   /* widths of each number's line to next number or of the number's point */
    this.tmout  = 0;                    /* hit the server after this many milliseconds */
}


a_stream_graph_stream.prototype.min     = function()
{
    return(this.nms.length ? Math.min.apply(null, this.nms) : 0);
};

a_stream_graph_stream.prototype.max     = function()
{
    return(this.nms.length ? Math.max.apply(null, this.nms) : 0);
};



a_stream_graph_stream.prototype.draw_line   = function(ctx, w, h, mn, nums)
{
    if  (nums.length > 1)
    {
        var i   = Math.max(0, nums.length - w);
        var x   = w - (nums.length - i);
        var y;

        ctx.beginPath();
        y       = Math.max(0, Math.min(h, nums[i] - mn));
        ctx.moveTo(x, -y);
        for (; x < w; ++i, ++x)
        {
            y   = Math.max(0, Math.min(h, nums[i] - mn));
            ctx.lineTo(x, -y);
        }
        ctx.stroke();
    }
};


a_stream_graph_stream.prototype.draw_ticks  = function(ctx, w, h, sh)
{
    if  (this.nmrate > 1.0)
    {
        var sm  = sh / h;
        var ss  = ctx.strokeStyle;
        ctx.strokeStyle = '#404040ff';
        for (x  = this.nmrate - this.nms.length % this.nmrate; x < w; x += this.nmrate)
        {
            ctx.beginPath();
            ctx.moveTo(x, 0);
            ctx.lineTo(x,     - (TICK_HITE * sm));
            ctx.moveTo(x, -sh);
            ctx.lineTo(x, -sh + (TICK_HITE * sm));
            ctx.stroke();
        }
        ctx.strokeStyle = ss;
    }
};


a_stream_graph_stream.prototype.paint   = function(client, ctx, w, h, mn, mx)
{
    this.draw_ticks(ctx, w, h, Math.max(1.0, mx - mn));
    this.draw_line( ctx, w, h, mn, this.nms);
};





function    a_stream_graph_client(div_element)
{
    this.XY_TYPE_NORM           = 0;
    this.XY_TYPE_LOG            = 1;
    this.TIMEOUT                = 257;                  /* milliseconds 'tween paints */
    this.width                  = 600;
    this.hite                   = 400;

    this.div_el                 = div_element;          /* this <div> we control and live in */
    this.snm                    = 1;                    /* the next stream number - always increments */
    this.polling                = true;                 /* are we polling the server(s)? */
    this.x_type                 = this.XY_TYPE_NORM;    /* is the X scale logrithmic? */
    this.y_type                 = this.XY_TYPE_NORM;    /* is the Y scale logrithmic? */
    this.streams                = {};                   /* all the streams we are polling, keyed by snm - the (incrementing) stream number we give each stream */
    this.reqs                   = {};                   /* all xmlhttp requests we've got going, keyed by url */
    this.npnt                   = false;                /* do we need to paint the graph right now? */
    this.painter                = null;
    this.painting               = true;                 /* are we painting? or if false, frozen? */

    this.log_el                 = document.getElementById('log');

    this.div_el.style.width     = this.width + 4;
    this.div_el.style.height    = this.hite  + 4;

    this.div_el.innerHTML       = '<canvas></canvas>';
    this.canvas                 = this.div_el.firstChild;
    if (!this.canvas || (this.canvas.tagName != 'CANVAS'))
    {
        this.div_el.innerHTML   = 'I only work on HTML5 Canvas browsers like Firefox / Chrome / Webkit-based (Safari and modern phones)';
        this.canvas             = null;
    }
    else
    {
        this.div_el.style.borderWidth   = '2px';
        this.div_el.style.borderStyle   = 'solid';
        this.canvas.style.margin        = '1px';
        this.canvas.style.padding       = '1px';
        this.canvas.style.width         = this.width;
        this.canvas.style.height        = this.hite;
        this.canvas.setAttribute('width',    this.canvas.style.width);          /* note: if these aren't set, the canvas isn't painted to or the lines are too thick or whatever. Anyway, depending on the program, this setAttribute seems to do good magic. */
        this.canvas.setAttribute('height',   this.canvas.style.height);

        /***    !!!! combine all streams with the same URL in one request ***/

        this.needs_paint();

        var me              = this;
        var tmout_fn        = function(me)
        {
            if (me.painter)
            {
                window.clearTimeout(me.painter);
                me.painter  = null;
            }

            me.paint();
            me.painter      = window.setTimeout(function () { tmout_fn(me); }, this.TIMEOUT);
        };

        tmout_fn(me);
    }
}


a_stream_graph_client.prototype.paint   = function()
{
    if (this.npnt && this.canvas && this.painting)
    {
        this.npnt           = false;

        var w               = this.canvas.width;
        var h               = this.canvas.height;
        this.canvas.width   = w;                            /* this resets all the drawing stuff and erases the canvas. It also stops FF from killing X to need either a looong wait or a reboot. */

        var ctx             = this.canvas.getContext("2d");
            ctx.clearRect(0, 0, w, h);
            ctx.strokeStyle = "rgba(0, 0, 255, 1.0)";
            ctx.lineCap     = "round";
            ctx.lineWidth   = 1;

        if  (this.streams)
        {
            var s0 = null;
            var si;

            for (si in this.streams)
            {
                s0          = this.streams[si];
                break;
            }
            if (s0)
            {
                var mn          = 0.0 + s0.min();                       /* auto-scale the canvas to handle all streams - which better be on about the same scales, themselves in X and Y !!!! */
                var mx          = 0.0 + s0.max();
                for (si in this.streams)
                {
                    mn          = Math.min(mn, this.streams[si].min());
                    mx          = Math.max(mn, this.streams[si].max());
                }

                ctx.translate(0, h);

                ctx.scale(1.0, h / Math.max(1.0, mx - mn));

                for (si in this.streams)
                {
                    this.streams[si].paint(this, ctx, w, h, mn, mx);
                }
            }
        }
    }
};




a_stream_graph_client.prototype.needs_paint = function()
{
    this.npnt   = true;
};


a_stream_graph_client.prototype.do_srv_hit  = function(ss)      /* note: xmlhttp may be better for positive control over the results of the server hit, but if there's any chance of going cross-site, then iframes don't have the OPTION hit */
{
    if  (this.polling)
    {
        var req;
        if  (window.XMLHttpRequest)
        {
            req  = new XMLHttpRequest();
        }
        else
        {
            req  = new ActiveXObject("Microsoft.XMLHTTP");
        }

        req._client = this;         /* give the request result js code access to everything */
        req._ss     = ss;           /* remember a stream so we can fire another request up again */

        req.onreadystatechange      =   function()
        {
            if  (this.readyState   == 4)
            {
                if  (this.status   == 200)
                {
                    this._ss.tmout  = Math.max(200.0, this._ss.tmout / 2.0);
                    if  (this._client.polling && (this._client.reqs[this._ss.url] === this))
                    {
                        this._ss.tmout  = 50.0;                 /* note: the server can change this because we set it before doing his code */
                        var client      = this._client;
                        var ss          = this._ss;
                        client.needs_paint();
                        eval(this.responseText);
                    }
                }
                else
                {
                    this._ss.tmout  = Math.min(20000.0, (this._ss.tmout + 100.0) * 2.7);
                    /*** window.console.log("wait a while"); ***/
                }

                var fn              = function() { req._client.do_srv_hit(req._ss); };
                try
                {
                    this._ss.req    = window.setTimeout(fn, this._ss.tmout);
                }
                catch (err)
                {
                    this._ss.req    = null;
                }
            }
        };

        this.reqs[req._ss.url]  = req;

        req.open("POST", req._ss.url, true);
        req.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
        var nop;
        try
        {
            req.send('ci=' + req._ss.ci + '&si=' + req._ss.sid + '&csi=' + req._ss.nam + '&vc=' + req._ss.vc + '&rn=' + Math.random());         /* note: field names must match those in the server */
        }
        catch (err)
        {
            nop = err;         /* note: can't go to a file:// */
        }
    }
};


a_stream_graph_client.prototype.log = function(htm, no_br)
{
    if  (this.log_el)
    {
        this.log_el.innerHTML      +=  htm;
        if  (!no_br)
        {
            this.log("<br>\n", true);
        }

        if (this.log_el.innerHTML.length > 2000)
        {
            this.log_el.innerHTML   = this.log_el.innerHTML.slice(this.log_el.innerHTML.length - 1000);
        }
    }
};


a_stream_graph_client.prototype.dump_object = function(ob)
{
    for (var k in ob)
    {
        this.log("<br>" + k + ':' + ob[k], false);
    }
    this.log('');
};


function    parse_url(url)
{
    var el      = document.createElement("a");
        el.href = url;
    return(el);         /* el.protocol==http:  el.host    el.hostname   el.pathname   el.port   el.url */
}


a_stream_graph_client.prototype.add_stream  = function(url, sid)
{
    url     = url ? url : '';
    if (url && (url.substr(0, 4) !== 'http'))
    {
        url = window.location.pathname + url;
        url = url.replace(/\/+/g, "/");
        /*** this.log("url [" + url + "][" + window.location.pathname + "]", false); ***/
    }
    var sn  = this.snm;
    ++this.snm;                                                     /* note: kludge to avoid warning from jslint */
    var ss  = new a_stream_graph_stream(sn, url, sid);
    this.streams[ss.nam]    = ss;

    /***    start polling if it's a new URL ***/
    if  (url && !(url in this.reqs) && this.canvas)
    {
        this.do_srv_hit(ss);
    }

    return(ss);
};


a_stream_graph_client.prototype.remove_stream   = function(ss)
{
    del(this.streams[ss.nam]);
};


a_stream_graph_client.prototype.set_polling   = function(how)
{
    var ov  = this.polling;
    if (how !== null)
    {
        this.polling    = polling ? true : false;
        if (!this.polling)
        {
            for (var k in this.streams)
            {
                var ss                  = this.streams[k];
                this.streams[ss.nam]    = new a_stream_graph_stream(ss.nam, ss.url, ss.sid);        /* start the streams over again, when we start polling again */
            }
        }
    }

    return(ov);
};


a_stream_graph_client.prototype.set_painting    = function(how)
{
    var ov  = this.painting;
    if  ((how === true) || (how === false))
    {
        this.painting   = how;
    }
    return(ov);
};




/* eof */
