#!/usr/bin/python

# tz_timer.py
#       --copyright--                   Copyright 2013 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       February 19, 2013       bar
#       September 23, 2013      bar     correct namespace
#       June 19, 2020           bar     python3
#       February 21, 2023       bar     able to be run and not run forever (for tz_lib_test.py)
#       --eodstamps--
##      \file
#       \namespace              tzpython.tz_timer
#
#
"""
    Routines for getting incrementing high precision timer.

    clock_gettime() code lifted from:
        http://stackoverflow.com/questions/1205722/how-do-i-get-monotonic-time-durations-in-python

"""

import  sys


__all__ = [ "tick" ]


CLOCK_MONOTONIC     = 1     # see <linux/time.h>
CLOCK_MONOTONIC_RAW = 4     # modern linux systems - this is apparently immune from ntp effects, but testing shows that on Ubuntu Lucid, 1 works just as well

prev_tick               = 0.0
op                      = CLOCK_MONOTONIC_RAW

def _hpet_tick() :
    """ Return a high precision, incrementing time using the POSIX routine. """
    global  prev_tick, op

    ts   = timespec()
    if  clock_gettime(    op, ctypes.pointer(ts)) != 0:
        op          = CLOCK_MONOTONIC
        if  clock_gettime(op, ctypes.pointer(ts)) != 0:
            errno_  = ctypes.get_errno()

            raise OSError(errno_, os.strerror(errno_))

        pass

    t   = ts.tv_sec + (ts.tv_nsec * 1e-9)
    if  t   < prev_tick :
        t   = prev_tick             # just in case we are getting NTP-adjusted ticks, try to fix things for the moment as best we can
        print ("@@@@")
        exit(1)

    return(t)


tick        = _hpet_tick
if  sys.platform.find('win32') >= 0 :
    import  time

    tick    = time.clock

else        :

    import  ctypes
    import  os

    class timespec(ctypes.Structure):
        _fields_ =  [
                        ('tv_sec',  ctypes.c_long),
                        ('tv_nsec', ctypes.c_long)
                    ]
        #   timespec

    librt                   = ctypes.CDLL('librt.so.1', use_errno = True)
    clock_gettime           = librt.clock_gettime
    clock_gettime.argtypes  = [ ctypes.c_int, ctypes.POINTER(timespec) ]


if  __name__ == "__main__":
    cnt = sys.maxsize
    if  len(sys.argv) > 1 :
        cnt = int(sys.argv[1])
    c   = 0
    while (cnt < 0) or (c < cnt) :
        c  += 1
        print (tick())
    pass


#
#
# eof

