#!/usr/bin/python
# -*- coding: utf8 -*-

# tzemail.py
#       --copyright--                   Copyright 2007 (C) Tranzoa, Co. All rights reserved.    Warranty: You're free and on your own here. This code is not necessarily up-to-date or of public quality.
#       --url--                         http://www.tranzoa.net/tzpython/
#       --email--                       pycode is the name to send to. tranzoa.com is the place to send to.
#       --bodstamps--
#       August 30, 2005         bar
#       October 19, 2006        bar     expose the re's
#       October 27, 2006        bar     dnsbl_count
#       October 31, 2006        bar     fix a comment
#       May 28, 2007            bar     email_adr_re
#       November 18, 2007       bar     turn on doxygen
#       November 27, 2007       bar     insert boilerplate copyright
#       May 17, 2008            bar     email adr
#       August 29, 2008         bar     basestring instead of StringType because of unicode strings and others
#       May 18, 2009            bar     show_info for dnsbl spam check
#       May 23, 2009            bar     demote sorbs from the dnsrbl list (poisoned? google's server is a spammer and ohio seems to have a problem with them, too)
#       November 29, 2011       bar     pyflake cleanup
#       May 27, 2012            bar     doxygen namespace
#       June 2, 2013            bar     mail_files()
#       August 20, 2013         bar     mail_files sets the file name as the base file name, not the whole path
#       September 24, 2013      bar     file_name_filter to mail_files()
#       November 27, 2013       bar     subject is only the base file name
#       April 5, 2014           bar     mail_files() can take a list
#       May 27, 2014            bar     mail_files() can take text
#       April 10, 2016          bar     mail_files() sends only one email for all the files
#       March 1, 2023           bar     python3
#       March 6, 2023           bar     quoted-printable issues with python2/3. not a good solution, but at least it doesn't crash - test in tz_lib_test.py
#       --eodstamps--
##      \file
#       \namespace              tzpython.tzemail
#
#
#       Various email routines, which really shouldn't be used, but they are quick and easy.
#
#       Regex to pick off well-formed email addresses in text:
#
#           \b[A-Z0-9._%-]+@[A-Z0-9.-]+\.[A-Z]{2,4}\b
#
#

from    __future__  import  print_function

import  copy
import  email.mime.application
import  email.mime.text
import  email.mime.multipart
import  os
import  re
import  smtplib
import  socket

import  tzlib
import  tz_parse_time


try :
    basestring
except NameError :
    basestring  = str

try :
    xrange
except NameError :
    xrange  = range



strip_eol_re    =   re.compile(r"[\r\n]+$",               re.DOTALL + re.MULTILINE)
# strip_eol_re  =   re.compile(r"\r?\n$", )
from_re         =   re.compile(r"^From .*\b(19|20)\d\d\b")
drop_hdr_re     =   re.compile(r"^\S+\s+")


email_adr_re    =   re.compile(r"\b[A-Z0-9._%-]+@[A-Z0-9.-]+\.[A-Z]{2,4}\b", re.IGNORECASE)



def read_mbox_file(fi) :
    """
        Read emails from a Unix/Netscape mbox/mailbox file.
        'fi' is either:
            open, binary file
            a file name of an existing file

        Return an array of emails.

            Each email is an array of text lines without EOL characters.
            The first text line of each email will be the "From ..." line,
            unless the file is simply one email without a "From ..." line.
    """

    fr  = None
    if  not hasattr(fi, 'read') :
        fi  =   fr  = open(fi, "rb")                    # binary 'cause of Control Z's

    emails  = []
    emale   = []

    while True :
        li  = fi.readline()
        if  not li :    break

        li  = tzlib.convert_to_unicode(li)
        li  = strip_eol_re.sub("", li)

        # print(li)

        if  from_re.match(li) :
            if  len(emale) :    emails.append(emale)
            emale   = []
        emale.append(li)

    if  len(emale) :            emails.append(emale)

    if  fr  :
        fr.close()

    return(emails)



def from_time(li) :
    """
        Return a Unix time value from a "From - Fri Oct 27 16:57:16 2006"
        line such as is the first line of emails in mailbox files.

        Return -1 if there is no time.
    """

    t   = tz_parse_time.parse_time(li)
    t   = t or -1

    return(t)





def read_email_from_file(fi) :
    """
        Read an email from a file (e.g. sys.stdin) in to an array of text lines lacking EOL characters.

        'fi' is either:
            open, binary file
            a file name of an existing file
    """

    fr  = None
    if  not hasattr(fi, 'read') :
        fi  =   fr  = open(fi, "rb")                    # binary 'cause of Control Z's

    emale   = []
    while True :
        li  = fi.readline()
        if  not li : break

        if  tzlib.is_byteish(li) :                      # python3 bytes?
            li  = tzlib.convert_to_unicode(li)          # dangerous. under python3 we don't do this for utf8 (?) strings from stdin (already converted) but do do this for bytes from a file
        li      = strip_eol_re.sub("", li)

        # print(li)

        emale.append(li)

    if  fr  :
        fr.close()

    return(emale)


def get_flattened_email_headers_array(emale) :
    """
        Given an array of text lines lacking EOL characters in blank-line-separates-header-from-body form,
        return an array with multi-line headers combined in to single lines.
    """
    hdrs    = []
    for li  in emale :
        if  len(li) == 0 :  break

        c1  = li[0:1]
        if  ((c1 == ' ') or (c1 == '\t')) and (len(hdrs) > 0):
            hdrs[len(hdrs) - 1] += li
        else :
            hdrs.append(li)
        pass

    if  len(hdrs) == 0 :    return("")

    return(hdrs)



def get_flattened_email_headers_str(emale) :
    """
        Given an array of text lines lacking EOL characters in blank-line-separates-header-from-body form,
        return a string with multi-line headers combined in to single lines.
    """
    hdrs    = get_flattened_email_headers_array(emale)

    if  len(hdrs) == 0 :    return("")

    return("\n".join(hdrs) + "\n")


def get_flattened_email_headers(emale) :
    """
        Given an array of text lines lacking EOL characters in blank-line-separates-header-from-body form,
        return a string with multi-line headers combined in to single lines.
    """

    return(get_flattened_email_headers_str(emale))


def get_email_body_string(emale) :
    """
        Given an array of text lines lacking EOL characters in blank-line-separates-header-from-body form,
        return a string containing the body of the email, LF line delimited.
    """

    try :
        i   = emale.index("") + 1
    except ValueError :
        i   = len(emale)
    if  i  >= len(emale) :  return("");

    return("\n".join(emale[i:]) + "\n")




def add_email_header(emale, hdr) :
    """
        Add the header line to an email (which is an array of text lines lacking EOL characters)
    """

    i   =   0
    while i < len(emale) :
        li  = emale[i]
        if  len(li) == 0 :  break
        i   += 1
    emale.insert(i, hdr)



def _fixed_header_name(hdr) :
    hdr =   hdr.lower().strip()
    if  not re.search(r":\s*$", hdr) :
        hdr += ": "

    return(hdr)



def eliminate_email_header(emale, hdr) :
    """
        Get rid of the given email header.
            'hdr' is case insensitive, optionally including the colon:space at the end.
            'emale' is an array of text lines lacking EOL characters.
    """

    hdr =   _fixed_header_name(hdr)

    i   =   0
    while i < len(emale) :
        li  = emale[i]
        if  len(li) == 0 :  break

        if  not li.lower().startswith(hdr) :
            i   += 1
        else :
            while True :
                del(emale[i])
                if  i >= len(emale) :   break

                li  = emale[i]
                if  len(li) == 0 :      break

                c1  = li[0:1]
                if  ((c1 != ' ') and (c1 != '\t')) :
                    break
                pass
            pass
        pass
    pass



def find_email_headers(emale, hdr) :
    """
        Get an array of flattened headers whose names match the desired header name.
            'hdr' is case insensitive, optionally including the colon:space at the end.
            'emale' is an array of text lines lacking EOL characters.
    """

    hdr     = _fixed_header_name(hdr)

    emale   = get_flattened_email_headers_array(emale)

    hdrs    = filter(lambda h : h.lower().startswith(hdr), emale)

    headers = list(map(lambda h : drop_hdr_re.sub(r"", h).strip(), hdrs))

    return(headers)



#
#
#       Big list of 'em at http://www.robtex.com/rbls/127.0.0.2.html
#       And, another at http://shopping.declude.com/Articles.asp?ID=97
#       And, another at http://www.dnsstuff.com/tools/ip4r.ch?ip=
#
#

black_hole_list = [
                    [ 'sbl-xbl.spamhaus.org',       5 ],

                    [ 'bl.spamcop.net',             5 ],
                    [ 'spam.dnsbl.sorbs.net',       1 ],
                    [ 'spam.dnsrbl.net',            5 ],

                    [ 'dnsbl-3.uceprotect.net',     1 ],

                    [ 'korea.services.net',         1 ],
                    [ 'no-more-funn.moensted.dk',   1 ],
                    [ 'dnsbl.jammconsulting.com',   1 ],
                    [ 'blocklist2.squawk.com',      1 ],
                    [ 'psbl.surriel.com',           1 ],

                    [ 'phishing.rbl.msrbl.net',     1 ],
                    [ 'map.spam-rbl.com',           1 ],
                    [ 'rbl.efnet.org',              1 ],

                    [ 'list.dsbl.org',              1 ],
                    [ 'multihop.dsbl.org',          2 ],

                    [ 'xbl.spamhaus.org',           1 ],
                  ]


#
#
#   dnsbl_count() can be called with these instead of with a black hole list.
#
#

white_list      = [
                    [ 'query.bondedsender.org',       1 ],
                  ]



have_adns   = True


def dnsbl_count(ip_or_name, dnsbl_names   = black_hole_list, stop_on_count = 0, show_info = False ) :
    """
        Look up an IP address or name in DNSBL listings.

        Pass an array of DNSBL hosts (each entry may be an array of [ host_name, count ]) if you want.

        Return ( hit_counts, total_count, [ hitted_dnsbl_name... ] )
    """

    global  have_adns


    stop_on_count   = stop_on_count or 0


    if  (not dnsbl_names) or (len(dnsbl_names) == 0) :
        dnsbl_names = black_hole_list

    dnsbl_names     = copy.deepcopy(dnsbl_names)
    if  not tzlib.is_listish(dnsbl_names) :
        dnsbl_names = [ dnsbl_names, 1 ]


    try :
        ip      = socket.gethostbyname(ip_or_name)
    except socket.gaierror:
        return(-1)                                      # can't find the name or IP adr under question

    ip      = ip.split('.')
    ip.reverse()
    ip      = ".".join(ip) + "."

    # ip      = str(random.randint(1, 255)) + "." + str(random.randint(1, 255)) + "." + str(random.randint(1, 255)) + "." + str(random.randint(1, 255)) + "."

    cnt     = 0
    tcnt    = 0
    hits    = []

    if  have_adns :
        opts    = ""
        for i in xrange(len(dnsbl_names)) :
            dn          = dnsbl_names[i]
            if  not tzlib.is_listish(dn) :
                dn      = [ dn, 1 ]
            opts       += " " + ip + dn[0]

        if  show_info   :
            print(opts)

        ( r, rs )       = tzlib.run_program("adnshost -a -ta " + opts)
        if  rs or (rs.find(dnsbl_names[0][0]) < 0) :
            have_adns   = False
        else :
            for i in xrange(len(dnsbl_names)) :
                tcnt   +=      dn[1]
                w       = ip + dn[0]
                rg      = re.compile(r"\b" + re.escape(w) + r"\s+A\s+127\.0\.\0\.\d+\b", re.DOTALL)
                if  rg.search(rs) :
                    if  show_info :
                        print("hit", w)

                    cnt += dn[1]
                    hits.append(dn[0])
                pass
            pass
        pass

    if  not have_adns :
        for i in xrange(len(dnsbl_names)) :
            dn          = dnsbl_names[i]
            if  not tzlib.is_listish(dn) :
                dn      = [ dn, 1 ]

            tcnt       +=      dn[1]
            w           = ip + dn[0]

            if  show_info :
                print(w)

            try:
                rip     = socket.gethostbyname(w)
                # print(w, ip)
                if  rip.startswith("127.0.0.") :
                    if  show_info :
                        print("hit", w)

                    cnt += dn[1]
                    hits.append(dn[0])
                    if  (stop_on_count > 0) and (cnt >= stop_on_count) :
                        break
                    pass
                pass
            except socket.gaierror:
                pass
            pass
        pass

    return( ( cnt, tcnt, hits ) )           # tell caller how bad things are, how bad things could be, and what lists had the address



def mail_files(amb_file_name, from_adr, to_adr, subject = None, text = None, encoding = None, file_name_filter = None) :     # 0244 Ʉ   00a9 (copyright) ©     0110 D stroke Đ    0692 arabic reh with small v ڒ     3026 hangzhou number six 〦
    """ Mail files matching the given ambiguous name (no files if amb name is "") or mail the files listed. Return a list of the sent files' names. """

    if  isinstance(amb_file_name, basestring) :
        fns = tzlib.ambiguous_file_list(amb_file_name)
    else    :
        fns = list(amb_file_name)           # he passed a list of files

    if  len(fns) or text :
        s   = smtplib.SMTP()
        s.connect()
        msg = email.mime.multipart.MIMEMultipart()
        if  text    :
            text    = tzlib.convert_to_unicode(text).encode('utf8')
            msg.attach(email.mime.text.MIMEText(text, 'plain', 'utf8'))
        ffn         = ""
        for fn in fns :
            fd      = tzlib.read_whole_binary_file(fn)
            if  (not encoding) and re.search(br"[^\r\n\t -\~]", fd) :               # if he's not telling us an encoding and if anything but normal ASCII text is in the file
                m   = email.mime.application.MIMEApplication(fd)                    # send the file
            else    :
                m   = email.mime.text.MIMEText(fd, 'plain', encoding or 'utf8')     # otherwise send text
            fn      = (file_name_filter and file_name_filter(fn)) or fn
            ffn     = ffn or fn
            m.add_header('Content-Disposition', 'attachment', filename = os.path.basename(fn))
            msg.attach(m)

        msg['Subject']  = tzlib.best_ascii(subject or "") or re.sub(r'^(?:\r?\n)*([^\r\n]+)', r'\1', tzlib.best_ascii(text or "")) or (tzlib.best_ascii(os.path.basename(ffn)) + (((len(fns) > 1) and (" " + ("+" * (len(fns) - 1)))) or ""))
        msg['From']     = from_adr
        msg['To']       = to_adr

        ta              = [ to_adr ]
        sa              = [ a for a in to_adr.split(',') if a.strip() ]
        if  len(sa)     > 1 :
            ta          = sa

        msg             = msg.as_string()
        s.sendmail(from_adr, ta, msg)

        s.quit()

    return(fns)


#
#
#
if __name__ == '__main__' :

    import  sys

    show_info   = False
    argv        = sys.argv[1:]

    while len(argv) :
        opt     = argv.pop(0)
        print("Testing:", opt)
        if  opt in [ '--send', '-s' ] :
            sbj = None
            if  len(argv) and (argv[0] in [ '--subject', '--sub', '--sbj', '--sj', '-j', ]) :
                argv.pop(0)
                sbj = argv.pop(0)
            txt = None
            if  len(argv) and (argv[0] in [ '--text', '--txt', '-t', ]) :
                argv.pop(0)
                txt = argv.pop(0)
            enc = None
            if  len(argv) and (argv[0] in [ '--encoding', '--enc', '-e', ]) :
                argv.pop(0)
                enc = argv.pop(0)
            fls = None
            if  len(argv) :
                fls = argv.pop(0)
            fns = mail_files(fls, "tzemail_py@tranzoa.com", "test@tranzoa.com", subject = sbj, text = txt, encoding = enc)      # send the (double quotad on unix) files and any text that follows after the file and optional --enc ENCODING option
            print("    Mailed:", fns)
        elif (not os.path.splitext(opt)[1] or (os.path.splitext(opt)[1].lower() == '.mbox')) and os.path.isfile(opt) :          # analyse an email or mbox file (mbox if with no ext or .mbox)
            emails = read_mbox_file(opt)                            # analyse an mbox file
            print("   ", len(emails), "emails:")
            if  len(emails) :
                for i in range(len(emails)) :
                    print("     %u lines" % len(emails[i]))
                    print("       [0]:", emails[i][0])
                    print("       [1]:", emails[i][1])
                pass
            pass
        elif (opt == '-') or os.path.isfile(opt) :
            fi      = ((opt == '-') and sys.stdin) or opt
            emale   = read_email_from_file(fi)                      # analyse an email
            print("    Email is", len(emale), "lines long.")
            if  len(emale) :
                print("       [0]:", emale[0])
                print("       [1]:", emale[1])
            pass
        else :                                                      # look up the given ip adr (in 'opt') in the black hole lists (this is slow)
            print("    Black hole",     dnsbl_count(opt,                show_info = show_info))
            print("    White listed",   dnsbl_count(opt, white_list,    show_info = show_info))
        pass
    pass



#
#
# eof
